@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: acconfig.h,v 2.2 1997/11/10 13:10:14 gena Exp $
+/* $Id: acconfig.h,v 2.3 1998/03/23 15:41:57 gena Exp $
  */
 
 #ifndef CONFIG_H
@@ -28,6 +28,9 @@
 /* some systems does not have XReadBitmapFileData */
 #undef	HAVE_XREADBITMAPFILEDATE
 
+/* some systems does not have XSupportsLocale */
+#undef	HAVE_XSUPPORTSLOCALE
+
 /* Where new mail is spooled */
 #undef _CONF_PATH_MAILDIR
 
