@@ -20,7 +20,7 @@
  ***********************************************************************/
 
 /*
- * Copyright(c) 1997 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1997-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -38,7 +38,7 @@
  *
  */
 
-/*       $Id: textbuf.c,v 2.7 1997/12/16 16:44:09 gena Exp $
+/*       $Id: textbuf.c,v 2.12 1998/04/21 13:53:57 gena Exp $
  */
 
 #include <stdio.h>
@@ -86,6 +86,7 @@ tb_init(TextBuf *tb)
     tb->linewrap = 0;
     tb->flags = 0;
     tb->maxchars = 0;
+    tb->attr_def = ATTR_EMPTY;
 
     return;
 } /* tb_init() */
@@ -103,7 +104,10 @@ tb_clear(TextBuf *tb)
     tl = tb->firstline;
     while (tl) {
 	tlold = tl;
-	free(tl->buf);
+	if (tl->buf)
+		free(tl->buf);
+	if (tl->attr)
+		free(tl->attr);
 	tl = tl->next;
 	free(tlold);
     }
@@ -226,6 +230,81 @@ tb_get_line_by_num(TextBuf *tb, char **line, int n)
    }
 }
 
+void 
+tb_set_block_attr(TextBuf *tb, int r0, int c0, int r1, int c1, int attr)
+/*
+ * Desc: set attrbutes of specific block
+ */	 
+{
+int i, k, rtop, rbottom;
+struct TextLine *tl;
+
+    if (r0 > r1)  {
+     rtop = r0;
+     rbottom = r1; }
+    else	{
+     rtop = r1;
+     rbottom = r0;
+		}
+
+    if (rtop == rbottom)	{
+     if ((tl = tb_get_lineptr_by_num(tb, rtop)) == NULL)
+	return;
+
+     if (c0 < 0)
+	c0 = tl->strlen;
+     if (c1 < 0)
+	c1 = tl->strlen;
+
+     if (c1 < c0)	{
+	i = c0;
+	c0 = c1;
+	c1 = i;		}
+
+     if (c1 > tl->strlen)
+	c1 = tl->strlen;
+
+     if (c0 > tl->strlen)
+	 c0 = tl->strlen;
+
+     if (c0 == c1)
+	return;
+
+     for (i = c0; i < c1; i++)
+	tl->attr[i] = attr;
+
+     tl->flags |= TLINE_MODIFIED;
+
+     return;			}
+
+    for (i = rbottom; i <= rtop; i++)	{
+     if ((tl = tb_get_lineptr_by_num(tb, i)) == NULL)
+	continue;
+
+     if (i == rbottom)	{
+        if (c0 >= tl->strlen)
+		continue;
+	for (k = c0; k < tl->strlen; k++)
+		tl->attr[k] = attr;
+			}
+     else
+     if (i == rtop)	{
+	if ((c1 >= tl->strlen) || (c1 < 0))
+		c1 = tl->strlen;
+	for (k = 0; k < c1; k++)
+		tl->attr[k] = attr;
+			}
+     else	{
+	for (k = 0; k < tl->strlen; k++)
+		tl->attr[k] = attr;
+		}
+
+     tl->flags |= TLINE_MODIFIED;
+					}
+
+    return;
+}
+
 void 
 tb_get_block(TextBuf *tb, int r0, int c0, int r1, int c1, char **buf)
 /*
@@ -238,7 +317,7 @@ char *block, *line;
 
     if (r0 > r1)  {
      rtop = r0;
-     rbottom = 1; }
+     rbottom = r1; }
     else	{
      rtop = r1;
      rbottom = r0;
@@ -350,7 +429,8 @@ tb_insert_char(TextBuf *tb, int n, char ch)
 /*
  * Desc: insert the character <ch> into <tb> at the n-th position 
  *	 in the current line.
- * returns -1 if line was wrapped or number of characters added
+ * returns number of characters added or minus this number if the line
+ * was wrapped
  */
 {
     TextLine *tl;
@@ -378,16 +458,25 @@ tb_insert_char(TextBuf *tb, int n, char ch)
 	    return 0;
 	}
 	tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, tl->buflen+TB_BUFINC);
+	if (!newbuf) {
+	    tb_error("tb_insert_char(): Could not realloc attr, character not inserted");
+	    return 0;
+	}
+	tl->attr = newbuf;
+
 	tl->buflen = tl->buflen + TB_BUFINC;
     }
-    /* tl->strlen+1 < tl->buflen !!! */
 
     /* first move the charachter after n one position */
     for (i=tl->strlen; i>=n; i--) {
 	tl->buf[i+1] = tl->buf[i];
+	tl->attr[i+1] = tl->attr[i];
     }
     /* now insert the character */
     tl->buf[n] = ch;
+    tl->attr[n] = tb->attr_def;
     
     /* adjust strlen */
     tl->strlen++;
@@ -400,7 +489,7 @@ tb_insert_char(TextBuf *tb, int n, char ch)
     i = tb_reformat(tb);
     tb_fix_line(tl);
 
-    return i ? -1 : added;
+    return i ? -1*added : added;
 
 } /* tb_insert_char() */
 
@@ -411,6 +500,7 @@ tb_insert_line(TextBuf *tb, char *line)
  */
 {
     TextLine *tlnew;
+    int i;
 
     /* Allocate a new TextLine buffer */
     tlnew = (TextLine *) malloc( sizeof(TextLine) );
@@ -419,19 +509,35 @@ tb_insert_line(TextBuf *tb, char *line)
 	return;
     }
     /* allocate a new line buffer and copy the line into it */
-    tlnew->buf = (char *) malloc( strlen(line) + 1 );
+    tlnew->buf = (char *) malloc(strlen(line) + 1);
     if (!tlnew->buf) {
 	tb_error("tb_insert_line(): could not malloc line buffer");
 	free(tlnew);
 	return;
     }
     strcpy(tlnew->buf, line);
-    tlnew->buflen = strlen(line)+1;
     tlnew->strlen = strlen(line);
+
+    tlnew->attr = (char *) malloc(tlnew->strlen + 1);
+    if (!tlnew->attr) {
+	tb_error("tb_insert_line(): could not malloc line attr buffer");
+	free(tlnew->buf);
+	free(tlnew);
+	return;
+    }
+    for (i = 0; i < tlnew->strlen; i++)
+	tlnew->attr[i] = tb->attr_def;
+    tlnew->attr[tlnew->strlen] = '\0';
+
+    tlnew->buflen = strlen(line)+1;
     tlnew->bgcolor = tb->bgcolor_def;
     tlnew->fgcolor = tb->fgcolor_def;
     tlnew->cont = NULL;
     tlnew->flags = TLINE_MODIFIED;
+    tlnew->clk_callback = NULL;
+    tlnew->dbl_callback = NULL;
+    tlnew->cur_callback = NULL;
+    tlnew->callback_data = 0;
 
     /* check if there are any lines */
     if (tb->currentline) {
@@ -475,7 +581,7 @@ tb_insert_block(TextBuf *tb, int r, int c, char *buf)
  */
 {
     TextLine *tl, *tlold;
-    int len, oneline, oldwrap;
+    int i, len, oneline, oldwrap;
     char *newbuf, *p;
 
     /* store the current line */
@@ -529,6 +635,15 @@ tb_insert_block(TextBuf *tb, int r, int c, char *buf)
 	    return;
 	}
 	tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, tl->buflen+len+1);
+	if (!newbuf) {
+	    tb_error("tb_insert_buf(): Could not realloc attr, character not inserted");
+	    tb->currentline = tlold;
+	    return;
+	}
+	tl->attr = newbuf;
+
 	tl->buflen = tl->buflen + len + 1;
     				      }
     /* store part of the string after <c> */
@@ -539,8 +654,17 @@ tb_insert_block(TextBuf *tb, int r, int c, char *buf)
     /* add remaining part of the string */
     strcat(tl->buf, newbuf);
     tl->strlen += len;
-    tb_handle_tabs(tb);
     free(newbuf);
+
+    newbuf = strdup(tl->attr + c);
+    tl->attr[c] = '\0';
+    for (i = c; i < len + c; i++)
+	tl->attr[i] = tb->attr_def;
+    tl->attr[len + c] = '\0';
+    strcat(tl->attr, newbuf);
+    free(newbuf);
+
+    tb_handle_tabs(tb);
     if (oneline) {
      /* wrap the line if needed */
      tb_reformat(tb);
@@ -584,8 +708,17 @@ tb_insert_block(TextBuf *tb, int r, int c, char *buf)
 	    tb->currentline = tlold;
 	    return;
 			}
-     tl->buf = newbuf;
-     tl->buflen = tl->buflen + len + 1;
+        tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, tl->buflen+len+1);
+	if (!newbuf) 	{
+	    tb_error("tb_insert_buf(): Could not realloc attr, character not inserted");
+	    tb->currentline = tlold;
+	    return;
+			}
+        tl->attr = newbuf;
+
+        tl->buflen = tl->buflen + len + 1;
     				      }
      tl->strlen += len;
      /* store part of the string after <c> */
@@ -595,6 +728,14 @@ tb_insert_block(TextBuf *tb, int r, int c, char *buf)
      /* add remaining part of the string */
      strcat(tl->buf, newbuf);
      free(newbuf);
+
+     newbuf = strdup(tl->attr);
+     for (i = 0; i < len; i++)
+	tl->attr[i] = tb->attr_def;
+     tl->attr[len] = '\0';
+     strcat(tl->attr, newbuf);
+     free(newbuf);
+
      tb_handle_tabs(tb);
 			}
 
@@ -630,43 +771,45 @@ tb_insert_cr(TextBuf *tb, int c)
  *	  and move the remainder of the line one line down.
  */
 {
-    char *line;
     char *newline;
     TextLine *tl;
 
-    /* get the current line */
-    tb_get_line(tb, &line);
-    if (!line) {
+    if (!tb->currentline) {
 	/* we must have a virgin textbuffer!, just append two lines */
 	tb_append_line(tb, "");
 	tb_append_line(tb, "");
-	return;
-    }
+	return;		  }
 
     /* check to see if c is valid */
-    if (c<0 || c>tb->currentline->strlen) {
+    if ((c < 0) || (c > tb->currentline->strlen))
 	return;
-    }
     
-    if (c<tb->currentline->strlen) {
+    if (c < tb->currentline->strlen) {
 	/* get the line from the c-th position */
-	newline = strdup(line+c);
+	newline = strdup(tb->currentline->buf + c);
 	if (!newline) {
 	    tb_error("tb_insert_newline(): Could not allocate space for newline");
-	    return;
-	}
-	line[c] = '\0';
+	    return;   }
+	tb->currentline->buf[c] = '\0';
 	tb->currentline->strlen = c;
 	tb->currentline->flags |= TLINE_MODIFIED;
-	if (tb->currentline == tb->lastline) {
-	    /* append a line */
+	if (tb->currentline == tb->lastline)
 	    tb_append_line(tb, newline);
-	} else {
+	else {
 	    /* insert a line at the next line */
 	    tb_set_next_line(tb);
 	    tb_insert_line(tb, newline);
 	    tb_set_prev_line(tb);
-	}
+	     }
+	free(newline);
+
+	newline = strdup(tb->currentline->attr + c);
+	if (!newline) {
+	    tb_error("tb_insert_newline(): Could not allocate space for newline attr");
+	    return;   }
+	tb->currentline->attr[c] = '\0';
+	if (tb->currentline->next)
+		strcpy(tb->currentline->next->attr, newline);
 	free(newline);
     } else {
 	/* we are at the end of the line, c == tb->currentline->strlen */
@@ -703,6 +846,7 @@ tb_append_line(TextBuf *tb, char *line)
  */
 {
     TextLine *tlnew, *tlcur;
+    int i;
 
     /* allocate a new TextLine */
     tlnew = (TextLine *)malloc(sizeof(TextLine));
@@ -715,12 +859,27 @@ tb_append_line(TextBuf *tb, char *line)
 	tb_error("tb_append_line(): Could not allocate line buffer");
 	free(tlnew);
 	return;      }
+
+    tlnew->attr = strdup(line ? line : "");
+    if (!tlnew->attr) {
+	tb_error("tb_append_line(): Could not allocate line attr buffer");
+	free(tlnew->buf);
+	free(tlnew);
+	return;      }
+
     tlnew->strlen = strlen(tlnew->buf);
+    for (i = 0; i < tlnew->strlen; i++)
+	tlnew->attr[i] = tb->attr_def;
+
     tlnew->buflen = tlnew->strlen + 1;
     tlnew->bgcolor = tb->bgcolor_def;
     tlnew->fgcolor = tb->fgcolor_def;
     tlnew->cont = NULL;
     tlnew->flags = TLINE_MODIFIED;
+    tlnew->clk_callback = NULL;
+    tlnew->dbl_callback = NULL;
+    tlnew->cur_callback = NULL;
+    tlnew->callback_data = 0;
     tb_fix_line(tlnew);
 
     /* check if there are any lines at all yet */
@@ -759,26 +918,42 @@ tb_append_buf(TextBuf *tb, char *buf, u_long len)
  */
 {
     TextLine *tlnew, *tlcur;
+    int i;
 
     /* allocate a new TextLine */
     tlnew = (TextLine *) malloc( sizeof(TextLine) );
     if (!tlnew) {
-	tb_error("tb_append_line(): Could not allocate TextLine");
+	tb_error("tb_append_buf(): Could not allocate TextLine");
 	return; }
     /* Allocate the buffer in TextLine and copy the line into it */
-    tlnew->buf = (char *) malloc(len + 1);
+    tlnew->buf = (char *)malloc(len + 1);
     if (!tlnew->buf) {
-	tb_error("tb_append_line(): Could not allocate line buffer");
+	tb_error("tb_append_buf(): Could not allocate line buffer");
 	free(tlnew);
 	return;      }
+    tlnew->attr = (char *)malloc(len + 1);
+    if (!tlnew->attr) {
+	tb_error("tb_append_buf(): Could not allocate line attr buffer");
+	free(tlnew->buf);
+	free(tlnew);
+	return;      }
+
+    for (i = 0; i < len; i++)
+	tlnew->attr[i] = tb->attr_def;
+
     strncpy(tlnew->buf, buf, len);
     tlnew->buf[len] = '\0';
+    tlnew->attr[len] = '\0';
     tlnew->buflen = len+1;
     tlnew->strlen = len;
     tlnew->bgcolor = tb->bgcolor_def;
     tlnew->fgcolor = tb->fgcolor_def;
     tlnew->cont = NULL;
     tlnew->flags = TLINE_MODIFIED;
+    tlnew->clk_callback = NULL;
+    tlnew->dbl_callback = NULL;
+    tlnew->cur_callback = NULL;
+    tlnew->callback_data = 0;
     tb_fix_line(tlnew);
 
     /* check if there are any lines at all yet */
@@ -817,16 +992,15 @@ tb_append_to_line(TextBuf *tb, char *line)
 {
 TextLine *tl, *tlcur;
 char *newbuf;
-int incr;
+int incr, i;
 
   if (!tb->currentline)
 	return;
 
   tl = tb->currentline;
   tl->flags |= TLINE_MODIFIED;
-  if (tl->strlen + strlen(line) +1 >= tl->buflen) {
-	/* reallocate the buffer space and add needed amount of space 
-	   to it */
+  if (tl->strlen + strlen(line) + 1 >= tl->buflen) {
+	/* reallocate the buffer space and add needed amount of space to it */
 	incr = (strlen(line) + 1 > TB_BUFINC) ? (strlen(line) + 1) : TB_BUFINC;
 	newbuf = realloc(tl->buf, tl->buflen + incr);
 	if (!newbuf) {
@@ -834,11 +1008,23 @@ int incr;
 	    return;
 	}
 	tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, tl->buflen + incr);
+	if (!newbuf) {
+	    tb_error("tb_append_to_line(): Could not realloc attr, buffer not appended");
+	    return;
+	}
+	tl->attr = newbuf;
+
 	tl->buflen += incr;
     						  }
 
+  for (i = tl->strlen; i < tl->strlen + strlen(line); i++)
+	tl->attr[i] = '\0';
+
   strcat(tl->buf, line);
   tl->strlen = strlen(tl->buf);
+  tl->attr[tl->strlen] = '\0';
 
   tlcur = tb->currentline;
   tb_fix_line(tl);
@@ -855,42 +1041,46 @@ tb_wrap_line(TextBuf *tb)
  * Desc: wrap current line if needed
  */
 {
-char *p, *nextline, *newbuf, c;
+char *p, *nextline, *nextattr, *newbuf, c;
 TextLine *tlwrap;
-int nlen, i;
+int nlen, i, linewrap = tb->linewrap;
 
     tlwrap = tb->currentline;
-    if (!tlwrap || (tb->linewrap == 0))	{ /* is wrapping on? */
+    if (!tlwrap || (linewrap == 0))	{ /* is wrapping on? */
 	if (tlwrap->strlen > tb->maxchars)
 		tb->maxchars = tlwrap->strlen;
 	return 0;			}
 
     /* check if line should be wrapped */
-    if (tlwrap->strlen <= abs(tb->linewrap))	{
+    if (tlwrap->strlen <= abs(linewrap))	{
 	if (tlwrap->strlen > tb->maxchars)
 		tb->maxchars = tlwrap->strlen;
 	return 0;				}
 
-    tlwrap->strlen = abs(tb->linewrap);
-    nextline  = tlwrap->buf + tlwrap->strlen;
+    tlwrap->strlen = abs(linewrap);
+    nextline = tlwrap->buf + tlwrap->strlen;
 
-    if (tb->linewrap < 0) {	/* word wrap */
+    if (linewrap < 0)	{	/* word wrap */
 	c = *nextline;
 	*nextline = '\0';
-	if ((p = strrchr(tlwrap->buf, ' ')) != NULL) {
+	if ((p = strrchr(tlwrap->buf, ' ')) != NULL)	{
 	 *nextline = c;
-	 tlwrap->strlen = p - tlwrap->buf;
-	 nextline = ++p;		             }
-	else
+	tlwrap->strlen = p - tlwrap->buf;
+	nextline = ++p;					}
+	else	{
+	 /* can not word wrap - wrap as usual */
 	 *nextline = c;
-	  		  }
+	 linewrap = abs(linewrap);
+		}
+	  		}
     nlen = strlen(nextline);
+    nextattr  = tlwrap->attr + tlwrap->strlen;
 
     /* if the line was already wrapped then add the rest */
     /* at the beginning of the next line */
     if (tlwrap->cont && (tlwrap->cont == tlwrap->next)) {
     /* if it's a word wrapping don't forget about the extra space */
-    if (tb->linewrap < 0)
+    if (linewrap < 0)
 	nlen++;
      tlwrap->flags |= TLINE_MODIFIED;
     /* check if the nextline can be fitted in the buffer */
@@ -903,16 +1093,28 @@ int nlen, i;
 	    return 0;
 	}
 	tlwrap->next->buf = newbuf;
+	newbuf = realloc(tlwrap->next->attr, tlwrap->next->buflen+nlen+1);
+	if (!newbuf) {
+	    tb_error("tb_wrap_line(): Could not realloc attr, character not inserted");
+	    return 0;
+	}
+	tlwrap->next->attr = newbuf;
+
 	tlwrap->next->buflen = tlwrap->next->buflen + nlen + 1;
     								}
+
      /* shift the line <nlen> places right */
-     for (i=tlwrap->next->strlen; i >= 0; i--)
+     for (i=tlwrap->next->strlen; i >= 0; i--)	{
 	tlwrap->next->buf[i+nlen] = tlwrap->next->buf[i];
+	tlwrap->next->attr[i+nlen] = tlwrap->next->attr[i];
+						}
      /* insert the newline */
-     for (i = 0; i < nlen; i++)
+     for (i = 0; i < nlen; i++)	{
 	tlwrap->next->buf[i] = nextline[i];
+	tlwrap->next->attr[i] = nextattr[i];
+				}
      /* add extra space */
-     if (tb->linewrap < 0)
+     if (linewrap < 0)
 	tlwrap->next->buf[nlen-1] = ' ';
      /* adjust the string length */
      tlwrap->next->strlen += nlen;
@@ -938,6 +1140,7 @@ int nlen, i;
 
     /* terminate the wrapped line */
     tlwrap->buf[tlwrap->strlen] = '\0';
+    tlwrap->attr[tlwrap->strlen] = '\0';
 
     if (tlwrap->strlen > tb->maxchars)
 	tb->maxchars = tlwrap->strlen;
@@ -1105,29 +1308,27 @@ tb_del_char(TextBuf *tb, int n)
 {
     int i, l;
     char *line, *newbuf;
-    char *nline;
+    char *nline, *nattr;
     TextLine *tl;
 
-    if (n > tb_get_linelen(tb)) {
+    if (n > tb_get_linelen(tb))
 	return(FALSE);
-    }
 
     /* get the current line */
     tb_get_line(tb, &line);
-    if (!line) {
+    if (!line)
 	return(FALSE);
-    }
 
     tb->bufchanged = TRUE;
     if (n == tb_get_linelen(tb)) {
 	/* we are at eol, join the current line with the next one */
-	if (tb_set_next_line(tb)) {
+	if (tb_set_next_line(tb))	{
 	    tb_get_line(tb, &nline);
-	    tb_set_prev_line(tb);
-	} else {
-	    /* there is no next line at all */
-	    return(FALSE);
-	}
+	    nattr = tb->currentline->attr;
+	    tb_set_prev_line(tb);	}
+	else
+	    return FALSE;
+
 	l = strlen(nline);
 	tl = tb->currentline;
 	tl->flags |= TLINE_MODIFIED;
@@ -1143,9 +1344,17 @@ tb_del_char(TextBuf *tb, int n)
 	    return(FALSE);
 	}
 	tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, l);
+	if (!newbuf) {
+	    tb_error("tb_del_char(): Could not realloc attr, character not deleted");
+	    return(FALSE);
+	}
+	tl->attr = newbuf;
 	tl->buflen = l;
 					  }
 	strcat(tl->buf, nline);
+	strcat(tl->attr, nattr);
 	tl->strlen = strlen(tl->buf);
 
 	/* delete the next line */
@@ -1164,8 +1373,9 @@ tb_del_char(TextBuf *tb, int n)
     /* check if the line has any characters in it */
     if (tb->currentline->strlen > 0) {
 	tb->currentline->flags |= TLINE_MODIFIED;
-	for (i=n; i<tb->currentline->strlen; i++) {
-	    line[i] = line[i+1];
+	for (i = n; i < tb->currentline->strlen; i++) {
+	    tb->currentline->buf[i] = tb->currentline->buf[i + 1];
+	    tb->currentline->attr[i] = tb->currentline->attr[i + 1];
 	}
 	/* adjust strlen */
 	tb->currentline->strlen--;
@@ -1224,6 +1434,7 @@ tb_del_line(TextBuf *tb)
 	}
     }
     free(tl->buf);
+    free(tl->attr);
     free(tl);
 
     tl = tb->currentline;
@@ -1277,8 +1488,10 @@ int i, k, rtop, rbottom, sellen;
 								}
 
 	sellen = c1 - c0;
-	for (i = c1; i <= tb->currentline->strlen; i++)
+	for (i = c1; i <= tb->currentline->strlen; i++)	{
 		tb->currentline->buf[i - sellen] = tb->currentline->buf[i];
+		tb->currentline->attr[i - sellen] = tb->currentline->attr[i];
+							}
 
 	tb->currentline->strlen -= sellen;
 	tb->currentline->flags |= TLINE_MODIFIED;
@@ -1299,8 +1512,10 @@ int i, k, rtop, rbottom, sellen;
 			break;
 								}
 	   else	{
-		for (k = c1; k <= tb->currentline->strlen; k++)
+		for (k = c1; k <= tb->currentline->strlen; k++)	{
 			tb->currentline->buf[k - c1] = tb->currentline->buf[k];
+			tb->currentline->attr[k - c1] = tb->currentline->attr[k];
+								}
 		tb->currentline->strlen -= c1;
 		tb->currentline->flags |= TLINE_MODIFIED;
 		}
@@ -1316,6 +1531,7 @@ int i, k, rtop, rbottom, sellen;
 			}
 	   else	{
 		tb->currentline->buf[c0] = '\0';
+		tb->currentline->attr[c0] = '\0';
 		tb->currentline->strlen = c0;
 		tb->currentline->flags |= TLINE_MODIFIED;
 		}
@@ -1343,10 +1559,8 @@ tb_handle_tabs(TextBuf *tb)
     char *p, *p1, *newbuf;
     int pos, spaces, i, moved;
 
-    if (!tl) {
-	/* no lines to handle */
+    if (!tl)
 	return 0;
-    }
 
     moved = 0;
     /* scan line for TABs */
@@ -1377,16 +1591,26 @@ tb_handle_tabs(TextBuf *tb)
 	   return moved;
 	}
 	tl->buf = newbuf;
+
+	newbuf = realloc(tl->attr, tl->buflen+TB_BUFINC);
+	if (!newbuf) {
+	   tb_error("tb_insert_char(): Could not realloc attr, spaces not inserted");
+	   return moved;
+	}
+	tl->attr = newbuf;
+
 	tl->buflen = tl->buflen + TB_BUFINC;
     }
     /* tl->strlen+spaces < tl->buflen !!! */
 
     /* first move the characters after pos spaces position */
-    for (i=tl->strlen; i>pos; i--)
+    for (i=tl->strlen; i>pos; i--)	{
 	tl->buf[i+spaces] = tl->buf[i];
+	tl->attr[i+spaces] = tl->attr[i]; }
     /* insert spaces */
-    for (i=pos; i<=pos+spaces;i++)
+    for (i=pos; i<=pos+spaces;i++)	{
 	tl->buf[i] = ' ';
+	tl->attr[i] = tb->attr_def;	}
     /* adjust strlen */
     tl->strlen += spaces;
     /* continue the search */
@@ -1780,11 +2004,10 @@ tb_get_linelen(TextBuf *tb)
  * Desc: return the lenght of the current line
  */
 {
-    if (tb->currentline) {
-	return(tb->currentline->strlen);
-    } else {
+    if (tb->currentline)
+	return tb->currentline->strlen;
+    else
 	return(0);
-    }
 }/* tb_get_linelen() */
 
 void
@@ -1800,6 +2023,7 @@ tb_set_linelen(TextBuf *tb, int len)
 	tb->bufchanged = TRUE;
 	tb->currentline->flags |= TLINE_MODIFIED;
 	tb->currentline->strlen = len;
+	tb->currentline->attr[len] = '\0';
 	tb->currentline->buf[len] = '\0'; }
 }
 
@@ -1964,6 +2188,7 @@ unsigned char c;
 	if (p[1] == '\0')	{
 		*p = '\0';
 		tl->strlen = strlen(tl->buf);
+		tl->attr[tl->strlen] = '\0';
 		return;		}
 	*p = '_';
     break;
@@ -1972,12 +2197,17 @@ unsigned char c;
 	 if ((p[1] == '\n') && (p[2] == '\0'))	{
 		*p = '\0';
 		tl->strlen = strlen(tl->buf);
+		tl->attr[tl->strlen] = '\0';
 		return;				}
 	*p = '_';
     break;
 		}
    p++;
 	   	    }
+
+ tl->attr[tl->strlen] = '\0';
+
+ return;
 }
 
 int
@@ -1995,120 +2225,78 @@ int oflags = tb->flags;
  return oflags;
 }
 
-/***********************************************************************
- *
- * Testing
- *
- ***********************************************************************/
-
-#ifdef TEXT_TEST
-
-void
-print_tb(TextBuf *tb)
-{
-    char *line;
-
-    tb_set_current_line(tb, 0);
-    tb_get_line(tb, &line);
-    printf("%s\n", line);
-    while (tb_set_next_line(tb)) {
-	tb_get_line(tb, &line);
-	printf("%s\n", line);
-    }
-
-    return;
-} /* print_tb() */
-
-int
-main(int argc, char **argv)
-{
-    TextBuf tb;
-    TextLine *tl;
-    char *line;
-    int i;
-
-    if (argc != 2) {
-	printf("usage: %s <fname>\n", argv[0]);
-	return(0);
-    }
-
-    tb_init(&tb);
-    tb_load_file(&tb, argv[1]);
-
-
-    /* now insert 10 characters */
-    tb_set_current_line(&tb, 5);
-    for (i=0; i<10; i++) {
-	tb_insert_char(&tb, i, 'A'+i);
-    }
-    
-
-    /* insert 5 lines */
-    tb_set_current_line(&tb, 8);
-    for (i=0; i<5; i++) {
-	tb_insert_line(&tb, "lijntje a;lskdjf;jk kldkldklsdkl");
-    }
-
-    tb_set_current_line(&tb, 0);
-    do  {
-	tb_get_line(&tb, &line);
-	printf("%s\n", line);
-    } while (tb_set_next_line(&tb));
-
-    tb_save_file(&tb, "kak");
-
-    return(0);
-} /* main() */
-
-#endif
-
-#ifdef not_used
-
-void
-read_line(FILE *f, int *linelen, char **line)
-/* 
- * Desc: read a line from the file f and return a pointer to the
- *	 newly malloc'ed space for it
+textline_callback
+tb_set_callback(TextBuf *tb, textline_callback callback, int cbtype, int data)
+/*
+ * Desc: set callback on current line
+ * Ret: old callback value
  */
 {
-    char *ptr;
-    int crfound, i;
+textline_callback ocb = NULL;
+TextLine *tl;
 
-    /* read a line from the file and look for a newline character
-       if none is present, realloc the line buffer to the double
-       length and try again, proceed in this fashion until a 
-       newline character has been found and thus the line has
-       been read. Check also for eof(), in case there are no 
-       newline characters in the file. */
+  if (tb->currentline == NULL)
+	return NULL;
 
-    if (*line == NULL) {
-	*line = (char *) malloc( *linelen );
-	if (!*line) {
-	    return;
-	}
-    }
-    
-    i = 0; /* append to the line from this point */
-    crfound = FALSE;
-    while (!crfound && !feof(f)) {
-	fgets(*line+i, *linelen-i, f);
-	if (strchr(*line, '\n')) {
-	    crfound = TRUE;
-	} else {
-	    /* realloc the string to the double size */
-	    i = *linelen;
-	    *linelen *= 2;
-	    ptr = (char *) realloc( *line, *linelen );
-	    if (!ptr) {
-		free(*line);
-		*line = NULL;
-		return;
-	    }
-	    *line = ptr;
-	}
-    } /* crfound or feof(), return the line found */
+  tb->currentline->callback_data = data;
 
-    return;
-} /* read_line() */
+  switch (cbtype) {
+   case FL_TEXTLINE_CLK_CALLBACK:
+	ocb = tb->currentline->clk_callback;
+	tb->currentline->clk_callback = callback;
+   break;
+
+   case FL_TEXTLINE_DBL_CALLBACK:
+	ocb = tb->currentline->dbl_callback;
+	tb->currentline->dbl_callback = callback;
+   break;
+
+   case FL_TEXTLINE_CUR_CALLBACK:
+	ocb = tb->currentline->cur_callback;
+	tb->currentline->cur_callback = callback;
+   break;
+
+   default:
+	return NULL;
+   break;
+		  }
+
+  tl = tb->currentline;
+  while (tl->cont && (tl->next == tl->cont))  {
+   tl = tl->cont;
+   switch (cbtype) {
+    case FL_TEXTLINE_CLK_CALLBACK:
+	tl->clk_callback = callback;
+    break;
+
+    case FL_TEXTLINE_DBL_CALLBACK:
+	tl->dbl_callback = callback;
+    break;
+
+    case FL_TEXTLINE_CUR_CALLBACK:
+	tl->cur_callback = callback;
+    break;
+		   }
+						}
+
+  tl = tb->currentline;
+  while (tl->prev && (tl->prev->cont == tl))	{
+   tl = tl->prev;
+   switch (cbtype) {
+    case FL_TEXTLINE_CLK_CALLBACK:
+	tl->clk_callback = callback;
+    break;
+
+    case FL_TEXTLINE_DBL_CALLBACK:
+	tl->dbl_callback = callback;
+    break;
+
+    case FL_TEXTLINE_CUR_CALLBACK:
+	tl->cur_callback = callback;
+    break;
+		   }
+						}
+
+  return ocb;
+}
 
-#endif
