@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996,1997 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: fmail.h,v 2.13 1997/12/26 11:46:58 gena Exp $
+/* $Id: fmail.h,v 2.28 1998/04/21 13:58:35 gena Exp $
  */
 
 #ifndef _FMAIL_H
@@ -39,6 +39,16 @@
 #include <unistd.h>
 #endif
 
+#if STDC_HEADERS || defined(USE_STDARG)
+#if HAVE_STDARG_H
+#include <stdarg.h>
+#endif
+#else
+#if HAVE_VARARGS_H
+#include <varargs.h>
+#endif
+#endif
+
 #if STDC_HEADERS
 #include <string.h>
 #ifndef bzero
@@ -171,16 +181,35 @@ extern int sys_nerr;
 #include <locale.h>
 #endif
 
+/* structures to be defined later */
 struct _mail_msg;
 struct _mail_folder;
 struct _mime_msg;
+struct _retrieve_src;
+struct _spool_src;
+struct _pop_src;
+struct _imap_src;
+
+/* stuff from util.c */
+#ifndef	HAVE_UTIMES
+extern int utimes(const char *, const struct timeval *);
+#endif
 
 extern int     do_move (char *, char *);
 extern int     fastcopy (char *, char *, struct stat *);
+
 #ifndef HAVE_UNSETENV
 extern void    Unsetenv (char *);
 #endif
 
+#ifndef	HAVE_SNPRINTF
+extern int snprintf(char *, size_t, const char *, ...);
+#endif
+
+#ifndef	HAVE_VSNPRINTF
+extern int vsnprintf(char *, size_t, const char *, va_list);
+#endif
+
 #define MAX_FIELD_NAME_LEN	32	/* limit of field name length in mail header */
 #define MAX_FIELDS_IN_HDR	64	/* maximum fields in msg header */
 #define MAX_FIELD_LEN		512	/* limit on length of field */
@@ -197,7 +226,7 @@ extern void    Unsetenv (char *);
 #define MAX_MSG_IN_FOLDER	10240	/* maximal number of messages in folder */
 #define MAX_FOLDERS		1024	/* maximal number of folders */
 #define	MAX_SUBFOLDERS		64	/* maximal number of subfolders */
-#define	MAX_NESTLEVEL		8	/* maximal nesting level for subfold. */
+#define	MAX_NESTLEVEL		16	/* maximal nesting level for subfold. */
 #define MAX_HIDDEN_FOLDERS	8	/* maximal number of hidden folders */
 #define	MAX_MSG_LEN		3072000	/* maximal message length (3Mb) */
 #define	MAX_HEADER_LEN		(MAX_FIELDS_IN_HDR * MAX_FIELD_LEN / 4) /* limit on header length */
@@ -210,11 +239,21 @@ extern void    Unsetenv (char *);
 #define	MAX_CHARSETS		64	/* limit on number of MIME charsets */
 #define MAX_EXTRA_HEADERS	16	/* limit on number of extra header
 						fields */
+#define	MAX_FCC_LEN		255	/* limit on length of Fcc list */
 #define	MAX_FOLDER_ALIAS_NAME	32	/* max. length of folder alias name */
+#define	MAX_HOST		128	/* host name length limit  */
+#define	MAX_PREVIEW_SIZE	65535	/* preview no more than this */
+#define	MAX_PREVIEW_LINES	2048	/* preview no more than this */
+#define	MAX_CHARSET_NAME	24	/* limit on charset name length */
+#define	SOCKET_TIMEOUT		300	/* data transfer timeout */
+#define	RESOLV_TIMEOUT		60	/* DNS timeout */
+
 #define	REPLY_ORGMSG		"XF-OrgMsg" /* pointer to original message */
 					    /* <folder>;<uid;<validity> */
 #define	FWD_ORGMSG		"XF-FwdOrgMsg" /* pointer to original message */
 #define	BOUNCE_ORGMSG		"XF-BncOrgMsg" /* pointer to original message */
+#define	SOURCE_FIELD		"XF-Source" /* where the message was retrieved
+									from */
 
 /************* various formatting options ****************/
 #define NEAT_SPACED_INDENTS	 1 /* put in space after indent if there
@@ -330,7 +369,7 @@ extern struct _mail_folder *mailbox[MAX_FOLDERS];
 /* number of folders */
 extern int folders_num;
 /* pointers to system folders */
-extern struct _mail_folder *trash, *inbox, *outbox, *sentm, *draft, *template, *ftemp, *fimap, *fmbox;
+extern struct _mail_folder *trash, *inbox, *outbox, *sentm, *draft, *template, *ftemp, *fmbox;
 /* path of mail directory ( usually ~/Mail) */
 extern char mailbox_path[255];
 extern char *months[12];
@@ -384,6 +423,7 @@ typedef struct _msg_header {
 	mail_addr *Cc;		/* Cc address list */
 	mail_addr *Bcc;		/* Bcc address list */
 	newsgroup *News;	/* Newsgroups list */
+	char *Fcc;		/* Fcc folder list */
 	char *Subject;		/* subject */
 	time_t snt_time;	/* time when the message was sent */
 	time_t rcv_time;	/* time when the message was received */
@@ -503,6 +543,7 @@ typedef int (*fld_search) (struct _mail_folder *, char *, char *,int, fld_search
 typedef long (*fld_uid) (struct _mail_folder *);
 typedef struct _mail_msg * (*fld_getmsg) (struct _mail_folder *, long uid); /* return message with specified UID */
 typedef int (*fld_refresh) (struct _mail_folder *);	/* refresh folder */
+typedef int (*fld_expunge) (struct _mail_folder *);	/* expunge folder */
 
 typedef struct _mail_folder {
 	char fold_path[255];	/* path of folder directory */
@@ -560,6 +601,9 @@ typedef struct _mail_folder {
 #define	FALIAS	0x00080000   /* sname is an alias */
 #define	FNSCAN	0x00100000   /* don't rescan folder */
 #define	FEXPNG	0x00200000   /* EXPUNGE when deselection/closing (IMAP) */
+#define	FNOMOD	0x00400000   /* don't modify (ask if modify) */
+#define	FTOP	0x00800000   /* top folder of the hierarchy */
+#define	FSUBS	0x01000000   /* subscribed folder */
 	/* folder manipulation functions */
 	fld_name	name;
 	fld_open	open;
@@ -575,6 +619,7 @@ typedef struct _mail_folder {
 	fld_uid		getuid;
 	fld_getmsg	getmsg;
 	fld_refresh	refresh;
+	fld_expunge	expunge;
 	} mail_folder;
 
 /* how to open folder */
@@ -697,7 +742,7 @@ extern int  mmapfd;		/* mmaped file descriptor */
 /* cache stuff */
 
 #define	CACHE_DIR	".cache"	/* directory with cached messages */
-#define	CACHE_MAGIC	0x7f7f0004	/* cache file magic identifier */
+#define	CACHE_MAGIC	0x7f7f0005	/* cache file magic identifier */
 
 /* initialize cache */
 int init_cache();
@@ -851,15 +896,16 @@ typedef struct _mime_msg {
 	char *boundary;			/* Boundary (if multipart) */
 
 	u_int flags;			/* MIME flags */
-#define	PREAMBLE	0x01		/* multipart preamble */
-#define	EPILOG		0x02		/* multipart epilog */
-#define	ATTACHMENT	0x04		/* multipart attachment */
+#define	PREAMBLE	0x0001		/* multipart preamble */
+#define	EPILOG		0x0002		/* multipart epilog */
+#define	ATTACHMENT	0x0004		/* multipart attachment */
 	/* modifiers */
-#define	TEXT_PART	0x08		/* visible part in multipart */
-#define	MSG_BODY	0x10		/* takes all message body */
-#define	FILE_TEMP	0x20		/* attached file should be removed */
-#define	PGP_PART	0x40		/* part encoded with PGP */
-#define	LOCK_PART	0x80		/* part locked by viewer */
+#define	TEXT_PART	0x0008		/* visible part in multipart */
+#define	MSG_BODY	0x0010		/* takes all message body */
+#define	FILE_TEMP	0x0020		/* attached file should be removed */
+#define	PGP_PART	0x0040		/* part encoded with PGP */
+#define	LOCK_PART	0x0080		/* part locked by viewer */
+#define	DELETE_PART	0x0100		/* part to be deleted */
 } mime_msg;
 
 typedef int (*mcap_view) (struct _mail_msg *, struct _mime_msg *);
@@ -979,7 +1025,10 @@ int is_mime(struct _mail_msg *msg);
 int is_mime_msg(struct _mail_msg *msg);
 
 /* returns pointer to mailcap entry with specified type and subtype */
-/* if exact == 0 then wildcard entries will be checked too */
+/* if exact == 0 then check wildcard entries and return new mcap */
+/* if exact == 1 then check for exact match only and return new mcap */
+/* if exact == 2 then check for wildcard entries and return mcap from
+  the mailcap table */
 /* returns NULL if such entry does not exists */
 struct _mime_mailcap *find_mailcap(char *type, char *subtype, int exact);
 
@@ -1001,7 +1050,10 @@ void save_mailcap();
 struct _mime_mailcap *get_mailcap_entry(struct _mail_msg *msg, struct _mime_msg *mime);
 
 /* free memory taken by mime structure */
-void discard_mime (struct _mime_msg *mime);
+void discard_mime(struct _mime_msg *mime);
+
+/* free memory taken by mailcap structure */
+void discard_mcap(struct _mime_mailcap *mcap);
 
 /* searches for field with name str */
 /* returns NULL if search failed */
@@ -1183,6 +1235,9 @@ char *get_mime_fname(struct _mime_msg *mime);
 /* returns 0 if size can not be extracted */
 u_long get_mime_fsize(struct _mime_msg *mime);
 
+/* returns line with full description of supplied mime structure */
+char *get_mime_line(int mnum, struct _mail_msg *msg, struct _mime_msg *mime);
+
 /* attach file to msg. mcap is attachment type, if it's NULL then   */
 /* application/octet-stream is choosed. encoding is encoding type.  */
 /* if -1 then base64 is used. Returns either pointer to new mime    */
@@ -1206,6 +1261,10 @@ char *rem_tr_space(char *str);
 /* warning: str is modified */
 char *rem_tr_spacequotes(char *str);
 
+/* returns next word (either quoted or separated by spaces) from the string */
+/* *str will point to the next word. warning: str is modified */
+char *get_quoted_str(char **str);
+
 /* remove \n (and \r if exists) at the end of string */
 /* returns 1 if newline was stripped and 0 if not    */
 int strip_newline(char *str);
@@ -1270,7 +1329,8 @@ void add_signature_to_msg(struct _mail_msg *msg, char *tfile);
 
 /* adds contents of file to the end of tfile */
 /* offt specfies where in the text file should be inserted */
-void add_file_to_msg(char *tfile, char *file, u_long offt);
+/* if separators is no 0, header and footer lines are added */
+void add_file_to_msg(char *tfile, char *file, u_long offt, int separators);
 
 /* include text of original message in reply */
 /* if type == 0 each line is prefixed */
@@ -1299,6 +1359,9 @@ int uudecode(struct _mail_msg *msg);
 /* returns NULL otherwise */
 struct _head_field *need_read_confirm(struct _mail_msg *msg);
 
+/* delete notification request from message header */
+void delete_read_confirm(struct _mail_msg *msg);
+
 /* send read confirmation message */
 void read_confirm(struct _mail_msg *msg);
 
@@ -1309,9 +1372,9 @@ void read_confirm(struct _mail_msg *msg);
 /* returns NULL on error */
 struct _mail_msg *get_reply_msg(struct _mail_msg *msg , char *rtextbuf, int to_all);
 
-/* generate forward message */
-/* returns NULL on error */
-struct _mail_msg *get_fwd_msg(struct _mail_msg *msg);
+/* generate forward message, fwdtext specifies the text to be forwarded */
+/* (can be NULL). Returns NULL on error. */
+struct _mail_msg *get_fwd_msg(struct _mail_msg *msg, char *fwdtext);
 
 /* generate "vacation" message from msg and vacation file */
 /* returns NULL on error */
@@ -1341,6 +1404,10 @@ char *get_charset_addr_line(char *prefix,struct _mail_addr *addr, int *charset);
 /* parse line in mail header . Always return allocated field structure */
 struct _head_field *get_field(char *str);
 
+/* parse one field in open message file. The field can be split across */
+/* multiple lines. Returns NULL on error.                              */
+struct _head_field *get_folded_field(FILE *ffd);
+
 /* finds field with name str in header of message msg */
 /* returns pointer to found field or NULL if not found */
 struct _head_field *find_field(struct _mail_msg *msg, char *str);
@@ -1471,8 +1538,12 @@ int rescan_folder(struct _mail_folder *folder);
 /* messages was found */
 int refresh_folder(struct _mail_folder *folder);
 
+/* expunge deleted messages in folder */
+/* returns 0 on success and -1 on error */
+int expunge_folder(struct _mail_folder *folder);
+
 /* Close folder. All memory allocated for messages is freed */
-void close_folder (struct _mail_folder *folder);
+void close_folder(struct _mail_folder *folder);
 
 /* deallocated memory taken by message header */
 void discard_message_header(struct _mail_msg *msg);
@@ -1559,7 +1630,7 @@ int send_message(struct _mail_msg *msg);
 int sendmail_send_message(struct _mail_msg *msg);
 
 /* send message msg using POP, return 0 on success */
-int pop_send_message(struct _mail_msg *msg);
+int pop_send_message(struct _pop_src *pop, struct _mail_msg *msg);
 
 /* send message msg using smtp, return 0 on success */
 /* and -1 on error */
@@ -1615,31 +1686,30 @@ void discard_all_folders();
 
 /* incorporate messages from spool file according to rules */
 /* returns number of messages actually incorporated or -1 on error */
-/* if trunc > 0 then the spool file is truncated to 0 */
 /* notify returns number of message which require sound/visual */
 /* notification */
-int mbox_inc_mail(char *spool, int trunc, int *notify);
+int mbox_inc_mail(struct _retrieve_src *source, int *notify);
 
 /* incorporate messages from POP server according to rules */
 /* returns number of messages actually incorporated or -1 on error */
 /* notify returns number of message which require sound/visual */
 /* notification */
-int pop_inc(int *notify);
+int pop_inc(struct _retrieve_src *source, int *notify);
 
 /* retrieve message body from POP server if previosly only header */
 /* was retrieved. Returns -1 on error and 0 on success */
-int pop_getfull_msg(struct _mail_msg *msg);
+int pop_getfull_msg(struct _pop_src *pop, struct _mail_msg *msg);
 
 /* delete message on the server if previously only header was retrieved */
 /* Returns -1 on error and 0 on success */
-int pop_delmsg_by_uidl(struct _mail_msg *msg);
+int pop_delmsg_by_uidl(struct _pop_src *pop, struct _mail_msg *msg);
 
 /* initialize POP server connection (connect and log in) */
 /* Returns -1 on error and 0 on success */
-int pop_init();
+int pop_init(struct _pop_src *pop);
 
 /* close connection to POP server */
-void pop_end();
+void pop_end(struct _pop_src *pop);
 
 /* initialize connection array */
 void init_cinfo();
@@ -1776,13 +1846,6 @@ int ishebrew(u_int c);
 /* converts str according to hebrew rules (right to left) */
 char *hebrew_conv(char *str);
 
-/* lock mailbox path using lock file */
-/* returns -1 on error and 0 on success */
-int lockmbox(char *path);
-
-/* unlock mailbox */
-void unlockmbox();
-
 /* calculate real message len replacing CR with CRLF */
 long calc_msg_len(struct _mail_msg *msg);
 
@@ -1793,42 +1856,45 @@ int count_recipients(struct _mail_msg *msg);
 
 /* connect to IMAP server and initialize all folders */
 /* returns -1 on error and 0 on success */
-int imap_init();
+int imap_init(struct _imap_src *imap);
 
 /* disconnect from IMAP server */
-void imap_close(int bygui);
+void imap_close(struct _imap_src *imap, int bygui);
 
 /* returns 1 if connection to IMAP server exists and 0 otherwise */
-int imap_isconnected();
+int imap_isconnected(struct _imap_src *imap);
+
+/* returns number of established IMAP connections */
+int imap_connected();
 
 /* find IMAP folder by full name */
 /* returns pointer to folder if found and NULL if not */
-struct _mail_folder *find_imap_folder(char *name);
+struct _mail_folder *find_imap_folder(struct _imap_src *imap, char *name);
 
 /* find IMAP folder by name */
 /* returns pointer to folder if found and NULL if not */
-struct _mail_folder *find_imap_folder_by_name(char *name);
+struct _mail_folder *find_imap_folder_by_name(struct _imap_src *imap, char *name);
 
 /* return path of IMAP folder */
 /* returns NULL if path does not exists */
-char *get_imap_folder_path(struct _mail_folder *folder);
+char *get_imap_folder_path(struct _imap_src *imap, struct _mail_folder *folder);
 
 /* returns short name of IMAP folder */
-char *get_imap_folder_short_name(struct _mail_folder *folder);
+char *get_imap_folder_short_name(struct _imap_src *imap, struct _mail_folder *folder);
 
 /* mark folder as IMAP */
-void imap_folder(struct _mail_folder *folder);
+void imap_folder(struct _imap_src *imap, struct _mail_folder *folder);
 
 /* mark message as IMAP */
-void imap_message(struct _mail_msg *message);
+void imap_message(struct _imap_src *imap, struct _mail_msg *message);
 
 /* delete all IMAP folders and free all memory used by them and thei messages */
 /* all locked messages are being relocated to FTEMP */
-void discard_imap_folders();
+void discard_imap_folders(struct _imap_src *imap);
 
 /* create new IMAP folder with given name */
 /* returns pointer to new folder on success and NULL on error */
-struct _mail_folder *create_imap_folder(struct _mail_folder *folder,char *name);
+struct _mail_folder *create_imap_folder(struct _imap_src *imap, struct _mail_folder *folder,char *name);
 
 /* inactivity timer callback */
 void imap_timer_cb();
@@ -1836,10 +1902,22 @@ void imap_timer_cb();
 /* retrieve mail from IMAP */
 /* notify returns number of message which require sound/visual */
 /* notification */
-int imap_inc(int *notify);
+int imap_inc(struct _retrieve_src *source, int *notify);
 
-/* EXPUNGE messages in given folder (if needed) */
+/* EXPUNGE messages in given IMAP folder (if needed) */
 int imap_folder_expunge(struct _mail_folder *folder);
+
+/* save all folders from connected servers */
+void save_imap_folders();
+
+/* close connection to all servers */
+void imap_close_all(int bygui);
+
+/* returns IMAP connection structure for specific account (if connected) */
+/* if name is NULL then any IMAP account with established connection */
+/* is returned */
+struct _imap_src *get_imap_connection(char *name);
+
 /* ------------------------------------- */
 
 /* returns first found unread msg */
@@ -1915,6 +1993,36 @@ int pipe_msg(struct _mail_msg *msg, char *command);
 /* update paths of subfolders if path of parent folder has been changed */
 void update_cfold_path(struct _mail_folder *folder);
 
+/* add folder to Fcc list. Return -1 on error and 0 on success */
+int add_fcc_list(struct _msg_header *msgh, struct _mail_folder *folder);
+
+/* return folder pointed by fptr from Fcc list */
+/* Returns -1 on error and 0 on success */
+int del_fcc_list(struct _mail_msg *msg, char *fptr);
+
+/* returns next folder from Fcc list */
+/* returns NULL if Fcc list is empty or we've reached the end of the list */
+char *scan_fcc_list(struct _mail_msg *msg, char *fptr);
+
+/* parse and append supplied comma-separated Fcc list */
+void parse_fcc_list(struct _msg_header *msgh, char *fcclist);
+
+/* print Fcc list in header format */
+void print_fcc_list(struct _mail_msg *msg, FILE *fcc);
+
+/* copy message to folders according to Fcc list */
+/* returns -1 on error and 0 on success */
+int process_fcc_list(struct _mail_msg *msg);
+
+typedef struct _url_spec {
+ int url_s, url_e;
+ struct _url_spec *next;
+} url_spec;
+
+/* highlight (find) all URLs in supplied string */
+/* returns linked list of found URLs or NULL of none were found */
+struct _url_spec *highlight_urls(char *str);
+
 /* ------------------------------- */
 
 /* ------- MBOX stuff ------------ */
@@ -1924,6 +2032,13 @@ size_t  fsize;          /* latest size */
 char    mode[3];        /* open mode   */
 } mbox_spec;
 
+extern int locking;	/* locking method */
+#define	MBOX_DOT_LOCK		0x01	/* use <spool>.lock */
+#define	MBOX_KERNEL_LOCK	0x02	/* use flock() or lockf() */
+
+#define REALENGTH	"X-Real-Length"	/* store real lenght in this field */
+#define FROMLINE	"X-From-Line"	/* store original From line here */
+
 /* make folder local (MBOX-style) */
 void mbox_folder(struct _mail_folder *folder);
 
@@ -1977,6 +2092,10 @@ int is_parent(struct _mail_folder *folder1, struct _mail_folder *folder2);
 /* returns -1 otherwise */
 int is_tree_parent(struct _mail_folder *folder1, struct _mail_folder *folder2);
 
+/* returns the topmost ancestor of specific folder */
+/* returns NULL if folder has not ancestors */
+struct _mail_folder *get_ancestor(struct _mail_folder *folder);
+
 /* finds two ancestors of folder1 and folder2 which have the same father */
 /* returns -1 on error, 0 if the ancestors were found - in this case */
 /* folder1 and folder2 points to those ancestors , */
@@ -1985,8 +2104,151 @@ int find_ancestors(struct _mail_folder **folder1,struct _mail_folder **folder2);
 
 /* expand (collapse) folder tree, if all > 0, all subtrees will be */
 /* expanded (collapsed) as well. returns -1 on error and 0 on success */
+void expand_tree(struct _mail_folder *folder, int all);
+void collapse_tree(struct _mail_folder *folder, int all);
 int expand_collapse_tree(struct _mail_folder *folder, int all);
 
 /* ------------------------------- */
 
+/* ---- multisource retrieve ---- */
+
+#define	MAX_RETR_SOURCES	16	/* limit no number of sources */
+
+/* retrieve mail */
+typedef int (*get_mail) (struct _retrieve_src *, int *notify);
+/* free spec structure in _retrieve_src */
+typedef void (*free_rsrc) (struct _retrieve_src *);
+/* initialize spec structute in _retrieve_src */
+typedef void (*init_rsrc) (struct _retrieve_src *);
+/* load spec structure from file */
+typedef int (*load_rsrc) (struct _retrieve_src *, FILE *);
+/* save spec strucutre to file */
+typedef int (*save_rsrc) (struct _retrieve_src *, FILE *);
+
+typedef struct _retrieve_src {
+	char name[32];		/* source name */
+	int flags;
+#define	RSRC_DISABLED	0x01	/* source disabled */
+#define	RSRC_MARKREAD	0x02	/* mark all retrieved messages as read */
+#define	RSRC_NONOTIFY	0x04	/* don't notify */
+#define	RSRC_SAVEADDR	0x08	/* save addresses from retrieved messages */
+#define	RSRC_NOTIMER	0x10	/* don't retrieve if invoked by timer */
+	int type;
+#define	RSRC_SPOOL	0x01	/* spool file */
+#define	RSRC_POP	0x02	/* POP server */
+#define	RSRC_IMAP	0x04	/* IMAP server */
+	void *spec;
+	/* functions */
+	get_mail retrieve;
+	free_rsrc free;
+	init_rsrc init;
+	load_rsrc load;
+	save_rsrc save;
+} retrieve_src;
+
+extern struct _retrieve_src retrieve_srcs[];	/* list of sources */
+
+typedef struct _spool_src {
+	struct _retrieve_src *source;	/* source */
+	char path[MAXPATHLEN];
+	int flags;
+#define	SSRC_TRUNCATE	0x01		/* truncate spool after retrieving */
+#define	SSRC_REWRITE	0x02		/* rewrite periodically */
+} spool_src;
+
+#define	MAX_UIDL	71	/* max. length of UID string        */
+#define	MAX_POP_UIDS	3000	/* maxmimal number of UID's in file */
+
+/* cache data structure */
+struct _uidl {
+	char uidlstr[MAX_UIDL];	/* message unique ID */
+	u_int mnum;		/* message number on the server */
+	u_long mlen;		/* message length as reported by LIST */
+	struct _uidl *next;	/* next cache entry */
+	u_int flags;		/* special flags */
+#define	POP_DELETED	0x01	/* message has been deleted in this session */
+#define	POP_LENGTH	0x02	/* message length is stored in mlen */
+	} uidl;
+
+typedef struct _pop_src {
+	struct _retrieve_src *source;	/* source */
+	char hostname[MAX_HOST];	/* host to connect */
+	char service[16];		/* port number or service name */
+	char username[32];		/* username */
+	char password[32];		/* password */
+	long maxmsg;			/* don't retrieve longer messages */
+	int flags;
+#define	PSRC_POP2	0x0001		/* use POP2 */
+#define	PSRC_DELETE	0x0002		/* delete messages from server */
+#define	PSRC_DMARK	0x0004		/* don't mark retrieve */
+#define	PSRC_STOREPWD	0x0008		/* store password */
+#define	PSRC_LOGSESSION	0x0010		/* log session */
+#define	PSRC_GETHEADER	0x0020		/* get only header of big message */
+#define	PSRC_SKIPBIG	0x0040		/* skip big message */
+#define	PSRC_CHECKUID	0x0080		/* check if retrieved by UID */
+#define	PSRC_CHECKSTAT	0x0100		/* check if retrieved by status */
+#define	PSRC_STATHEADER	0x0200		/* get status from header */
+#define	PSRC_STATXLST	0x0400		/* get status from XLST */
+#define	PSRC_APOP	0x0800		/* use APOP */
+	/* run-time variables */
+	int popsock;			/* communication socket */
+	FILE *pop_in;			/* incoming data */
+	FILE *pop_out;			/* outgoing data */
+	struct _uidl *uidlcache;	/* cache of UIDs and lengths */
+	int nouidl;			/* set if UIDL is not supported */
+	int num_msgs;			/* number of messages on server */
+	char *pop_uids[MAX_POP_UIDS];	/* array to store POP UID numbers */
+	int uidfirst;			/* first element in the array */
+	char response[512];		/* server response */
+} pop_src;
+
+typedef struct _imap_src {
+	struct _retrieve_src *source;	/* source */
+	char hostname[MAX_HOST];	/* host to connect */
+	char service[16];		/* port number or service name */
+	char username[32];		/* username */
+	char password[32];		/* password */
+	char list[128];			/* hierarchies to list */
+	int flags;
+#define	ISRC_AUTOCON	0x01		/* autoconnect at startup */
+#define	ISRC_STOREPWD	0x02		/* store password */
+#define	ISRC_LOGSESSION	0x04		/* log session */
+#define	ISRC_TRASH	0x08		/* use trash */
+#define	ISRC_CACHE	0x10		/* cache between sessions */
+#define	ISRC_NOEXPUNGE	0x20		/* no autoexpunge */
+#define	ISRC_RETRIEVE	0x40		/* retrieve mail from this server */
+	/* run-time variables */
+	int imapsock;			/* communication socket */
+	FILE *imap_in;			/* incoming data */
+	FILE *imap_out;			/* outgoing data */
+	int icap;			/* server capabilities */
+	int icauth;			/* server auth capabilities */
+	int istate;			/* server state */
+	char *response;			/* server response */
+	struct _mail_folder *ifold;	/* selected folder */
+	struct _mail_folder *iinbox;	/* INBOX folder */
+	struct _mail_folder *itrash;	/* TRASH folder */
+	struct _mail_folder *svfold;	/* saved folder */
+	struct _mail_folder *fimap;	/* MH storage folder */
+	struct _mail_msg *imsg;		/* current message */
+	u_long *search_res;		/* search results */
+	int uid;			/* UID */
+	int lflags;			/* flags for LIST/LSUB folders */
+	int conninprogress;		/* connect in progress */
+	char *plist;			/* parenthized list */
+	char *elem;			/* element of parenthized list */
+	time_t lastcom;			/* when last command was sent */
+#if defined(SASL) && defined(PROT_STDIO)	/* protected stdio */
+	struct protstream *pout;
+	struct protstream *pin;
+#endif
+} imap_src;
+
+/* mark source as spool, POP or IMAP */
+void spool_source(struct _retrieve_src *src);
+void pop_source(struct _retrieve_src *src);
+void imap_source(struct _retrieve_src *src);
+
+/* ------------------------------- */
+
 #endif /* _FMAIL_H */
