@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1996,1997 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: mbox.c,v 2.16 1997/12/16 16:46:27 gena Exp $
+/* $Id: mbox.c,v 2.28 1998/05/07 08:29:12 gena Exp $
  */
 
 #include <fmail.h>
@@ -37,6 +37,7 @@ int mbox_find_text(struct _mail_folder *folder, char *text, char *where, int fla
 long get_mbox_folder_uid(struct _mail_folder *folder);
 struct _mail_msg *get_mbox_msg_by_uid(struct _mail_folder *folder, long uid);
 int refresh_mbox_folder(struct _mail_folder *folder);
+int expunge_mbox_folder(struct _mail_folder *folder);
 
 int delete_mbox_message(struct _mail_msg *msg);
 int print_mbox_message(struct _mail_msg *msg, FILE *fd, int send);
@@ -60,11 +61,11 @@ int relock_fd(struct _mail_folder *folder);
 int create_mbox_file(struct _mail_folder *folder);
 
 #define	MAXLOCKATTEMPT	5	/* max attempts to aquire a lock */
+int locking = -1;		/* locking method */
 
 #define	READMBOXMODE	"r"
 #define	WRITEMBOXMODE	"r+"
 #define	APPENDMBOXMODE	"a+"
-#define	REALENGTH	"X-Real-Length"	/* store real lenght in this field */
 
 static char *mboxmsg = NULL;	/* mmapped message	*/
 static size_t mboxmsglen = 0;	/* mmapped part length	*/
@@ -87,6 +88,7 @@ struct _mail_folder *folder;
  folder->getuid = get_mbox_folder_uid;
  folder->getmsg = get_mbox_msg_by_uid;
  folder->refresh = refresh_mbox_folder;
+ folder->expunge = expunge_mbox_folder;
  folder->hdelim = '/';
  folder->type = F_MBOX;
 }       
@@ -174,14 +176,20 @@ struct _mbox_spec *spec = (struct _mbox_spec *)folder->spec;
  if (!spec->ffd || !(folder->status & FLOCKED))
 	return 0;
 
+ if (locking == -1)
+	locking = b_getcfg_int(conf_name, "spoolock",
+				MBOX_DOT_LOCK|MBOX_KERNEL_LOCK);
+
+ if (locking & MBOX_KERNEL_LOCK)	{
 #if !defined(HAVE_FLOCK) && defined(HAVE_LOCKF)
   if (lockf(fileno(spec->ffd), F_LOCK, 0) == -1) {
 #else
- if (flock(fileno(spec->ffd), LOCK_EX) == -1)	 {
+  if (flock(fileno(spec->ffd), LOCK_EX) == -1)	 {
 #endif
 	display_msg(MSG_WARN, "lock", "Can not re-lock folder\n");
 	unlockfolder(folder);
 	return -1;				 }
+					}
 
  return 0;
 }
@@ -190,7 +198,7 @@ int
 lockfolder(folder)
 struct _mail_folder *folder;
 {
-FILE *ffd;
+FILE *ffd = NULL;
 char buf[255];
 int fd, statfailed = 0, lattempts = 0, uerr = 0;
 struct stat st;
@@ -203,75 +211,87 @@ time_t now;
 	folder->status |= FLOCKED;
 	return 0;		}
 
+ if (locking == -1)
+	locking = b_getcfg_int(conf_name, "spoolock",
+				MBOX_DOT_LOCK|MBOX_KERNEL_LOCK);
+
+ if (locking & MBOX_KERNEL_LOCK)	{
 #if !defined(HAVE_FLOCK) && defined(HAVE_LOCKF)
- if ((ffd = get_mbox_folder_fd(folder, WRITEMBOXMODE)) == NULL)
+  if ((ffd = get_mbox_folder_fd(folder, WRITEMBOXMODE)) == NULL)
 	return -1;
 
- if (fseek(ffd, 0, SEEK_SET) == -1) {
+  if (fseek(ffd, 0, SEEK_SET) == -1) {
 	display_msg(MSG_WARN, "LOCK", "Can not seek");
 	return -1;		    }
 
- for (;; sleep(1))	{
-  if (lattempts++ > MAXLOCKATTEMPT)	{
+  for (;; sleep(1))	{
+   if (lattempts++ > MAXLOCKATTEMPT)	{
 	display_msg(MSG_WARN, "LOCK", "Can not lock folder (lockf() failed)\n%s", folder->fold_path);
 	break;				}
-  if (lockf(fileno(ffd), F_LOCK, 0) == -1)	{
-   if (errno != EAGAIN)	{
-    init_mbox_spec(folder);
-    if ((ffd = get_mbox_folder_fd(folder, WRITEMBOXMODE)) == NULL)
+   if (lockf(fileno(ffd), F_LOCK, 0) == -1)	{
+    if (errno != EAGAIN){
+     init_mbox_spec(folder);
+     if ((ffd = get_mbox_folder_fd(folder, WRITEMBOXMODE)) == NULL)
 	return -1;
 
-    if (fseek(ffd, 0, SEEK_SET) == -1) {
+     if (fseek(ffd, 0, SEEK_SET) == -1) {
 	display_msg(MSG_WARN, "LOCK", "Can not seek");
 	return -1;		       }
 			}
 
 						}
-  else
+   else
 	break;
 			}
 #else
- if ((ffd = get_mbox_folder_fd(folder, READMBOXMODE)) == NULL)
+  if ((ffd = get_mbox_folder_fd(folder, READMBOXMODE)) == NULL)
 	return -1;
 
- if (flock(fileno(ffd), LOCK_EX) == -1) {
+  if (flock(fileno(ffd), LOCK_EX) == -1) {
         display_msg(MSG_WARN, "LOCK", "Can not lock folder (flock() failed)\n%s", folder->fold_path);
-        return -1;			}
+        return -1;			 }
 #endif
+					}
 
- lattempts = 0;
- sprintf(buf, "%s.lock", folder->fold_path);
- if (stat(buf, &st) < 0)
+ if (locking & MBOX_DOT_LOCK)	{
+  lattempts = 0;
+  snprintf(buf, sizeof(buf), "%s.lock", folder->fold_path);
+  if (stat(buf, &st) < 0)
 	statfailed++;
- for (;; sleep(1))	{
-  if (lattempts++ > MAXLOCKATTEMPT)	{
+  for (;; sleep(1))	{
+   if (lattempts++ > MAXLOCKATTEMPT)	{
 	display_msg(MSG_WARN, "LOCK", "Can not lock\n%s", buf);
 	uerr = 1;
 	break;			}
 
-  fd = open(buf, O_WRONLY|O_EXCL|O_CREAT, 00644);
-  if (fd >= 0) {
+   fd = open(buf, O_WRONLY|O_EXCL|O_CREAT, 00644);
+   if (fd >= 0) {
 	close(fd);
-	break; }
+	break;  }
 
-  if (errno == EACCES)	{
-   if (statfailed == 0)
+   if (errno == EACCES)	{
+    if (!(locking & MBOX_KERNEL_LOCK))	{
+	display_msg(MSG_WARN, "LOCK", "Can not create dotlock");
+	uerr = 1;			}
+
+    if (statfailed == 0)
 	uerr = 1;
-   break;		}
+    break;		}
 
-  if (stat(buf, &st) < 0)	{
+   if (stat(buf, &st) < 0)	{
 	if (statfailed++ > MAXLOCKATTEMPT) {
 		display_msg(MSG_WARN, "LOCK", "Can not stat\n%s", buf);
 		uerr = 1;
 		break;    		   }
 	continue;
 				}
-  statfailed = 0;
-  time(&now);
-  if (now < st.st_ctime + 300)
+   statfailed = 0;
+   time(&now);
+   if (now < st.st_ctime + 300)
 	continue;
-  unlink(buf);
+   unlink(buf);
 			}
+				}
 
  if (!uerr) {
   folder->status |= FLOCKED;
@@ -280,23 +300,25 @@ time_t now;
 #endif
   return 0; }
 
+ if (locking & MBOX_KERNEL_LOCK)	{
 #ifdef HAVE_FLOCK
- if (flock(fileno(ffd), LOCK_UN) == -1) {
+  if (flock(fileno(ffd), LOCK_UN) == -1) {
 	display_msg(MSG_WARN, "UNLOCK", "flock failed");
-	return -1;			}
+	return -1;			 }
 #else
 #ifdef HAVE_LOCKF
- if (fseek(ffd, 0, SEEK_SET) == -1) {
+  if (fseek(ffd, 0, SEEK_SET) == -1) {
 	display_msg(MSG_WARN, "UNLOCK", "Can not seek");
-	return -1;		    }
+	return -1;		     }
 
- if (!(folder->status & FRONLY)) {
+  if (!(folder->status & FRONLY)) {
    if (lockf(fileno(ffd), F_ULOCK, 0) == -1) {
 	display_msg(MSG_WARN, "UNLOCK", "lockf failed");
 	return -1;			     }
-				 }
+				  }
 #endif
 #endif
+					}
 
  return -1;
 }
@@ -316,14 +338,21 @@ struct stat sb;
  if (folder->status & FRONLY)
 	return 0;
 
- sprintf(buf, "%s.lock", folder->fold_path);
+ if (locking == -1)
+	locking = b_getcfg_int(conf_name, "spoolock",
+				MBOX_DOT_LOCK|MBOX_KERNEL_LOCK);
+
+ if (locking & MBOX_DOT_LOCK)	{
+ snprintf(buf, sizeof(buf), "%s.lock", folder->fold_path);
  if (stat(buf, &sb) != -1) {
   if (unlink(buf) == -1)	{
 	display_msg(MSG_WARN, "UNLOCK", "Can not remove lockfile %s", buf);
 	return -1;
 			 	}
 			   }
+				}
 
+ if (locking & MBOX_KERNEL_LOCK)	{
 #if !defined(HAVE_FLOCK) && defined(HAVE_LOCKF)
  if ((ffd = get_mbox_folder_fd(folder, WRITEMBOXMODE)) == NULL)
 	return -1;
@@ -349,6 +378,7 @@ struct stat sb;
 				 }
 #endif
 #endif
+					}
 
 #ifdef	LOCK_DEBUG
   fprintf(stderr, "Folder %s unlocked\n", folder->sname);
@@ -621,6 +651,9 @@ off_t shuid = uid;
 	add_field(msg, "Date", get_arpa_date(msg->header->snt_time));
 	folder->status |= FREWRTE;		}
 
+ strip_newline(buf);
+ replace_field_noload(msg, FROMLINE, buf);
+
  hf = find_field_noload(msg, MIME_C_LENGTH);
  if (hf) {
 	clength = (size_t)atoi(hf->f_line);
@@ -634,7 +667,7 @@ off_t shuid = uid;
 	msg->msg_len = (unsigned long)(spec->fsize - shuid - 1);
         if (msg->msg_len > MAX_MSG_LEN)				{
 		msg->status |= MTOOBIG;
-		sprintf(buf, "%ld", msg->msg_len);
+		snprintf(buf, sizeof(buf), "%ld", msg->msg_len);
 		replace_field_noload(msg, REALENGTH, buf);	}
 	fseek(ffd, 0, SEEK_END);
 	return msg;		}
@@ -646,7 +679,7 @@ off_t shuid = uid;
 	msg->msg_len = (unsigned long)(clength - shuid - 1);
         if (msg->msg_len > MAX_MSG_LEN)				{
 		msg->status |= MTOOBIG;
-		sprintf(buf, "%ld", msg->msg_len);
+		snprintf(buf, sizeof(buf), "%ld", msg->msg_len);
 		replace_field_noload(msg, REALENGTH, buf);	}
 
 	fseek(ffd, clength, SEEK_SET);
@@ -668,7 +701,7 @@ off_t shuid = uid;
  msg->msg_len = ftell(ffd) - shuid - eoh;
  if (msg->msg_len > MAX_MSG_LEN)			{
 	msg->status |= MTOOBIG;
-	sprintf(buf, "%ld", msg->msg_len);
+	snprintf(buf, sizeof(buf), "%ld", msg->msg_len);
 	replace_field_noload(msg, REALENGTH, buf);	}
 
  return msg;
@@ -699,6 +732,7 @@ char buf[255];
 				}
   folder->status &= ~FREWRTE;
 
+  reopen_folder_fd(folder);
   if ((ffd = get_mbox_folder_fd(folder, READMBOXMODE)) == NULL)
 	return -1;
 
@@ -890,13 +924,15 @@ size_t clength;
   update_mbox_fsize(folder);
   folder->num_msg = 0;
   folder->unread_num = 0;
-  if (spec->fsize == 0)
-	return 0;
+  if (spec->fsize == 0)		{
+	init_mbox_spec(folder);
+	return 0;		}
   fseek(ffd, 0, SEEK_SET);
 
   if (!(folder->status & FLOCKED)) {
    if (lockfolder(folder) == -1){
         display_msg(MSG_WARN, "rescan folder", "Can not lock folder\n%s", folder->fold_path);
+	init_mbox_spec(folder);
         return -1;		}
    locked = 1;			  }
 
@@ -929,7 +965,12 @@ size_t clength;
 
   if (locked)
 	unlockfolder(folder);
-  return ferror(ffd) ?  -1 : 0;
+  if (ferror(ffd))	{
+	init_mbox_spec(folder);
+	return -1;	}
+
+  init_mbox_spec(folder);
+  return 0;
 }
 
 void
@@ -948,7 +989,7 @@ char buf[255];
           msg_lock = msg; }
         else 	{
 	  if (msg->num > 0) {
-		sprintf(buf, "%s/%d", fmbox->fold_path, msg->num);
+		snprintf(buf, sizeof(buf), "%s/%d", fmbox->fold_path, msg->num);
 		unlink(buf);}
           discard_message(msg);
 		}
@@ -1223,7 +1264,7 @@ struct stat sb;
 	display_msg(MSG_WARN, "rename folder", "folder with name %s already exists", name);
 	return -1;		  }
 
- sprintf(new_path, "%s/%s", dir_path(folder->fold_path), name);
+ snprintf(new_path, sizeof(new_path), "%s/%s", dir_path(folder->fold_path), name);
  if (stat(new_path, &sb) == 0)	{
     if (!(sb.st_mode & S_IFREG))
         {
@@ -1269,6 +1310,7 @@ struct stat sb;
 int locked = 0;
 off_t hoff, shuid;
 FILE *ffd;
+char buf[255];
  
  if (!msg || !folder || !(folder->type & F_MBOX))
         return -1;
@@ -1319,11 +1361,14 @@ FILE *ffd;
 	return -1;				}
 
  msg->get_header(msg);
- get_from(msg, NULL, ffd);
+ get_from(msg, buf, ffd);
  shuid = ftell(ffd);
  update_clen(msg);
  set_status_by_flags(msg);
+ delete_all_fields(msg, FROMLINE);
  print_message_header(msg, ffd);
+ strip_newline(buf);
+ add_field(msg, FROMLINE, buf);
  hoff = ftell(ffd);
  if (msg->print_body(msg, ffd) == -1) {
 	display_msg(MSG_WARN, "move", "Can not write message");
@@ -1372,6 +1417,10 @@ FILE *ffd;
         folder->messages = msg;
         msg->status &= ~CHANGED;
         update_message_length(msg);
+        discard_mime(msg->mime);
+        msg->mime = NULL;
+        if (msg->msg_body)
+                msg->free_text(msg);
                                 }
  else    
         discard_message(msg);  
@@ -1571,11 +1620,19 @@ struct stat sb;
  return 0;
 }
 
+int
+expunge_mbox_folder(folder)
+struct _mail_folder *folder;
+{
+ return mbox_rewrite(folder);
+}
+
 int
 refresh_mbox_folder(folder)
 struct _mail_folder *folder;
 {
 struct _mbox_spec *spec = (struct _mbox_spec *)folder->spec;
+struct _mail_folder *pfold;
 struct _mail_msg *msg;
 struct stat sb;
 size_t fsize = spec->fsize;
@@ -1624,6 +1681,7 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 		unlockfolder(folder);
 	return 0;	}
 
+ reopen_folder_fd(folder);
  if (spec->ffd && (lseek(fileno(spec->ffd), 0, SEEK_SET) != 0))
 	init_mbox_spec(folder);
  if ((ffd = get_mbox_folder_fd(folder, READMBOXMODE)) == NULL)	{
@@ -1643,7 +1701,10 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 	spec->fsize = sb.st_size;
 	if (locked)
 		unlockfolder(folder);
-	return open_mbox_folder(folder, openflags);			}
+	if (open_mbox_folder(folder, openflags) != 0)
+		return -1;
+	return 1;
+									}
 
  if (fsize == sb.st_size) {
    if (!folder->messages)
@@ -1667,7 +1728,10 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 	display_msg(MSG_LOG, "refresh", "Folder %s has been modified (the size is still the same), rescanning", folder->sname);
 	if (locked)
 		unlockfolder(folder);
-	return open_mbox_folder(folder, openflags);			}
+	if (open_mbox_folder(folder, openflags) != 0)
+		return -1;
+	return 1;
+									}
 
    if (locked)
 	unlockfolder(folder);
@@ -1680,7 +1744,10 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 	spec->fsize = sb.st_size;
 	if (locked)
 		unlockfolder(folder);
-	return open_mbox_folder(folder, openflags);			}
+	if (open_mbox_folder(folder, openflags) != 0)
+		return -1;
+	return 1;
+									}
  else
  if (!fgets(buf, 255, ffd) ||
 	!is_from(buf, NULL, 0))						{
@@ -1689,7 +1756,10 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 	display_msg(MSG_LOG, "refresh", "Folder %s changed size (increased), rescanning", folder->sname);
 	if (locked)
 		unlockfolder(folder);
-	return open_mbox_folder(folder, openflags);			}
+	if (open_mbox_folder(folder, openflags) != 0)
+		return -1;
+	return 1;
+									}
 
  folder->status |= FRESCAN;
  fstat(fileno(ffd), &sb);
@@ -1697,7 +1767,7 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
  fseek(ffd, fsize, SEEK_SET);
  nextofft = fsize;
 
- while (!feof(ffd) && (sb.st_size > 0) &&
+ while ((nextofft < sb.st_size) && (sb.st_size > 0) &&
 	(msg = get_mbox_message(nextofft, folder)) != NULL) {
         nextofft = (long)ftell(ffd);
 	msg_cache_deluid(folder, msg->uid);
@@ -1716,21 +1786,32 @@ fprintf(stderr, "Refreshing folder %s ...\n", folder->sname);
 
 	msg->folder = folder;
 	msg->next = folder->messages;
+	msg->status |= RECENT;
 	folder->messages = msg;
+	folder->status &= ~SORTED; 
+	folder->status |= FRECNT; 
 	if (folder->num_msg >= MAX_MSG_IN_FOLDER)
 		break;
 							      }
 
- if (!feof(ffd) && (sb.st_size > 0))			{
+ if ((nextofft > sb.st_size) && (sb.st_size > 0))	{
 	display_msg(MSG_LOG, "refresh", "Folder %s changed unexpectedly, rescanning", folder->sname);
 	folder->status |= FRESCAN;
 	spec->fsize = sb.st_size;
 	if (locked)
 		unlockfolder(folder);
-	return open_mbox_folder(folder, openflags);	}
+	if (open_mbox_folder(folder, openflags) != 0)
+		return -1;
+	return 1;
+							}
 
  if (locked)
 	unlockfolder(folder);
+
+ if (folder->status & FRECNT)		{
+   for (pfold = folder->pfold; pfold; pfold = pfold->pfold)
+	pfold->status &= ~FMRKTMP;	}
+
  if (ferror(ffd))	{
 	display_msg(MSG_WARN, "refresh folder", "Error reading from folder");
 	init_mbox_spec(folder);
@@ -1796,7 +1877,8 @@ struct _mime_msg *mime;
         hf = msg->header->other_fields;
         while(hf) {
           if (!(send && strip_when_send(hf)) &&
-		strcasecmp(hf->f_name, REALENGTH))
+		strcasecmp(hf->f_name, REALENGTH) &&
+		strcasecmp(hf->f_name, FROMLINE))
                 print_header_field(hf, file, send);
 	  if (!strcasecmp(hf->f_name, "Date"))
 		hasdate = 1;
@@ -1956,6 +2038,8 @@ struct _mime_msg *mime1;
  lofft = (msg->uid % mpage);
  mofft = msg->uid - lofft;
  mlen = ((maxlen > (lofft + msg->msg_len)) || (maxlen <= 0)) ? (lofft + msg->msg_len) : maxlen + lofft;
+ if ((mlen + mofft) > spec->fsize)
+	mlen = spec->fsize - mofft;
 
  if (mbox_changed(msg->folder))
 	refresh_mbox_folder(msg->folder);
@@ -1986,6 +2070,11 @@ struct _mime_msg *mime1;
    else
 	mlen--;				}
 
+ if (locking == -1)
+	locking = b_getcfg_int(conf_name, "spoolock",
+				MBOX_DOT_LOCK|MBOX_KERNEL_LOCK);
+
+ if (locking & MBOX_KERNEL_LOCK)	{
 #if !defined(HAVE_FLOCK) && defined(HAVE_LOCKF)
  if (fseek(ffd, 0, SEEK_SET) == -1) {
 	display_msg(MSG_WARN, "get_message_text", "Can not seek");
@@ -1994,6 +2083,7 @@ struct _mime_msg *mime1;
  if (msg->folder->status & FLOCKED)
 	lockf(fileno(ffd), F_ULOCK, 0);
 #endif
+					}
 
  if ((msg->msg_body = (char *)mmap(NULL, (size_t)mlen, PROT_READ|PROT_WRITE, MAP_PRIVATE,fileno(ffd), (off_t)mofft)) == (char *)-1)
         {
@@ -2011,7 +2101,9 @@ struct _mime_msg *mime1;
  mboxmsglen = mlen;
 
 #ifdef HAVE_MADVISE
+#ifdef	MADV_SEQUENTIAL
  madvise(msg->msg_body, mlen, MADV_SEQUENTIAL);
+#endif
 #endif
 
  msg->msg_body_len = msg->msg_len > maxlen ? maxlen : msg->msg_len;
@@ -2053,6 +2145,11 @@ FILE *ffd;
   msg->msg_body = NULL;
   msg->msg_body_len = 0;
 
+ if (locking == -1)
+	locking = b_getcfg_int(conf_name, "spoolock",
+				MBOX_DOT_LOCK|MBOX_KERNEL_LOCK);
+
+ if (locking & MBOX_KERNEL_LOCK)	{
 #if !defined(HAVE_FLOCK) && defined(HAVE_LOCKF)
  if (msg->folder->status & FLOCKED)	{
     if ((ffd = get_mbox_folder_fd(msg->folder, READMBOXMODE)) != NULL)	{
@@ -2060,6 +2157,7 @@ FILE *ffd;
 	lockf(fileno(ffd), F_LOCK, 0);					}
 					}
 #endif
+					}
 
 #ifdef  MMAP_MSG
   mmsg = NULL;
@@ -2093,7 +2191,7 @@ size_t len;
 	return NULL;
 
  if (msg->num != -1)	{
-  sprintf(buf, "%s/%d", fmbox->fold_path, msg->num);
+  snprintf(buf, sizeof(buf), "%s/%d", fmbox->fold_path, msg->num);
   if (lstat(buf, &sb) == 0)
         return buf;
   msg->num = -1;
@@ -2103,7 +2201,7 @@ size_t len;
         display_msg(MSG_WARN, "MBOX", "No space in %s", fmbox->fold_path);
         return NULL;                       }
 
- sprintf(buf, "%s/%d", fmbox->fold_path, mnum);
+ snprintf(buf, sizeof(buf), "%s/%d", fmbox->fold_path, mnum);
  if ((fd = fopen(buf, "w")) == NULL) {
 	display_msg(MSG_WARN, "MBOX", "Can not create\n%s", buf);
 	return NULL;		     }
@@ -2262,7 +2360,7 @@ get_mbox_message_header(msg)
 struct _mail_msg *msg;
 {
 FILE *ffd;
-char buf[255];
+char buf[255], fromline[255];
 int eoh;
 time_t rt;
 off_t shuid = msg->uid;
@@ -2306,6 +2404,11 @@ struct _head_field *hf;
 
  shuid = ftell(ffd);
 
+ *fromline = '\0';
+ *buf = '\0';
+ if ((hf = find_field_noload(msg, FROMLINE)) != NULL)
+	strcpy(fromline, hf->f_line);
+
  if ((hf = find_field_noload(msg, REALENGTH)) != NULL)
 	strcpy(buf, hf->f_line);
 
@@ -2322,8 +2425,10 @@ struct _head_field *hf;
 	msg->header->snt_time = rt;
  msg->flags |= msg->header->flags;
  msg->status &= ~H_SHORT;
- if (hf)
+ if (*buf)
 	replace_field_noload(msg, REALENGTH, buf);
+ if (*fromline)
+	replace_field_noload(msg, FROMLINE, fromline);
 
  return 0;
 }
@@ -2429,7 +2534,7 @@ struct _mbox_spec *spec;
   if (*name == '/')
 	strcpy(path, name);
   else
-	sprintf(path, "%s/%s", pfolder ? pfolder->fold_path : mailbox_path, name);
+	snprintf(path, sizeof(path), "%s/%s", pfolder ? pfolder->fold_path : mailbox_path, name);
 
   if ((fld = get_mh_folder_by_path(path)) != NULL)
                 return fld;
@@ -2619,6 +2724,16 @@ char *buf;
 FILE *file;
 {
 char abuf[255], *p;
+struct _head_field *hf;
+
+ if ((hf = find_field_noload(msg, FROMLINE)) != NULL)	{
+   if (file)
+	fprintf(file, "%s\n", hf->f_line);
+
+   if (buf)
+	sprintf(buf, "%s\n", hf->f_line);
+
+   return;						}
 
  strcpy(abuf, msg->header->From ? msg->header->From->addr : "unknown");
  while ((p = strchr(abuf, ' ')) != NULL)
@@ -2685,6 +2800,8 @@ off_t mofft, shuid;
 int wnum, onum, oflags, locked = 0;
 struct _head_field *hf;
 unsigned long mlen;
+struct timeval tval[2];
+struct stat sb;
 
  if (!need_rewrite(folder))
 	return 0;
@@ -2699,14 +2816,22 @@ unsigned long mlen;
 
  if (folder->status & FRONLY) 	{
 	folder->status &= ~FREWRTE;
-	return 0;
-				}
+	return 0;		}
+
+ if ((folder->status & FNOMOD) &&
+	!display_msg(MSG_QUEST, "save changes", "Rewrite %s mailbox?", folder->sname))			{
+	folder->status &= ~FREWRTE;
+	return 0;	}
 
  if (!(folder->status & FLOCKED)) {
   if (lockfolder(folder) == -1)
 	return -1;
   locked = 1;			 }
 
+ if (stat(folder->fold_path, &sb) == -1)	{
+	display_msg(MSG_WARN, "rewrite", "can not stat %s", folder->fold_path);
+	return -1;				}
+
  folder->sort = BY_UID|MSG_ASCEND;
  sort_folder(folder);
  folder->sort = osort;
@@ -2751,7 +2876,7 @@ unsigned long mlen;
 						}
 	else
 	if (msg->num > 0) {
-		sprintf(buf, "%s/%d", fmbox->fold_path, msg->num);
+		snprintf(buf, sizeof(buf), "%s/%d", fmbox->fold_path, msg->num);
 		unlink(buf);
 			  }
 
@@ -2778,9 +2903,12 @@ unsigned long mlen;
   update_clen(msg);
   set_status_by_flags(msg);
   mofft = ftell(nfd);
-  get_from(msg, NULL, nfd);
+  get_from(msg, buf, nfd);
   shuid = ftell(nfd);
+  delete_all_fields(msg, FROMLINE);
   print_message_header(msg, nfd);
+  strip_newline(buf);
+  add_field(msg, FROMLINE, buf);
   if (fflush(nfd) == EOF) {
         display_msg(MSG_WARN, "rewrite", "Can not write message");
 	if (locked)
@@ -2831,7 +2959,7 @@ unsigned long mlen;
   msg->msg_len = m_len - shuid;
   msg->header->header_len = h_len - shuid;
   if (msg->num > 0) {
-	sprintf(buf, "%s/%d", fmbox->fold_path, msg->num);
+	snprintf(buf, sizeof(buf), "%s/%d", fmbox->fold_path, msg->num);
 	unlink(buf);}
   msg->num = -1;
   msg->status &= ~CHANGED;
@@ -2861,10 +2989,16 @@ unsigned long mlen;
     fwrite(buf, 1, l, ffd);
  fclose(nfd);
 
+ spec->fsize = ftell(ffd);
  if (ftruncate(fileno(ffd), ftell(ffd)) == -1)
 	display_msg(MSG_WARN, "write", "can not truncate mailbox");
  reopen_folder_fd(folder);
- spec->fsize = ftell(ffd);
+
+ tval[0].tv_sec = sb.st_atime;
+ tval[1].tv_sec = sb.st_mtime;
+ tval[0].tv_usec = tval[1].tv_usec = 0;
+ if (utimes(folder->fold_path, tval) != 0)  
+	display_msg(MSG_LOG, "rewrite", "failed to set modification time on %s", folder->fold_path);
 
  unlink(tmpfile);
  if (locked)
@@ -2894,35 +3028,37 @@ size_t clen = msg->msg_len - msg->header->header_len;
 }
 
 int
-mbox_inc_mail(spool, trunc, notify)
-char *spool;
-int trunc, *notify;
+mbox_inc_mail(source, notify)
+struct _retrieve_src *source;
+int *notify;
 {
 struct _mail_folder *ifold;
 struct _mail_msg *msg;
-char spool_file[255];
+struct _spool_src *spool;
 int num_msgs = 0, num_diff = 0, locked = 0;
 
-  if (!spool || !*spool)
-        sprintf(spool_file, "%s/%s", _PATH_MAILDIR, user_n);
-  else  
-        strcpy(spool_file, spool);
+  if (source->flags & RSRC_DISABLED)
+	return 0;
 
-  if ((ifold = get_mbox_folder_by_path(spool_file)) == NULL) {
-   if ((ifold = create_mbox_folder(NULL, spool_file)) == NULL) 	{
-	display_msg(MSG_WARN, "spool", "Can not access %s", spool_file);
+  spool = (struct _spool_src *)source->spec;
+
+  if ((ifold = get_mbox_folder_by_path(spool->path)) == NULL) {
+   if ((ifold = create_mbox_folder(NULL, spool->path)) == NULL)	{
+	display_msg(MSG_WARN, "spool", "Can not access %s", spool->path);
 	return -1;						}
-   ifold->status |= FRESCAN;
-							     }
+							      }
+  ifold->status |= (FRESCAN|SYSTEM|NOINFR|FNOCLSE);
   ifold->status &= ~FSHORTH;
+  if (!(spool->flags & SSRC_REWRITE))
+	ifold->status |= FNOMOD;
 
   if (get_mbox_folder_fd(ifold, WRITEMBOXMODE) == NULL) {
-	display_msg(MSG_WARN, "spool", "Can not access %s in read-write mode", spool_file);
+	display_msg(MSG_WARN, "spool", "Can not access %s in read-write mode", spool->path);
 	return -1;					}
 
   if (!(ifold->status & FLOCKED)) {
    if (lockfolder(ifold) == -1) {
-	display_msg(MSG_WARN, "spool", "%s is locked, please try again later", spool_file);
+	display_msg(MSG_WARN, "spool", "%s is locked, please try again later", spool->path);
 	return -1;		}
    locked = 1;			  }
 
@@ -2938,7 +3074,6 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 		unlockfolder(ifold);
 	return -1;			}
 	}
-  ifold->status |= FNOCLSE;
 
   msg = ifold->messages;
   while (msg)	   {
@@ -2957,12 +3092,16 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 				}
 
 
-   if ((msg->flags & UNREAD) || trunc ||
+   if ((msg->flags & UNREAD) ||
+	(spool->flags & SSRC_TRUNCATE) ||
 	(msg->status & RECENT)) {
 	set_flags_by_status(msg);
 	convert_fields(msg); 
 	msg->status |= (CHANGED | RECENT);
+	if (source->flags & RSRC_MARKREAD)
+		msg->flags &= ~UNREAD;
 	replace_field(msg, "X-RDate", get_arpa_date(time(NULL)));
+	replace_field(msg, SOURCE_FIELD, source->name);
 	msg->header->rcv_time = time(NULL);     
 
 #ifdef FACES
@@ -2971,7 +3110,8 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 
 	switch(apply_rule(msg, 0)) {
 	  case 0:
-		(*notify)++;
+		if (!(source->flags & RSRC_NONOTIFY))
+			(*notify)++;
 	  break;
           
 	  case -1:
@@ -2981,7 +3121,8 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 	  break;
 				   }
 
-	if (trunc && !(msg->status & MTOOBIG))
+	if ((spool->flags & SSRC_TRUNCATE) &&
+			!(msg->status & MTOOBIG))
 	  msg->status |= (DELETED|DELPERM);
 	else {
 	  msg->folder = ifold;
@@ -3010,12 +3151,14 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 	}
    msg = msg->next; }
 
-  if (trunc || ((num_diff*100/(ifold->num_msg + 1)) > 10))	{
+  if ((spool->flags & SSRC_TRUNCATE) ||
+	(!(ifold->status & FNOMOD) &&
+	((num_diff*100/(ifold->num_msg + 1)) > 10)))	{
    if (mbox_rewrite(ifold) == -1)	{
 	if (locked)
 		unlockfolder(ifold);
 	return -1;			}
-								}
+							}
 
   ifold->status &= ~FRECNT;
   display_msg(MSG_STAT, NULL, "");
@@ -3024,3 +3167,99 @@ int num_msgs = 0, num_diff = 0, locked = 0;
 
   return num_msgs;
 }
+
+void
+free_spool_source(source)
+struct _retrieve_src *source;
+{
+ if (source->spec)	{
+  free(source->spec);
+  source->spec = NULL;	}
+
+ return;
+}
+
+void
+init_spool_source(source)
+struct _retrieve_src *source;
+{
+struct _spool_src *spool;
+
+ if (source->spec == NULL)			{
+  source->spec = (struct _spool_src *)malloc(sizeof(struct _spool_src));
+  spool = (struct _spool_src *)source->spec;
+  spool->source = source;
+  snprintf(spool->path, sizeof(spool->path), "%s/%s", _PATH_MAILDIR, user_n);
+  spool->flags = SSRC_TRUNCATE|SSRC_REWRITE;	}
+
+ return;
+}
+
+int
+load_spool_source(source, fd)
+struct _retrieve_src *source;
+FILE *fd;
+{
+struct _spool_src *spool;
+char buf[MAXPATHLEN + 4];
+struct _mail_folder *ifold;
+
+ spool = (struct _spool_src *)source->spec;
+ if (!fgets(buf, MAXPATHLEN + 3, fd))
+	return -1;
+ if (sscanf(buf, "%d %s", &spool->flags, spool->path) != 2)
+	return -1;
+
+ if ((ifold = get_mbox_folder_by_path(spool->path)) == NULL) {
+   if ((ifold = create_mbox_folder(NULL, spool->path)) == NULL)	{
+	display_msg(MSG_WARN, "spool", "Can not access %s", spool->path);
+	return -1;						}
+							     }
+ ifold->status |= (FRESCAN|SYSTEM|NOINFR|FNOCLSE);
+ ifold->status &= ~FSHORTH;
+ if (!(spool->flags & SSRC_REWRITE))
+	ifold->status |= FNOMOD;
+ else
+	ifold->status &= ~FNOMOD;
+
+ if (ifold->sname) 	    {
+	snprintf(buf, sizeof(buf), "%s (spool)", get_folder_short_name(ifold));
+	free(ifold->sname); }
+ else
+	sprintf(buf, "spool");
+ ifold->sname = strdup(buf);
+ if (!ifold->descr)
+	ifold->descr = strdup("spool mailbox");
+ ifold->open(ifold, FOPEN_NOCACHE);
+
+ return 0;
+}
+
+int
+save_spool_source(source, fd)
+struct _retrieve_src *source;
+FILE *fd;
+{
+struct _spool_src *spool;
+
+ spool = (struct _spool_src *)source->spec;
+ fprintf(fd, "%d %s\n", spool->flags, spool->path);
+
+ return 0;
+}
+
+void
+spool_source(source)
+struct _retrieve_src *source;
+{
+ init_spool_source(source);
+ source->type = RSRC_SPOOL;
+ source->retrieve = mbox_inc_mail;
+ source->free = free_spool_source;
+ source->init = init_spool_source;
+ source->load = load_spool_source;
+ source->save = save_spool_source;
+
+ return;
+}
+
