@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996,1997 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for
  * non-commercial purposes as long as above copyright
@@ -17,14 +17,14 @@
  *
  */
 
-/* $Id: pop.c,v 2.7 1997/12/16 16:46:28 gena Exp $
+/* $Id: pop.c,v 2.13 1998/03/17 16:09:50 gena Exp $
  */
 
 #include <fmail.h>
 #include <umail.h>
+#include "md5.h"
 
-#define	MAX_HOST	128	/* host name length limit  */
-#define	MAX_CONNECTIONS	4	/* connection number limit */
+#define	MAX_CONNECTIONS	20	/* connection number limit */
 #define	CONN_BUF	128	/* connection buffer       */
 
 struct _xf_conn {
@@ -38,40 +38,13 @@ static struct _xf_conn *connections[MAX_CONNECTIONS] = { NULL };
 #define inaddr_copy(hp,sin) \
     memcpy((char *)&((sin)->sin_addr), (hp)->h_addr, (hp)->h_length)
 
-#define	MAX_UIDL	71	/* max. length of UID string */
-
-/* cache data structure */
-
-struct _uidl {
-	char uidlstr[MAX_UIDL];	/* message unique ID */
-	u_int mnum;		/* message number on the server */
-	u_long mlen;		/* message length as reported by LIST */
-	struct _uidl *next;	/* next cache entry */
-	u_int flags;		/* special flags */
-#define	POP_DELETED	0x01	/* message has been deleted in this session */
-#define	POP_LENGTH	0x02	/* message length is stored in mlen */
-	} uidl;
-
-static struct _uidl *uidlcache = NULL;
-static int nouidl = 0;		/* UIDL not supported by server */
-static int num_msgs = -1;	/* number of messages on server */
-
-#define	MAX_POP_UIDS	3000	/* maxmimal number of UID's in file          */
-static char *pop_uids[MAX_POP_UIDS];	/* array to store POP UID numbers */
-static int uidfirst = -2;	/* pointer to first element in the array */
-
-static int popsock = -1;
-static FILE *pop_in = NULL;
-static FILE *pop_out = NULL;
-static char response[512];
-
-struct _uidl *get_popmsg_by_uidl(char *uidl);
-int get_popmsg_num();
-void free_uidlist();
-void save_uidlist();
-void compare_uidlist();
-int check_uidlist(char *uidstr);
-void delete_uidlist(char *uidstr);
+struct _uidl *get_popmsg_by_uidl(struct _pop_src *pop, char *uidl);
+int get_popmsg_num(struct _pop_src *pop);
+void free_uidlist(struct _pop_src *pop);
+void save_uidlist(struct _pop_src *pop);
+void compare_uidlist(struct _pop_src *pop);
+int check_uidlist(struct _pop_src *pop, char *uidstr);
+void delete_uidlist(struct _pop_src *pop, char *uidstr);
 
 void
 init_cinfo()
@@ -223,7 +196,7 @@ int len;
 	if (logging & LOG_NET)
 	 display_msg(MSG_LOG, "async_gethostbyname", "Resolving %s ...", s);
 
-	if (my_check_io_forms(ipcsock, 0) < 0)  {
+	if (my_check_io_forms(ipcsock, 0, RESOLV_TIMEOUT) < 0)  {
 	  close(ipcsock);
 	  display_msg(MSG_STAT, NULL, "");
 	  kill(dnspid, SIGKILL);
@@ -236,13 +209,14 @@ int len;
 		display_msg(MSG_WARN, "async_gethostbyname", "error reading from socket");
 		close(ipcsock);
 	  	display_msg(MSG_STAT, NULL, "");
+		kill(dnspid, SIGKILL);
 		dnspid = 0;
 		return NULL;
 					      }
 
 	iaddr = ntohl(iaddr);
 
-	if ((iaddr == -1) || (my_check_io_forms(ipcsock, 0) < 0))  {
+	if ((iaddr == -1) || (my_check_io_forms(ipcsock, 0, RESOLV_TIMEOUT) < 0))  {
 	  close(ipcsock);
 	  display_msg(MSG_STAT, NULL, "");
 	  kill(dnspid, SIGKILL);
@@ -255,6 +229,7 @@ int len;
 		display_msg(MSG_WARN, "async_gethostbyname", "error reading from socket");
 		close(ipcsock);
 	  	display_msg(MSG_STAT, NULL, "");
+		kill(dnspid, SIGKILL);
 		dnspid = 0;
 		return NULL;
 				    }
@@ -449,7 +424,7 @@ int pport, res;
 
 conwait:
  coninprogress = 1;
- if ((res = my_check_io_forms(sock, 1)) < 0) {
+ if ((res = my_check_io_forms(sock, 1, SOCKET_TIMEOUT)) < 0) {
     if (res == -2)
 	display_msg(MSG_LOG,"connect", "Connect to %s aborted by user", c_host);
     else {
@@ -550,7 +525,7 @@ long rdone, res;
   rdone = res;
 				}
 
- if ((res = my_check_io_forms(fileno(iop), 0)) < 0) {
+ if ((res = my_check_io_forms(fileno(iop), 0, SOCKET_TIMEOUT)) < 0) {
 	lbuf[0] = '\0';
 	return res;				    }
 
@@ -558,7 +533,7 @@ long rdone, res;
      if ((res = read(fileno(iop), p, (int)(n - rdone) > 127 ? 127 : (int)(n - rdone))) == -1) 	{
 	if ((errno == EAGAIN) ||
 		(errno == EWOULDBLOCK)) {
-	  if ((res = my_check_io_forms(fileno(iop), 0)) < 0) 	{
+	  if ((res = my_check_io_forms(fileno(iop), 0, SOCKET_TIMEOUT)) < 0) 	{
 		lbuf[0] = '\0';
 		return res;					}
 
@@ -647,7 +622,7 @@ int len, res, allocbuf, len1;
 	len = 0;
 	}
 
-    if ((res = my_check_io_forms(fileno(iop), 0)) < 0) 		{
+    if ((res = my_check_io_forms(fileno(iop), 0, SOCKET_TIMEOUT)) < 0) 	{
 	lbuf[0] = '\0';
 	if (allocbuf)
 		free(s);
@@ -663,7 +638,7 @@ int len, res, allocbuf, len1;
      if ((res = read(fileno(iop), p, len1)) == -1) {
 	if ((errno == EAGAIN) ||
 		(errno == EWOULDBLOCK)) {
-	  if ((res = my_check_io_forms(fileno(iop), 0)) < 0) 		{
+	  if ((res = my_check_io_forms(fileno(iop), 0, SOCKET_TIMEOUT)) < 0) 		{
 		lbuf[0] = '\0';
 		if (allocbuf)
 			free(s);
@@ -727,7 +702,7 @@ int sent, chunk, res;
 
  if (s) {
 chwrite1:
-    if ((res = my_check_io_forms(fileno(iop), 1)) < 0) 	{
+    if ((res = my_check_io_forms(fileno(iop), 1, SOCKET_TIMEOUT)) < 0) 	{
 	lbuf[0] = '\0';
 	return res;					}
 
@@ -768,7 +743,7 @@ chwrite1:
    else
     lastchar = chunk ? sbuf[chunk - 1] : '\0';
 
-   if ((res = my_check_io_forms(fileno(iop), 1)) < 0) 	{
+   if ((res = my_check_io_forms(fileno(iop), 1, SOCKET_TIMEOUT)) < 0) 	{
 	lbuf[0] = '\0';
 	return res;					}
 
@@ -785,7 +760,7 @@ chwrite1:
       }
 
 chwrite2:
-  if ((res = my_check_io_forms(fileno(iop), 1)) < 0) 	{
+  if ((res = my_check_io_forms(fileno(iop), 1, SOCKET_TIMEOUT)) < 0) 	{
 	lbuf[0] = '\0';
 	return res;					}
 
@@ -817,9 +792,9 @@ int res;
 	return -1;
 			  }
 
-    sprintf(sbuf, "%s\r\n", s);
+    snprintf(sbuf, sizeof(sbuf), "%s\r\n", s);
 chwrite:
-    if ((res = my_check_io_forms(fileno(iop), 1)) < 0)
+    if ((res = my_check_io_forms(fileno(iop), 1, SOCKET_TIMEOUT)) < 0)
 	return res;
 
     if (write(fileno(iop), sbuf, strlen(sbuf)) == -1) {
@@ -835,7 +810,8 @@ chwrite:
 }
 
 char *
-pop_command(fmt, a, b, c, d)
+pop_command(pop, fmt, a, b, c, d)
+struct _pop_src *pop;
 char   *fmt,
        *a,
        *b,
@@ -844,158 +820,180 @@ char   *fmt,
 {
 static char commandln[514];
 
-    (void) sprintf (commandln, fmt, a, b, c, d);
+    snprintf(commandln, sizeof(commandln), fmt, a, b, c, d);
 
-    if (logging & LOG_POP) {
+    if (pop->flags & PSRC_LOGSESSION)	{
      if (strncasecmp(commandln, "pass ", 5))
 	display_msg(MSG_LOG, "pop", "-> %-.127s", commandln);
      else
 	display_msg(MSG_LOG, "pop", "-> PASS *******");
-			   }
+					}
 
-    if (putline(commandln, pop_out) == -1)
+    if (putline(commandln, pop->pop_out) == -1)
         return NULL;
 
-    response[0] = '\0';
+    pop->response[0] = '\0';
 
-    if (getline(response, 513, pop_in) == NULL)
+    if (getline(pop->response, 513, pop->pop_in) == NULL)
 	return NULL;
 
-    if (logging & LOG_POP)
-     display_msg(MSG_LOG, "pop", "<- %-.127s", response);
+    if (pop->flags & PSRC_LOGSESSION)
+     display_msg(MSG_LOG, "pop", "<- %-.127s", pop->response);
 
-    if (response[0] == '+')
-	return response;
+    if (pop->response[0] == '+')
+	return pop->response;
 
-    if (strncasecmp(fmt, "UIDL", 4) && !strncasecmp(response, "-ERR ", 4))
-    	display_msg(MSG_WARN, "pop", "%-.127s", response + 4);
+    if (strncasecmp(fmt, "UIDL", 4) && !strncasecmp(pop->response, "-ERR ", 4))
+    	display_msg(MSG_WARN, "pop", "%-.127s", pop->response + 4);
 
     return NULL;
 }
 
 void
-pop_close()
+pop_close(pop)
+struct _pop_src *pop;
 {
 struct _uidl *ucache;
 
- host_disconnect(&popsock);
+ host_disconnect(&pop->popsock);
 
- while (uidlcache) 	{
-  ucache = uidlcache;
-  uidlcache = uidlcache->next;
-  free(ucache);
-			}
+ while (pop->uidlcache)	{
+  ucache = pop->uidlcache;
+  pop->uidlcache = pop->uidlcache->next;
+  free(ucache);		}
 
- if ((pop_in != NULL) || (pop_out != NULL))
-	fclose(pop_in);
+ if ((pop->pop_in != NULL) || (pop->pop_out != NULL))
+	fclose(pop->pop_in);
 
- pop_in = NULL;
- pop_out = NULL;
- popsock = -1;
+ pop->pop_in = NULL;
+ pop->pop_out = NULL;
+ pop->popsock = -1;
 
  return;
 }
 
 void
-pop_end()
+pop_end(pop)
+struct _pop_src *pop;
 {
-pop_command("QUIT");
-pop_close();
-save_uidlist();
-free_uidlist();
+pop_command(pop, "QUIT");
+pop_close(pop);
+save_uidlist(pop);
+free_uidlist(pop);
 }
 
 int
-pop_init()
+pop_init(pop)
+struct _pop_src *pop;
 {
-char buf[514], user[32];
-char *p;
-int logattempts = 2;
+char buf[514], apoptstamp[512], apopdigest[40], *p, *p1;
+int logattempts = 2, i;
+unsigned char digest[16];
+MD5_CTX ctx;
 
- if (popsock != -1) {
+ if (pop->popsock != -1) {
 	display_msg(MSG_WARN, "pop", "POP busy");
-	return -1;
-		    }
+	return -1;	 }
 
- nouidl = 0;
+ pop->nouidl = 0;
+ pop->popsock = host_connect(pop->hostname, pop->service, NULL);
 
- strcpy(buf, b_getcfg_str(conf_name, "pophost", "localhost"));
-
- popsock = host_connect(buf, b_getcfg_str(conf_name, "poport", "110"), NULL);
-
- if (popsock == -1)
+ if (pop->popsock == -1)
 	return -2;
 
- if ((pop_in = fdopen(popsock, "r+")) == NULL) {
+ if ((pop->pop_in = fdopen(pop->popsock, "r+")) == NULL) {
 	display_msg(MSG_WARN, "pop", "fdopen failed");
-	pop_close();
-	return -1;
-	}
- pop_out = pop_in;
+	pop_close(pop);
+	return -1;					 }
 
+ pop->pop_out = pop->pop_in;
  
- if (getline(buf, 513, pop_in) == NULL) {
-	pop_close();
-	return -1; }
+ if (getline(buf, 513, pop->pop_in) == NULL)	{
+	pop_close(pop);
+	return -1;				}
 
  if (buf[0] != '+') {
 	display_msg(MSG_WARN, "pop", "Invalid response from pop server");
-	pop_close();
-	return -1; }
+	pop_close(pop);
+	return -1;  }
 
- p = b_getcfg_str(conf_name,"popasswd", "");
+ if (pop->flags & PSRC_APOP)	{
+ apoptstamp[0] = '\0';
+ if (((p = strchr(buf, '<')) != NULL) &&
+	((p1 = strchr(p, '>')) != NULL))	{
+	i = p1 - p + 1;
+	strncpy(apoptstamp, p, i);
+	apoptstamp[i] = '\0';			}
+ else
+	display_msg(MSG_LOG, pop->source->name, "APOP is not supported on this server");
+				}
 
  if (supress_errors != 1)	{
-  if ((strlen(p) <= 1) &&
-		(b_getcfg_int(conf_name,"popstorpwd", 0) == 0))
-	pop_account();
+  if ((strlen(pop->password) <= 1) &&
+		!(pop->flags & PSRC_STOREPWD))
+	pop_account(pop);
 				}
 
  while (logattempts > 0)	{
-  strncpy(user, b_getcfg_str(conf_name, "popuser", user_n), 31);
-  user[31] = '\0';
-  if (pop_command("USER %s", user) == NULL)	{
-	pop_close();
-	return -1;				}
+  if ((pop->flags & PSRC_APOP) &&
+	*apoptstamp)	{
+   MD5Init(&ctx);
+   MD5Update(&ctx, (unsigned char *)apoptstamp, strlen(apoptstamp));
+   MD5Update(&ctx, (unsigned char *)pop->password, strlen(pop->password));
+   MD5Final(digest, &ctx);
 
-  p = b_getcfg_str(conf_name, "popasswd", "");
-  if (pop_command("PASS %s", p))
+   for (p = apopdigest, i = 0; i < 16; i++, p+= 2)
+	sprintf(p, "%02x", digest[i] & 0xff);
+   apopdigest[32] = '\0';
+
+   if (pop_command(pop, "APOP %s %s", pop->username, apopdigest))
 	return 0;
+			}
+  else	{
+   if (pop_command(pop, "USER %s", pop->username) == NULL)	{
+	pop_close(pop);
+	return -1;						}
 
-  if (strncasecmp(response, "-ERR ", 4)) {
-	pop_close();
-	return -1;			 }
+   if (pop_command(pop, "PASS %s", pop->password))
+	return 0;
+	}
+
+  if (strncasecmp(pop->response, "-ERR ", 4)) {
+	pop_close(pop);
+	return -1;			      }
 
   logattempts--;
-  pop_account();
+  pop_account(pop);
 				}
 
- pop_close();
+ pop_close(pop);
  return -1;
 }
 
 int
-multiline() 
+multiline(pop) 
+struct _pop_src *pop;
 {
 char buffer[514];
  
-    if (getline(buffer, 513, pop_in) == NULL)
+    if (getline(buffer, 513, pop->pop_in) == NULL)
         return -1;
 
     if (buffer[0] == '.') {
         if (buffer[1] == '\0')
             return 0;
         else
-            (void)strcpy(response, buffer + 1);
+            (void)strcpy(pop->response, buffer + 1);
     }
     else
-        (void)strcpy(response, buffer);
+        (void)strcpy(pop->response, buffer);
 
     return 1;
 }    
 
 int
-get_pop_msg(num, only_header, realen)
+get_pop_msg(pop, num, only_header, realen)
+struct _pop_src *pop;
 int num;
 int only_header;
 long *realen;
@@ -1013,35 +1011,33 @@ struct _uidl *ucache;
 	display_msg(MSG_WARN, "pop", "No space in %s", FTEMP);
 	return -1; }
 
- sprintf(buf, "%s/%d", ftemp->fold_path, mnum);
+ snprintf(buf, sizeof(buf), "%s/%d", ftemp->fold_path, mnum);
 
  if ((mfd = fopen(buf, "w")) == NULL) {
         display_msg(MSG_WARN, "pop", "Can not open file %s", buf);
         return -1; }
 
- if (only_header == 1) {
- if ((p = pop_command("TOP %d 0", num)) == NULL) {
+ if (only_header == 1)	{
+  if ((p = pop_command(pop, "TOP %d 0", num)) == NULL)	{
 	display_msg(MSG_WARN, "pop", "Failed to retrieve header of message %d from server", num);
 	fclose(mfd);
 	unlink(buf);
-	return -1;
-					}
- } else
+	return -1;					}
+			}
+ else
  if (only_header == 2) {
- if ((p = pop_command("TOP %d 999999", num)) == NULL) {
+  if ((p = pop_command(pop, "TOP %d 999999", num)) == NULL){
 	display_msg(MSG_WARN, "pop", "Failed to retrieve message %d from server", num);
 	fclose(mfd);
 	unlink(buf);
-	return -1;
-					}
- } else  {
- if ((p = pop_command("RETR %d", num)) == NULL) {
+	return -1;					   }
+ } else	{
+ if ((p = pop_command(pop, "RETR %d", num)) == NULL)	{
 	display_msg(MSG_WARN, "pop", "Failed to retrieve message %d from server", num);
 	fclose(mfd);
 	unlink(buf);
-	return -1;
+	return -1;					}
 	}
-					}
 
  rlen = -1;
  if ((p = strchr(p, ' ')) != NULL) {
@@ -1054,7 +1050,7 @@ struct _uidl *ucache;
 				   }
 
  if (rlen <= 0) {
-  ucache = uidlcache;
+  ucache = pop->uidlcache;
   while (ucache && (ucache->mnum != num))
 		ucache = ucache->next;
 
@@ -1078,8 +1074,8 @@ struct _uidl *ucache;
  pbuf = 0;
  gettimeofday(&tv1, NULL);
 
- while ((st = multiline()) == 1) {
-	resplen = strlen(response);
+ while ((st = multiline(pop)) == 1) {
+	resplen = strlen(pop->response);
 	rlen += resplen + 2;
 	acclen += resplen + 2;
 	if ((only_header != 1) && realen && (*realen > 8192)) 	{
@@ -1087,7 +1083,7 @@ struct _uidl *ucache;
 	    acclen = 0;
 	    gettimeofday(&tv2, NULL);
 	    tdiff = (tv2.tv_sec - tv1.tv_sec) * 1000000 - tv1.tv_usec + tv2.tv_usec + 1;
- 	    display_msg(MSG_STAT, NULL, "POP: retrieving message %d of %d (%d %% - %.2f kb/sec)", num, num_msgs, rlen * 100 / *realen,
+ 	    display_msg(MSG_STAT, NULL, "POP: retrieving message %d of %d (%d %% - %.2f kb/sec)", num, pop->num_msgs, rlen * 100 / *realen,
 		(double)(((double)rlen * 1000000)/ (double)tdiff / 1024) );
 					}
 								}
@@ -1100,7 +1096,7 @@ struct _uidl *ucache;
 		pbuf--;
 			}
 
-	  if (fputs(response, mfd) == EOF) {
+	  if (fputs(pop->response, mfd) == EOF) {
 		if (errno == ENOSPC)
 			display_msg(MSG_WARN, "pop", "DISK FULL!");
 		else
@@ -1135,7 +1131,8 @@ struct _uidl *ucache;
 }
 
 int
-pop_send_message(msg)
+pop_send_message(pop, msg)
+struct _pop_src *pop;
 struct _mail_msg *msg;
 {
 char *p;
@@ -1143,7 +1140,7 @@ char *p;
  if (!msg)
 	return -1;
 
- switch (pop_init())	{
+ switch (pop_init(pop))	{
    case -1:
 	return -1;
    break;
@@ -1153,26 +1150,27 @@ char *p;
    break;
 			}
 
- if ((p = pop_command("XTND XMIT")) == NULL) {
+ if ((p = pop_command(pop, "XTND XMIT")) == NULL) {
 	display_msg(MSG_WARN, "Transmit command failed!", "Probably it's not supported on this POP server");
-	pop_end();
-	return -1; }
+	pop_end(pop);
+	return -1;				  }
 
- if (smtp_message(msg, pop_out) == -1) {
-	pop_end();
-	return -1; }
+ if (smtp_message(msg, pop->pop_out) == -1) {
+	pop_end(pop);
+	return -1;			    }
 
- if ((p = pop_command(".")) == NULL) {
+ if ((p = pop_command(pop, ".")) == NULL) {
 	display_msg(MSG_WARN, "POP Send", "Failed to send message");
-	pop_end();
-	return -1; }
+	pop_end(pop);
+	return -1;			  }
 
- pop_end();
+ pop_end(pop);
  return 0;
 }
 
 long
-get_popmsg_len(num)
+get_popmsg_len(pop, num)
+struct _pop_src *pop;
 int num;
 {
 int st;
@@ -1180,21 +1178,21 @@ struct _uidl *ucache;
 u_long msg_len;
 u_int mnum;
 
- if ((uidlcache == NULL) ||
-	!(uidlcache->flags & POP_LENGTH))
+ if ((pop->uidlcache == NULL) ||
+	!(pop->uidlcache->flags & POP_LENGTH))
 					{
-  get_popmsg_by_uidl("");
-  if (uidlcache == NULL) {
-    if (!nouidl)
+  get_popmsg_by_uidl(pop, "");
+  if (pop->uidlcache == NULL) {
+    if (!pop->nouidl)
 	return -2;
 
-    if (pop_command("LIST %d", num) == NULL) {
+    if (pop_command(pop, "LIST %d", num) == NULL) {
 	display_msg(MSG_WARN, "pop", "Can not determine message length!");
-	return -2;
-					     }
+	return -2;				  }
+
 	mnum = -1;
 	msg_len = 0L;
-	sscanf(response, "%d %lu", &mnum, &msg_len);
+	sscanf(pop->response, "%d %lu", &mnum, &msg_len);
 	if ((mnum != num) || (msg_len == -1))
 		return -2;
 
@@ -1202,16 +1200,16 @@ u_int mnum;
 
 			 }
 
-  if (pop_command("LIST") == NULL) {
+  if (pop_command(pop, "LIST") == NULL) {
 	display_msg(MSG_WARN, "pop", "Can not determine message length!");
-	return -2; }
+	return -2;			}
 
-  ucache = uidlcache;
+  ucache = pop->uidlcache;
 
-  while ((st = multiline()) == 1)  {
+  while ((st = multiline(pop)) == 1)  {
    mnum = -1;
    msg_len = 0L;
-   sscanf(response, "%d %lu", &mnum, &msg_len);
+   sscanf(pop->response, "%d %lu", &mnum, &msg_len);
 
    if ((mnum == -1) || (msg_len == 0L))
 	continue;
@@ -1220,7 +1218,7 @@ u_int mnum;
 	ucache = ucache->next;
 
    if (!ucache) {
-	ucache = uidlcache;
+	ucache = pop->uidlcache;
 	while (ucache && (ucache->mnum != mnum))
 		ucache = ucache->next;
 		}
@@ -1234,7 +1232,7 @@ u_int mnum;
 				   }
 					}
 
-  ucache = uidlcache;
+  ucache = pop->uidlcache;
   while (ucache && (ucache->mnum != num))
 		ucache = ucache->next;
 
@@ -1246,18 +1244,19 @@ u_int mnum;
 }
 
 char *
-get_popmsg_uidl(num)
+get_popmsg_uidl(pop, num)
+struct _pop_src *pop;
 int num;
 {
 struct _uidl *ucache;
 
- if (uidlcache == NULL)
-	get_popmsg_by_uidl("");
+ if (pop->uidlcache == NULL)
+	get_popmsg_by_uidl(pop,"");
 
- if (nouidl)
+ if (pop->nouidl)
 	return NULL;
 
- ucache = uidlcache;
+ ucache = pop->uidlcache;
 
  while (ucache) {
 	if (ucache->mnum == num)
@@ -1270,24 +1269,25 @@ struct _uidl *ucache;
 }
 
 struct _uidl *
-get_popmsg_by_uidl(uidl)
+get_popmsg_by_uidl(pop, uidl)
+struct _pop_src *pop;
 char *uidl;
 {
 int st;
 char muidl[MAX_UIDL];
 struct _uidl *ucache, *uidl1, *uidl2;
 
- if (uidlcache == NULL) {
-	if (nouidl)
+ if (pop->uidlcache == NULL) {
+	if (pop->nouidl)
 		return NULL;
 
-  if (pop_command("UIDL") == NULL) {
-	nouidl = 1;
-	return NULL;		   }
+  if (pop_command(pop, "UIDL") == NULL) {
+	pop->nouidl = 1;
+	return NULL;		        }
 
 			}
  else	{
-  ucache = uidlcache;
+  ucache = pop->uidlcache;
   while (ucache) {
    if (!strcmp(ucache->uidlstr, uidl))
 	return ucache;
@@ -1300,10 +1300,10 @@ struct _uidl *ucache, *uidl1, *uidl2;
 
  uidl2 = NULL;
 
- while ((st = multiline()) == 1)  {
+ while ((st = multiline(pop)) == 1)  {
   muidl[0] = '\0';
   st = 0;
-  sscanf(response, "%d %70s", &st, muidl);
+  sscanf(pop->response, "%d %70s", &st, muidl);
   ucache = (struct _uidl *)malloc(sizeof(struct _uidl));
   ucache->mnum = st;
   ucache->mlen = 0;
@@ -1314,10 +1314,10 @@ struct _uidl *ucache, *uidl1, *uidl2;
   if (!strcmp(muidl, uidl))
 	uidl2 = ucache;
 
-  if (uidlcache == NULL)
-	uidlcache = ucache;
+  if (pop->uidlcache == NULL)
+	pop->uidlcache = ucache;
   else 	{
-   uidl1 = uidlcache;
+   uidl1 = pop->uidlcache;
    while (uidl1->next)
 	uidl1 = uidl1->next;
 
@@ -1325,31 +1325,33 @@ struct _uidl *ucache, *uidl1, *uidl2;
 	}
 				  }
 
- compare_uidlist();
+ compare_uidlist(pop);
  return uidl2;
 }
 
 int
-get_popmsg_num()
+get_popmsg_num(pop)
+struct _pop_src *pop;
 {
 char *p;
 char dumb[5];
 int msgs_len = 0;
 
- if ((p = pop_command("STAT")) == NULL)
+ if ((p = pop_command(pop, "STAT")) == NULL)
 	return -1;
 
- sscanf(p, "%s %d %d", dumb, &num_msgs, &msgs_len);
+ sscanf(p, "%s %d %d", dumb, &pop->num_msgs, &msgs_len);
 
- if (num_msgs == -1) {
+ if (pop->num_msgs == -1) {
 	display_msg(MSG_WARN, "pop", "STAT failed");
-        return -1; }
+        return -1;	  }
 
- return num_msgs;
+ return pop->num_msgs;
 }
 
 int
-pop_delmsg_by_uidl(msg)
+pop_delmsg_by_uidl(pop, msg)
+struct _pop_src *pop;
 struct _mail_msg *msg;
 {
 int mnum;
@@ -1357,7 +1359,10 @@ struct _head_field *fld;
 struct _uidl *uidl;
 int init;
 
- init = (popsock == -1) ? 1 : 0;
+ if (pop == NULL)
+	return -1;
+
+ init = (pop->popsock == -1) ? 1 : 0;
 
  if (!msg)
 	return -1;
@@ -1367,41 +1372,42 @@ int init;
 	return -1;				}
 
  if (init) {
-   if (pop_init() != 0)
+   if (pop_init(pop) != 0)
 	return -1;
 	   }
 
- if ((uidl = get_popmsg_by_uidl(fld->f_line)) == NULL)	{
-	if (nouidl)
+ if ((uidl = get_popmsg_by_uidl(pop, fld->f_line)) == NULL)	{
+	if (pop->nouidl)
 		display_msg(MSG_WARN, "pop", "You can not use this feature\nsince your POP server does not support UIDL command");
 	if (init)
-	 pop_end();
+	 pop_end(pop);
 	return -1;					}
 
  mnum = uidl->mnum;
 
  if (mnum == 0)		{
 	if (init)
-	 pop_end();
+	 pop_end(pop);
 	return -1;	}
 
  if (!(uidl->flags & POP_DELETED)) {
 	display_msg(MSG_STAT, NULL, "POP: deleting message %d", mnum);
-	pop_command("DELE %d", mnum);
+	pop_command(pop, "DELE %d", mnum);
 	uidl->flags |= POP_DELETED;
 				   }
 
  if (init)
-	pop_end();
+	pop_end(pop);
 
  msg->flags &= ~H_ONLY;
- delete_uidlist(fld->f_line);
+ delete_uidlist(pop, fld->f_line);
 
  return 0;
 }
 
 int
-pop_getfull_msg(msg)
+pop_getfull_msg(pop, msg)
+struct _pop_src *pop;
 struct _mail_msg *msg;
 {
 struct _head_field *fld;
@@ -1412,7 +1418,7 @@ char buf[255];
 char msgf[255];
 long realen;
 
- if (!msg)
+ if (!msg || !pop)
 	return -1;
 
  if (!(msg->flags & H_ONLY))
@@ -1422,44 +1428,44 @@ long realen;
 	display_msg(MSG_WARN, "pop", "Message does not have %s identifier", XUIDL);
 	return -1;				}
 
- if (pop_init() != 0)
+ if (pop_init(pop) != 0)
 	return -1;
 
- if ((uidl = get_popmsg_by_uidl(fld->f_line)) == NULL)	{
-	if (nouidl)
+ if ((uidl = get_popmsg_by_uidl(pop, fld->f_line)) == NULL)	{
+	if (pop->nouidl)
 	  display_msg(MSG_WARN, "pop", "You can not use thise feature\nsince your POP server does not support UIDL command");
 	else
 	  display_msg(MSG_WARN, "pop", "Failed to find message");
-	pop_end();
-	return -1;					}
+	pop_end(pop);
+	return -1;						}
 
  mnum = uidl->mnum;
 
  if (mnum == 0)		{
 	display_msg(MSG_WARN, "pop", "Can not find message, probably it's no longer on the server");
-	pop_end();
+	pop_end(pop);
 	return -1;	}
 
- if ((nnum = get_pop_msg(mnum, 0, &realen)) == -1)	{
-	pop_end();
+ if ((nnum = get_pop_msg(pop, mnum, 0, &realen)) == -1)	{
+	pop_end(pop);
 	return -1;					}
 
- if (b_getcfg_int(conf_name, "popdelete", 0))
-	pop_command("DELE %d", mnum);
+ if (pop->flags & PSRC_DELETE)
+	pop_command(pop, "DELE %d", mnum);
 
- sprintf(msgf, "%s/%d", ftemp->fold_path, nnum);
+ snprintf(msgf, sizeof(msgf), "%s/%d", ftemp->fold_path, nnum);
 
  if ((bfd = fopen(msgf, "r")) == NULL)	{
 	display_msg(MSG_WARN, "pop", "Can not open retrieved message");
 	unlink(msgf);
-	pop_end();
+	pop_end(pop);
 	return -1;			}
 
  if ((mfd = fopen(msg->get_file(msg), "a")) == NULL)	{
 	display_msg(MSG_WARN, "pop", "Can not open message %s", msg->get_file(msg));
 	fclose(bfd);
 	unlink(msgf);
-	pop_end();
+	pop_end(pop);
 	return -1;					}
 
  st = 0;
@@ -1480,22 +1486,23 @@ long realen;
 
  msg->flags &= ~H_ONLY;
  replace_field(msg, XUIDL, uidl->uidlstr);
- pop_end();
+ pop_end(pop);
 
  return 0;
 }
 
 int
-if_popmsg_retr(num, xlst)
-int num, xlst;
+if_popmsg_retr(pop, num)
+struct _pop_src *pop;
+int num;
 {
 char *p;
 int ml, st;
 
- if (xlst)
-	p = pop_command("XTND XLST Status %d", num);
+ if (pop->flags & PSRC_STATXLST)
+	p = pop_command(pop, "XTND XLST Status %d", num);
  else
-	p = pop_command("TOP %d 0", num);
+	p = pop_command(pop, "TOP %d 0", num);
 
  if (!p) {
 	display_msg(MSG_WARN, "pop", "Can not determine message status");
@@ -1503,8 +1510,8 @@ int ml, st;
 	}
 
  st = 0;
- while ((ml = multiline()) == 1)  {
-   if ((p = strstr(response, "Status:")) != NULL) {
+ while ((ml = multiline(pop)) == 1)  {
+   if ((p = strstr(pop->response, "Status:")) != NULL) {
 	p += 7;
 	if (strchr(p, 'R'))
 		st = 1;
@@ -1515,113 +1522,118 @@ int ml, st;
 }
 
 void
-free_uidlist()
+free_uidlist(pop)
+struct _pop_src *pop;
 {
 int i;
 
- if (uidfirst == -2)	{
+ if (pop->uidfirst == -2)	{
   for (i = 0; i < MAX_POP_UIDS; i++)
-   pop_uids[i] = NULL;
-			}
+   pop->pop_uids[i] = NULL;
+				}
  else	{
   for (i = 0; i < MAX_POP_UIDS; i++)	{
-   if (pop_uids[i])
-	free(pop_uids[i]);
-   pop_uids[i] = NULL;
+   if (pop->pop_uids[i] != NULL)
+	free(pop->pop_uids[i]);
+   pop->pop_uids[i] = NULL;
 					}
 	}
- uidfirst = -1;
+ pop->uidfirst = -1;
 
  return;
 }
 
 void
-load_uidlist()
+load_uidlist(pop)
+struct _pop_src *pop;
 {
 char uidfname[255];
 char uidstr[MAX_UIDL + 2];
 FILE *ufd;
 int i = 0;
 
- free_uidlist();
+ free_uidlist(pop);
 
- sprintf(uidfname, "%s/.xfmpopuid", configdir);
+ snprintf(uidfname, sizeof(uidfname), "%s/.xfmpopuid-%s", configdir, pop->source->name);
  if ((ufd = fopen(uidfname, "r")) == NULL)	{
-	uidfirst = 0;
+	pop->uidfirst = 0;
 	return; 
 						}
 
  while (fgets(uidstr, MAX_UIDL - 1, ufd)) {
   strip_newline(uidstr);
-  pop_uids[i] = strdup(uidstr);
+  pop->pop_uids[i] = strdup(uidstr);
   if (++i >= MAX_POP_UIDS)
 	break;
 				          }
  fclose(ufd);
- uidfirst = 0;
+ pop->uidfirst = 0;
 
  return;
 }
 
 void
-save_uidlist()
+save_uidlist(pop)
+struct _pop_src *pop;
 {
 FILE *ufd;
 char uidfname[255];
-int i = uidfirst;
+int i = pop->uidfirst;
 
- if (uidfirst < 0)
+ if (pop->uidfirst < 0)
 	return;
 
- sprintf(uidfname, "%s/.xfmpopuid", configdir);
+ snprintf(uidfname, sizeof(uidfname), "%s/.xfmpopuid-%s", configdir, pop->source->name);
  if ((ufd = fopen(uidfname, "w")) == NULL)	{
 	display_msg(MSG_WARN, "Message uids will not be stored", "Can not open %s", uidfname);
-	uidfirst = -3;
+	pop->uidfirst = -3;
 	return; 
 						}
 
  do {
-  if (pop_uids[i])    {
-    fputs(pop_uids[i], ufd);
+  if (pop->pop_uids[i])    {
+    fputs(pop->pop_uids[i], ufd);
     fputc('\n', ufd); }
 
   i++;
   if (i >= MAX_POP_UIDS)
 	i = 0;
- } while (i != uidfirst);
+ } while (i != pop->uidfirst);
  fclose(ufd);
 
  return;
 }
 
 void
-append_uidlist(uidstr)
+append_uidlist(pop, uidstr)
+struct _pop_src *pop;
 char *uidstr;
 {
  if (!uidstr || !*uidstr || (strlen(uidstr) >= MAX_UIDL))
 	return;
 
- if (uidfirst == -3)
+ if (pop->uidfirst == -3)
 	return;
 
- if (uidfirst < 0)
-	load_uidlist();
+ if (pop->uidfirst < 0)
+	load_uidlist(pop);
 
- if (check_uidlist(uidstr) != 0)
+ if (check_uidlist(pop, uidstr) != 0)
 	return;
 
- uidfirst--;
- if (uidfirst < 0)
-	uidfirst = MAX_POP_UIDS - 1;
- if (pop_uids[uidfirst])
-	free(pop_uids[uidfirst]);
- pop_uids[uidfirst] = strdup(uidstr);
+ pop->uidfirst--;
+ if (pop->uidfirst < 0)
+	pop->uidfirst = MAX_POP_UIDS - 1;
+ if (pop->pop_uids[pop->uidfirst])
+	free(pop->pop_uids[pop->uidfirst]);
+ pop->pop_uids[pop->uidfirst] = strdup(uidstr);
 
  return;
 }
 
 int
-check_uidlist(uidstr)
+check_uidlist(pop, uidstr)
+struct _pop_src *pop;
 char *uidstr;
 {
 int i = 0;
@@ -1630,14 +1642,14 @@ int i = 0;
 	(strlen(uidstr) >= MAX_UIDL))
 	return 0;
 
- if (uidfirst == -3)
+ if (pop->uidfirst == -3)
 	return 0;
 
- if (uidfirst < 0)
-	load_uidlist();
+ if (pop->uidfirst < 0)
+	load_uidlist(pop);
 
  for (i = 0; i < MAX_POP_UIDS; i++) {
-  if (pop_uids[i] && !strcmp(pop_uids[i], uidstr))
+  if (pop->pop_uids[i] && !strcmp(pop->pop_uids[i], uidstr))
 	return 1;
 				    }
 
@@ -1645,7 +1657,8 @@ int i = 0;
 }
 
 void
-delete_uidlist(uidstr)
+delete_uidlist(pop, uidstr)
+struct _pop_src *pop;
 char *uidstr;
 {
 int i;
@@ -1653,16 +1666,16 @@ int i;
  if (!uidstr || !*uidstr || (strlen(uidstr) >= MAX_UIDL))
 	return;
 
- if (uidfirst < 0)
-	load_uidlist();
+ if (pop->uidfirst < 0)
+	load_uidlist(pop);
 
- if (uidfirst == -3)
+ if (pop->uidfirst == -3)
 	return;
 
  for (i = 0; i < MAX_POP_UIDS; i++) {
-  if (pop_uids[i] && !strcmp(pop_uids[i], uidstr)) {
-	free(pop_uids[i]);
-	pop_uids[i] = NULL;
+  if (pop->pop_uids[i] && !strcmp(pop->pop_uids[i], uidstr)) {
+	free(pop->pop_uids[i]);
+	pop->pop_uids[i] = NULL;
 	break;					   }
 				    }
 
@@ -1670,92 +1683,89 @@ int i;
 }
 
 void
-compare_uidlist()
+compare_uidlist(pop)
+struct _pop_src *pop;
 {
 int i;
 
- if (uidlcache == NULL)
+ if (pop->uidlcache == NULL)
 	return;
 
- if (uidfirst < 0)
-	load_uidlist();
+ if (pop->uidfirst < 0)
+	load_uidlist(pop);
 
- if (uidfirst == -3)
+ if (pop->uidfirst == -3)
 	return;
 
  for (i = 0; i < MAX_POP_UIDS; i++)	{
-  if (pop_uids[i] &&
-	(get_popmsg_by_uidl(pop_uids[i]) == NULL)) {
-	free(pop_uids[i]);
-	pop_uids[i] = NULL;			   }
+  if (pop->pop_uids[i] &&
+	(get_popmsg_by_uidl(pop, pop->pop_uids[i]) == NULL))	{
+	free(pop->pop_uids[i]);
+	pop->pop_uids[i] = NULL;				}
 					}
 
  return;
 }
 
 int
-if_popmsg_uid_cached(msg)
+if_popmsg_uid_cached(pop, msg)
+struct _pop_src *pop;
 int msg;
 {
 char *uid;
 
- if (uidfirst < 0)
-	load_uidlist();
+ if (pop->uidfirst < 0)
+	load_uidlist(pop);
 
- if (uidfirst < 0)
+ if (pop->uidfirst < 0)
 	return -1;
 
-  if ((uid = get_popmsg_uidl(msg)) == NULL)
+  if ((uid = get_popmsg_uidl(pop, msg)) == NULL)
 	return -1;
 
-  return check_uidlist(uid);
+  return check_uidlist(pop, uid);
 }
 
 int
-pop_inc(notify)
+pop_inc(source, notify)
+struct _retrieve_src *source;
 int *notify;
 {
+struct _pop_src *pop;
+struct _mail_msg *msg;
 int i, honly, uidcached;
 int num_msgs, retr_num;
-struct _mail_msg *msg;
-int popmaxact, popcheckretr, popgetstat, popdelete, popdmark, pop2;
 long popmaxmsg;
 char mclen[16], *muidl;
 long realen;
 int nnum = -1;
 
- popmaxact = b_getcfg_int(conf_name, "popmaxact", 0);
- pop2 = b_getcfg_int(conf_name, "pop2", 0);
- popmaxmsg = b_getcfg_int(conf_name, "popmaxmsg", -1);
- if (popmaxmsg >= 0)
-	popmaxmsg *= 1024;
+ if (source->flags & RSRC_DISABLED)
+        return 0;
+
+ pop = (struct _pop_src *)source->spec;
+
+ if (pop->maxmsg >= 0)
+	popmaxmsg = pop->maxmsg * 1024;
  else
 	popmaxmsg = MAX_MSG_LEN;
 
  if (popmaxmsg > MAX_MSG_LEN)
 	popmaxmsg = MAX_MSG_LEN;
 
- popcheckretr = b_getcfg_int(conf_name, "popcheckretr", 0);
- popgetstat = b_getcfg_int(conf_name, "popgetstat", 0);
-
- popdelete = b_getcfg_int(conf_name, "popdelete", 0);
- popdmark = b_getcfg_int(conf_name, "popdmark", 0);
- if (popdmark)
-	popdmark = 2;
-
- if (pop_init() != 0)
+ if (pop_init(pop) != 0)
 	return -1;
 
- if ((num_msgs = get_popmsg_num()) == -1) {
-	pop_end();
-	return -1; }
+ if ((num_msgs = get_popmsg_num(pop)) == -1)	{
+	pop_end(pop);
+	return -1;				}
 
  if (num_msgs == 0) {
-	free_uidlist();
-	uidfirst = 0;
-	save_uidlist();
-	pop_end();
-	return 0; }
+	free_uidlist(pop);
+	pop->uidfirst = 0;
+	save_uidlist(pop);
+	pop_end(pop);
+	return 0;   }
 
  retr_num = 0;
 
@@ -1768,62 +1778,65 @@ int nnum = -1;
 	if (abortpressed())
 		break;
 
-	if (!pop2 && popcheckretr) {
-	  if (((popcheckretr == 1) || (popcheckretr == 3))
-		&& (if_popmsg_retr(i, popgetstat) > 0) )
+	if (!(pop->flags & PSRC_POP2) &&
+		((pop->flags & PSRC_CHECKUID) ||
+		(pop->flags & PSRC_CHECKSTAT)))	{
+	  if ((pop->flags & PSRC_CHECKSTAT) &&
+		(if_popmsg_retr(pop, i) > 0) )
 		continue;
 
-	  if (((popcheckretr == 2) || (popcheckretr == 3))
-		&& ( (uidcached = if_popmsg_uid_cached(i)) == 1) )
+	  if ((pop->flags & PSRC_CHECKUID) &&
+		((uidcached = if_popmsg_uid_cached(pop, i)) == 1))
 		continue;
-			  }
+						}
 
-	if (!pop2 && (popmaxmsg >= 0) && 
-		((realen = get_popmsg_len(i)) >= popmaxmsg)) {
-	  if (popmaxact > 0)
+	if (!(pop->flags & PSRC_POP2) &&
+		(popmaxmsg >= 0) && 
+		((realen = get_popmsg_len(pop, i)) >= popmaxmsg)) {
+	  if (pop->flags & PSRC_SKIPBIG)
 		continue;
 
-	  if (nouidl)   {
+	  if (pop->nouidl)	{
 		display_msg(MSG_WARN, "Can not retrieve message header, skipping", "Your POP server does not support UIDL command\nIt will be impossible to match header and message left on the server later");
-		continue;
-			}
+		continue;	}
 
  	  display_msg(MSG_STAT, NULL, "POP: retrieving header of message %d of %d", i, num_msgs);
-	  if ((nnum = get_pop_msg(i, 1, &realen)) == -1) {
-		pop_end();
-		return -1; }
+	  if ((nnum = get_pop_msg(pop, i, 1, &realen)) == -1)	{
+		pop_end(pop);
+		return -1;					}
 
 	  if ((msg = get_message(nnum, ftemp)) == NULL) {
-		pop_end();
-		return -1; }
+		pop_end(pop);
+		return -1;				}
 
 	  msg->flags |= H_ONLY;
 
 	  if (realen > 0) {
-		sprintf(mclen, "%lu", realen);
+		snprintf(mclen, sizeof(mclen), "%lu", realen);
 	  	replace_field(msg, MIME_C_LENGTH, mclen);
-			 }
+			  }
 
 	  honly = 1;
 	} else {
 
  	display_msg(MSG_STAT, NULL, "POP: retrieving message %d of %d", i, num_msgs);
-	if ((nnum = get_pop_msg(i, popdmark, &realen)) == -1) {
-		pop_end();
-		return -1; }
+	if ((nnum = get_pop_msg(pop, i,
+		(pop->flags & PSRC_DMARK) ? 2 : 0, &realen)) == -1) {
+		pop_end(pop);
+		return -1;					    }
 
-	if ((msg = get_message(nnum, ftemp)) == NULL) {
-		pop_end();
-		return -1; }
+	if ((msg = get_message(nnum, ftemp)) == NULL)	{
+		pop_end(pop);
+		return -1;				}
 
 		}
 
-	if (!pop2)	{
-	 if ((muidl = get_popmsg_uidl(i)) != NULL) {
+	if (!(pop->flags & PSRC_POP2))	{
+	 if ((muidl = get_popmsg_uidl(pop, i)) != NULL) {
 	  replace_field(msg, XUIDL, muidl);
-	  if (((popcheckretr == 2) || (popcheckretr == 3)) &&
-		(uidcached == 0) && (!popdelete || honly))
-		append_uidlist(muidl);
+	  if ((pop->flags & PSRC_CHECKUID) &&
+		(uidcached == 0) && (!(pop->flags & PSRC_DELETE) || honly))
+		append_uidlist(pop, muidl);
 						   }
 			}
 
@@ -1831,7 +1844,10 @@ int nnum = -1;
 	convert_fields(msg);
 	msg->folder = ftemp;
 	msg->status |= (CHANGED | RECENT);
-	add_field(msg, "X-RDate", get_arpa_date(time(NULL)));
+	if (source->flags & RSRC_MARKREAD)
+		msg->flags &= ~UNREAD;
+	replace_field(msg, "X-RDate", get_arpa_date(time(NULL)));
+	replace_field(msg, SOURCE_FIELD, source->name);
 	msg->header->rcv_time = time(NULL);
 
 #ifdef FACES
@@ -1840,11 +1856,12 @@ int nnum = -1;
 
 	switch(apply_rule(msg, 0))	{
 	   case 0:
-		(*notify)++;
+		if (!(source->flags & RSRC_NONOTIFY))
+			(*notify)++;
 	   break;
 
 	   case -1:
-		pop_end();
+		pop_end(pop);
 		unlink(msg->get_file(msg));
 		discard_message(msg);
 		return -1;
@@ -1853,11 +1870,156 @@ int nnum = -1;
 
 	retr_num++;
 
-	if (popdelete && !honly)
-		pop_command("DELE %d", i);
+	if ((pop->flags & PSRC_DELETE) && !honly)
+		pop_command(pop, "DELE %d", i);
 
 						}
 
- pop_end();
+ pop_end(pop);
  return retr_num;
 }
+
+void
+init_pop_source(source)
+struct _retrieve_src *source;
+{ 
+struct _pop_src *pop;
+struct _uidl *ucache;
+int i;
+ 
+ if (source->spec == NULL)      {
+  source->spec = (struct _pop_src *)malloc(sizeof(struct _pop_src));
+  pop = (struct _pop_src *)source->spec;
+  pop->source = source;
+  strcpy(pop->hostname, "127.0.0.1");
+  strcpy(pop->service, "110");
+  strcpy(pop->username, user_n);
+  *pop->password = '\0';
+  pop->maxmsg = -1;
+  pop->flags = PSRC_DELETE;
+				}
+ else	{
+  pop = (struct _pop_src *)source->spec;
+  if (pop->popsock > 0)
+	close(pop->popsock);
+  if (pop->pop_in)
+	fclose(pop->pop_in);
+  if (pop->pop_out)
+	fclose(pop->pop_out);
+
+  while (pop->uidlcache) 	{
+   ucache = pop->uidlcache;
+   pop->uidlcache = pop->uidlcache->next;
+   free(ucache);		}
+
+  for (i = 0; i < MAX_POP_UIDS; i++)	{
+   if (pop->pop_uids[i])
+	free(pop->pop_uids[i]);
+   pop->pop_uids[i] = NULL;		}
+	}
+
+ pop->popsock = -1;
+ pop->pop_in = NULL;
+ pop->pop_out = NULL;
+ pop->uidlcache = NULL;
+ pop->nouidl = 0;
+ pop->num_msgs = -1;
+ pop->uidfirst = -2;
+ *pop->response = '\0';
+
+ return;
+}
+
+void
+free_pop_source(source)
+struct _retrieve_src *source;
+{
+ if (source->spec)      {
+  init_pop_source(source);
+  free(source->spec);
+  source->spec = NULL;  }
+
+ return;
+}
+
+int
+load_pop_source(source, fd)
+struct _retrieve_src *source;
+FILE *fd;
+{
+struct _pop_src *pop;
+char buf[255], *p, *p1;
+
+ pop = (struct _pop_src *)source->spec;
+ if (!fgets(buf, 255, fd))
+	return -1;
+ strip_newline(buf);
+ if (sscanf(buf, "%s %15s", pop->hostname, pop->service) != 2)
+	return -1;
+
+ if (!fgets(buf, 255, fd))
+	return -1;
+ strip_newline(buf);
+
+ p1 = buf;
+ if ((p = get_quoted_str(&p1)) == NULL)
+	return -1;
+ strncpy(pop->username, p, 31);
+ pop->username[31] = '\0';
+ *pop->password = '\0';
+ if ((p = get_quoted_str(&p1)) != NULL)	{
+	strncpy(pop->password, p, 31);
+	pop->password[31] = '\0';	}
+
+ if (!fgets(buf, 255, fd))
+	return -1;
+ strip_newline(buf);
+ if (sscanf(buf, "%ld %d", &pop->maxmsg, &pop->flags) != 2)
+	return -1;
+
+ return 0;
+}
+  
+int
+save_pop_source(source, fd)
+struct _retrieve_src *source;
+FILE *fd;
+{
+struct _pop_src *pop;
+
+ pop = (struct _pop_src *)source->spec;
+ fprintf(fd, "%s %s\n", pop->hostname, pop->service);
+
+ if (strchr(pop->username, ' '))
+	fprintf(fd, "\"%s\"", pop->username);
+ else
+	fprintf(fd, "%s", pop->username);
+
+ if (pop->flags & PSRC_STOREPWD)	{
+  if (strchr(pop->password, ' '))
+	fprintf(fd, " \"%s\"\n", pop->password);
+  else
+	fprintf(fd, " %s\n", pop->password);
+					}
+ else
+	fprintf(fd, "\n");
+
+ fprintf(fd, "%ld %d\n", pop->maxmsg, pop->flags);
+ return 0;
+}
+
+void
+pop_source(source)
+struct _retrieve_src *source;
+{
+ init_pop_source(source);
+ source->type = RSRC_POP;
+ source->retrieve = pop_inc;
+ source->free = free_pop_source;
+ source->init = init_pop_source;
+ source->load = load_pop_source;
+ source->save = save_pop_source;
+
+ return;
+}
+
