@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: addr_book.c,v 2.9 1997/12/17 14:56:53 gena Exp $
+/* $Id: addr_book.c,v 2.14 1998/03/29 16:09:22 gena Exp $
  */
 
 #include <fmail.h>
@@ -236,9 +236,9 @@ FILE *afd;
   changed = 0;
 
   if (name)
-	sprintf(buf, "%s/.xfbook.%s", configdir, name);
+	snprintf(buf, sizeof(buf), "%s/.xfbook.%s", configdir, name);
   else
-	sprintf(buf, "%s/.xfbook", configdir);
+	snprintf(buf, sizeof(buf), "%s/.xfbook", configdir);
 
   if ((afd = fopen(buf, "r")) == NULL) {
 	  if ((afd = fopen(buf, "w")) == NULL)  {
@@ -388,14 +388,15 @@ char buf[255], buftmp[255];
 struct _abook_entry *entry;
 struct _mail_addr *addr;
 
-  if (book == NULL)
-	return 0;
+  if ((book == NULL) || readonly)	{
+	changed = 0;
+	return 0;			}
 
-  sprintf(buftmp, "%s/.__save_xfbook", configdir);
+  snprintf(buftmp, sizeof(buftmp), "%s/.__save_xfbook", configdir);
   if (book->name && strlen(book->name) && strcmp(book->name, "default"))
-	sprintf(buf, "%s/.xfbook.%s", configdir, book->name);
+	snprintf(buf, sizeof(buf), "%s/.xfbook.%s", configdir, book->name);
   else
-	sprintf(buf, "%s/.xfbook", configdir);
+	snprintf(buf, sizeof(buf), "%s/.xfbook", configdir);
 
   if ((afd = fopen(buftmp, "w")) == NULL)  {
 	display_msg(MSG_WARN, "save book", "Can not open\n%s", buftmp);
@@ -489,8 +490,8 @@ convert_book()
 FILE *afd, *nafd;
 char buf[255], buf1[255], buf2[255];
 
- sprintf(buf, "%s/.xfbook", configdir);
- sprintf(buf1, "%s_tmp", buf);
+ snprintf(buf, sizeof(buf), "%s/.xfbook", configdir);
+ snprintf(buf1, sizeof(buf1), "%s_tmp", buf);
 
  if ((afd = fopen(buf, "r")) == NULL) {
 	display_msg(MSG_WARN, "address book", "Can not open address book %s", buf);
@@ -664,9 +665,9 @@ char buf[255], tl = 1, i = 1;
 
    case ADDR_ALIAS:
 	if (entry->description)
-	 sprintf(buf, "[%s] %s", entry->description, get_full_addr_line(entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s] %s", entry->description, get_full_addr_line(entry->addr));
 	else
-	 sprintf(buf, "[%s]", get_full_addr_line(entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(entry->addr));
 	fl_add_browser_line(addr_book_obj->Address_Browse, buf);
    break;
 			}
@@ -814,10 +815,11 @@ else	{
 
 fl_do_only_forms(); 
 
-if (changed) {
+if (changed && !readonly) {
  if (display_msg(MSG_QUEST, "address book", "Do you want to save changes into the address book?"))
 	save_book();
 	}
+changed = 0;
 
 sprintf(geom, "%d %d", addr_book_obj->Address_Book->w, addr_book_obj->Address_Book->h);
 b_putcfg_str(conf_name, "abookgeom", geom);
@@ -825,7 +827,6 @@ fl_hide_form(addr_book_obj->Address_Book);
 fl_free_form(addr_book_obj->Address_Book);
 free(addr_book_obj);
 addr_book_obj = NULL;
-do_keyboard_hack();
 
 if (addr_type)
 	*addr_type = type;
@@ -898,7 +899,7 @@ struct _abook_entry *entry, *entry1;
  display_book();
  if (i <= fl_get_browser_maxline(addr_book_obj->Address_Browse)) {
   fl_select_browser_line(addr_book_obj->Address_Browse, i);
-  fl_set_browser_topline(addr_book_obj->Address_Browse, i);	 }
+  fl_show_browser_line(addr_book_obj->Address_Browse, i);	 }
 
  return;
 }
@@ -924,9 +925,9 @@ char buf[255];
 	continue;
 
    if (name && strcmp(name, "default"))
-	sprintf(buf, "%s/.xfbook.%s", configdir, name);
+	snprintf(buf, sizeof(buf), "%s/.xfbook.%s", configdir, name);
    else
-	sprintf(buf, "%s/.xfbook", configdir);
+	snprintf(buf, sizeof(buf), "%s/.xfbook", configdir);
 
    if ((afd = fopen(buf, "a")) == NULL) {
 	display_msg(MSG_WARN, "addr copy", "Can not append to\n%s", buf);
@@ -1097,7 +1098,7 @@ srch: for (i = sline; i <= eline; i++)
 
 	if (strcasestr((char *)ln, (char *)inp, 1)) {
 		fl_select_browser_line(addr_book_obj->Address_Browse, i);
-		fl_set_browser_topline(addr_book_obj->Address_Browse, i);
+		fl_show_browser_line(addr_book_obj->Address_Browse, i);
 		cur_entry = get_entry_by_idx(i);
 		fl_set_input(addr_book_obj->AB_PGP,
 		   cur_entry->addr->pgpid ? cur_entry->addr->pgpid : "");
@@ -1192,12 +1193,12 @@ char *addr = (char *)fl_get_input(addr_book_obj->AB_Input);
   changed = 1;
   if (entry->type == ADDR_ALIAS) 	{
    if (entry->description)
-    sprintf(buf, "[%s] %s", cur_entry->description, get_full_addr_line(entry->addr));
+    snprintf(buf, sizeof(buf), "[%s] %s", cur_entry->description, get_full_addr_line(entry->addr));
    else
-    sprintf(buf, "[%s]", get_full_addr_line(entry->addr));
+    snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(entry->addr));
 					}
   else
-   sprintf(buf, "%s", get_full_addr_line(entry->addr));
+   snprintf(buf, sizeof(buf), "%s", get_full_addr_line(entry->addr));
   fl_replace_browser_line(addr_book_obj->Address_Browse, get_entry_idx(entry), buf);
 
   return;
@@ -1330,9 +1331,9 @@ char buf[255];
 
 ee:
  if (cur_entry->description)
-  sprintf(buf, "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
+  snprintf(buf, sizeof(buf), "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
  else
-  sprintf(buf, "[%s]", get_full_addr_line(cur_entry->addr));
+  snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(cur_entry->addr));
  fl_replace_browser_line(addr_book_obj->Address_Browse, get_entry_idx(cur_entry), buf);
 
  fl_deactivate_form(addr_book_obj->Address_Book);
@@ -1384,7 +1385,7 @@ FILE *afd;
 	return;				}
 				  }
 
-   sprintf(buf, "%s/.xfbook.%s", configdir, p);
+   snprintf(buf, sizeof(buf), "%s/.xfbook.%s", configdir, p);
 
 
    if ((afd = fopen(buf, "w")) == NULL) {
@@ -1419,7 +1420,7 @@ char *p, buf[255];
     if (!display_msg(MSG_QUEST|MSG_DEFNO, NULL, "Address book %s will be deleted\nAre you sure?", p))
 	return;
 
-    sprintf(buf, "%s/.xfbook.%s", configdir, p);
+    snprintf(buf, sizeof(buf), "%s/.xfbook.%s", configdir, p);
     if (unlink(buf) == -1)
 	display_msg(MSG_WARN, "address book", "Failed to delete address book %s", p);
     else {
@@ -1688,9 +1689,9 @@ else
 changed = 1;
 display_entry();
 if (cur_entry->description)
- sprintf(buf, "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
+ snprintf(buf, sizeof(buf), "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
 else
- sprintf(buf, "[%s]", get_full_addr_line(cur_entry->addr));
+ snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(cur_entry->addr));
 fl_replace_browser_line(addr_book_obj->Address_Browse, get_entry_idx(cur_entry), buf);
 
 return;
@@ -1742,12 +1743,12 @@ int num, i;
   if (cur_entry->addr && cur_entry->addr->next_addr)  	{
 	cur_entry->type = ADDR_ALIAS;
 	if (cur_entry->description)
-	 sprintf(buf, "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
 	else
-	 sprintf(buf, "[%s]", get_full_addr_line(cur_entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(cur_entry->addr));
 							}
   else
-	sprintf(buf, "%s", get_full_addr_line(cur_entry->addr));
+	snprintf(buf, sizeof(buf), "%s", get_full_addr_line(cur_entry->addr));
 
   fl_replace_browser_line(addr_book_obj->Address_Browse, get_entry_idx(cur_entry), buf);
   return;
@@ -1809,12 +1810,12 @@ int num, i;
   if (cur_entry->addr)  	{
 	cur_entry->type = ADDR_ALIAS;
 	if (cur_entry->description)
-	 sprintf(buf, "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s] %s", cur_entry->description, get_full_addr_line(cur_entry->addr));
 	else
-	 sprintf(buf, "[%s]", get_full_addr_line(cur_entry->addr));
+	 snprintf(buf, sizeof(buf), "[%s]", get_full_addr_line(cur_entry->addr));
 				}
   else
-	sprintf(buf, "%s", get_full_addr_line(cur_entry->addr));
+	snprintf(buf, sizeof(buf), "%s", get_full_addr_line(cur_entry->addr));
 
   fl_replace_browser_line(addr_book_obj->Address_Browse, get_entry_idx(cur_entry), buf);
   return;
@@ -2265,7 +2266,7 @@ AB_Lookup_Call(FL_OBJECT *obj, long data)
   			 addr_book_obj->Address_Book);
   my_activate();
   if (keyid) {
-    sprintf(buf,"0x%X",keyid);
+    snprintf(buf, sizeof(buf), "0x%X", keyid);
     fl_set_input(addr_book_obj->AB_PGP,buf);
   }
 }
@@ -2277,7 +2278,7 @@ AMOD_Lookup_Call(FL_OBJECT *obj, long data)
   char buf[12];
   keyid=PGP_Lookup_PGPId(fl_get_input(amod_obj->AMOD_Input),amod_obj->addrmod);
   if (keyid) {
-    sprintf(buf,"0x%X",keyid);
+    snprintf(buf, sizeof(buf), "0x%X", keyid);
     fl_set_input(amod_obj->AMOD_PGP_Input,buf);
   }
 }
@@ -2294,7 +2295,7 @@ AL_Lookup_Call(FL_OBJECT *obj, long data)
   char buf[12];
   keyid=PGP_Lookup_PGPId(fl_get_input(alias_obj->AL_Input),alias_obj->aliased);
   if (keyid) {
-    sprintf(buf,"0x%X",keyid);
+    snprintf(buf, sizeof(buf), "0x%X", keyid);
     fl_set_input(alias_obj->AL_InputPGP,buf);
   }
 }
