@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996,1997 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: font.c,v 2.0 1997/09/23 15:55:00 gena Exp $
+/* $Id: font.c,v 2.6 1998/04/21 14:03:06 gena Exp $
  */
 
 #include <fmail.h>
@@ -37,7 +37,7 @@ static FL_OBJECT
 	*bg_obj,
 	*fg_obj;
 
-extern FL_OBJECT *b_folders, *b_messages;
+extern FL_OBJECT *b_folders, *b_messages, *msgpane;
 static int ready = 0;
 static int nfonts = 0;
 static FD_config_intern *charset_obj;
@@ -53,6 +53,7 @@ typedef struct _size_el {
 #define MENU 	0x02
 #define FONT 	0x03
 #define	XFDEF	0x04
+#define	XTEXT	0x05
 #define	NO_SIZE		0x10
 #define	NO_STYLE	0x20
 #define	NO_BGCOL	0x40
@@ -67,7 +68,7 @@ struct _size_el elements[MAX_ELEMENTS] = {
 { "ChooseFolder", "Folder chooser",NULL,       NO_TYPE, FL_NORMAL_SIZE, FL_FIXED_STYLE, FL_COL1, FL_YELLOW },
 { "AddrBook",	"Address book",    NULL,       NO_TYPE, FL_NORMAL_SIZE, FL_FIXED_STYLE, FL_COL1, FL_YELLOW },
 { "Signature",	"Signature editor",NULL,       NO_TYPE, FL_NORMAL_SIZE, FL_FIXED_STYLE, FL_COL1, FL_BLACK },
-{ "View",	"Message viewer",  NULL,       NO_TYPE, FL_NORMAL_SIZE, FL_NORMAL_STYLE, FL_COL1, FL_BLACK },
+{ "View",	"Message viewer",  &msgpane,   XTEXT, FL_NORMAL_SIZE, FL_NORMAL_STYLE, FL_COL1, FL_BLACK },
 { "Edit",	"Message editor",  NULL,       NO_TYPE, FL_NORMAL_SIZE, FL_NORMAL_STYLE, FL_COL1, FL_BLACK },
 { "InputFont",	"Input box font size",  NULL,       XFDEF|NO_STYLE|NO_BGCOL|NO_FGCOL|N_RESTART, FL_DEFAULT_SIZE, FL_NORMAL_STYLE, FL_WHITE, FL_BLACK },
 { "LabelFont",	"Label font size",  NULL,       XFDEF|NO_STYLE|NO_BGCOL|NO_FGCOL|N_RESTART, FL_DEFAULT_SIZE, FL_NORMAL_STYLE, FL_WHITE, FL_BLACK },
@@ -91,7 +92,7 @@ int i;
 char fname[16];
 
   for (i = 16; i < 32; i++) {
-	sprintf(fname, "Font%d", i);
+	snprintf(fname, sizeof(fname), "Font%d", i);
 	if (cfg_exist(fname)) 	{
 	  if (fl_set_font_name(i, b_getcfg_str(conf_name, fname, "")) < 0) {
 		display_msg(MSG_LOG, "font", "font %s can not be loaded", b_getcfg_str(conf_name, fname, ""));
@@ -106,7 +107,7 @@ void
 restore_charsets()
 {
 int k = 0;
-char cname[32], aname[16], adescr[16], *str;
+char cname[MAX_CHARSET_NAME+10], aname[MAX_CHARSET_NAME], adescr[MAX_CHARSET_NAME], *str;
 
  while (supp_charsets[k].charset_code != CHAR_UNKNOWN) 	{
 	if (is_charset_alias(supp_charsets[k].charset_name)) {
@@ -114,19 +115,19 @@ char cname[32], aname[16], adescr[16], *str;
 		continue;
 		}
 
-	sprintf(cname, "%sFStyle", supp_charsets[k].charset_name);
+	snprintf(cname, sizeof(cname), "%sFStyle", supp_charsets[k].charset_name);
 	set_charset_style(supp_charsets[k].charset_name, b_getcfg_int(conf_name, cname, supp_charsets[k].font_style));
 
-	sprintf(cname, "%sFSize", supp_charsets[k].charset_name);
+	snprintf(cname, sizeof(cname), "%sFSize", supp_charsets[k].charset_name);
 	set_charset_size(supp_charsets[k].charset_name, b_getcfg_int(conf_name, cname, supp_charsets[k].font_size));
 
-	sprintf(cname, "%sAlias", supp_charsets[k].charset_name);
+	snprintf(cname, sizeof(cname), "%sAlias", supp_charsets[k].charset_name);
 	if (cfg_exist(cname)) 	{
 	 str = b_getcfg_str(conf_name, cname, "");
 	 while (1) {
 	  aname[0] = '\0';
 	  adescr[0] = '\0';
-	  if (sscanf(str, "%12s %12s", aname, adescr) != 2)
+	  if (sscanf(str, "%24s %24s", aname, adescr) != 2)
 		break;
 	  add_charset(aname, adescr, supp_charsets[k].charset_code);
 	  if ((str = strchr(str, ' ')) == NULL)
@@ -180,9 +181,9 @@ char *str, fname[16];
 		}
 					 }
 
-  if (i < 8) 	{
+/*if (i < 8) 	{
 	display_msg(MSG_WARN, "Invalid font", "%s", str);
-	return;	}
+	return;	} */
 
   if (fl_set_font_name(nfonts, str) < 0) {
 	display_msg(MSG_WARN, "Invalid font", "%s", str);
@@ -207,7 +208,7 @@ int i;
 
  i = 0;
  while (strlen(elements[i].el_name) > 2) {
-	sprintf(fname, "%sFStyle", elements[i].el_name);
+	snprintf(fname, sizeof(fname), "%sFStyle", elements[i].el_name);
 	
 	if (b_getcfg_int(conf_name, fname, elements[i].def_style) == (nfonts - 1)) {
 		display_msg(MSG_WARN, "FONT", "The font is in use. Can not delete");
@@ -247,7 +248,7 @@ int i, type, typef;
 	fl_deactivate_object(sizeobj);
  else	{
 	fl_activate_object(sizeobj);
-	sprintf(buf, "%sFSize", elements[el_num].el_name);
+	snprintf(buf, sizeof(buf), "%sFSize", elements[el_num].el_name);
 	size = b_getcfg_int(conf_name,buf,size);
 	if (size == 0)
 		size = FL_DEFAULT_SIZE;
@@ -257,17 +258,20 @@ int i, type, typef;
 	fl_deactivate_object(styleobj);
  else 	{
 	fl_activate_object(styleobj);
-	sprintf(buf, "%sFStyle", elements[el_num].el_name);
+	snprintf(buf, sizeof(buf), "%sFStyle", elements[el_num].el_name);
 	style = b_getcfg_int(conf_name,buf,style);
 	}
 
  if (typef & NO_BGCOL)
 	fl_deactivate_object(bg_obj);
  else	{
-   if ((type == BROWSER) && (elements[el_num].el_obj))
+   if ((type == BROWSER) && elements[el_num].el_obj && *elements[el_num].el_obj)
 	bgcol = (*elements[el_num].el_obj)->col1;
+   else
+   if ((type == XTEXT) && elements[el_num].el_obj && *elements[el_num].el_obj)
+	bgcol = (*elements[el_num].el_obj)->col2;
 
-	sprintf(buf, "%sBgCol", elements[el_num].el_name);
+	snprintf(buf, sizeof(buf), "%sBgCol", elements[el_num].el_name);
 	bgcol = b_getcfg_int(conf_name,buf,bgcol);
 
 	fl_activate_object(bg_obj);
@@ -276,10 +280,13 @@ int i, type, typef;
  if (typef & NO_FGCOL)
 	fl_deactivate_object(fg_obj);
  else	{
-   if ((type == BROWSER) && (elements[el_num].el_obj))
+   if ((type == BROWSER) && elements[el_num].el_obj && *elements[el_num].el_obj)
 	fgcol = (*elements[el_num].el_obj)->col2;
+   else
+   if ((type == XTEXT) && elements[el_num].el_obj && *elements[el_num].el_obj)
+	fgcol = (*elements[el_num].el_obj)->col1;
 
-	sprintf(buf, "%sFgCol", elements[el_num].el_name);
+	snprintf(buf, sizeof(buf), "%sFgCol", elements[el_num].el_name);
 	fgcol = b_getcfg_int(conf_name,buf,fgcol);
 
 	fl_activate_object(fg_obj);
@@ -316,7 +323,8 @@ int i, type, typef;
  return;
 }
 
-void color_cb(FL_OBJECT *obj, long arg)
+void
+color_cb(FL_OBJECT *obj, long arg)
 {
 char buf[64];
 FL_COLOR col = fl_show_colormap( arg ? textobj->col1 : textobj->lcol); 
@@ -333,19 +341,24 @@ int num = fl_get_browser(placeobj) - 1, type;
  	display_size(num);
 	return;    }
 
- sprintf(buf, "%sBgCol", elements[num].el_name);
+ snprintf(buf, sizeof(buf), "%sBgCol", elements[num].el_name);
  b_putcfg_int(conf_name, buf, textobj->col1);
- sprintf(buf, "%sFgCol", elements[num].el_name);
+ snprintf(buf, sizeof(buf), "%sFgCol", elements[num].el_name);
  b_putcfg_int(conf_name, buf, textobj->lcol);
 
- if (type == BROWSER) {
+ if ((type == BROWSER) && elements[num].el_obj && *elements[num].el_obj) {
 	fl_activate_object(*elements[num].el_obj);
 	fl_set_object_color(*elements[num].el_obj, textobj->col1, textobj->lcol);
-	fl_redraw_object(*elements[num].el_obj);
-		     }
+	fl_redraw_object(*elements[num].el_obj);			 }
+ else
+ if ((type == XTEXT) && elements[num].el_obj && *elements[num].el_obj)	{
+	fl_activate_object(*elements[num].el_obj);
+	fl_set_textedit_color(*elements[num].el_obj, textobj->col1, textobj->lcol, -1, 1);
+									}
 
  display_size(num);
 
+ return;
 }
 
 void place_cb(FL_OBJECT *obj, long arg)
@@ -360,7 +373,7 @@ int num = fl_get_browser(placeobj) - 1;
 int style = fl_get_browser(obj) - 1, type;
 
  type = elements[num].type & 0x0f;
- sprintf(buf, "%sFStyle", elements[num].el_name);
+ snprintf(buf, sizeof(buf), "%sFStyle", elements[num].el_name);
  b_putcfg_int(conf_name, buf, style);
 
  if (elements[num].type & N_RESTART)
@@ -369,16 +382,21 @@ int style = fl_get_browser(obj) - 1, type;
  if (type == FONT)
 	set_charset_style(elements[num].el_name, style);
 
- if (type == BROWSER) 	{
+ if ((type == BROWSER) && elements[num].el_obj && *elements[num].el_obj) {
 	fl_set_browser_fontstyle(*elements[num].el_obj, style);
 	fl_set_object_lstyle(*elements[num].el_obj, style);
-	fl_redraw_object(*elements[num].el_obj);
-			}
+	fl_redraw_object(*elements[num].el_obj);			 }
+ else
+ if ((type == XTEXT) && elements[num].el_obj && *elements[num].el_obj)	{
+	fl_set_textedit_fontstyle(*elements[num].el_obj, style);
+	fl_set_object_lstyle(*elements[num].el_obj, style);
+	fl_redraw_object(*elements[num].el_obj);			}
 
  display_size(num);
 }
 
-void size_cb(FL_OBJECT *obj, long arg)
+void
+size_cb(FL_OBJECT *obj, long arg)
 {
 char buf[64];
 int num = fl_get_browser(placeobj) - 1;
@@ -401,7 +419,7 @@ int size = FL_DEFAULT_SIZE, type;
  if (type == FONT)
 	set_charset_size(elements[num].el_name, size);
 
- sprintf(buf, "%sFSize", elements[num].el_name);
+ snprintf(buf, sizeof(buf), "%sFSize", elements[num].el_name);
  b_putcfg_int(conf_name, buf, size);
 
  if (elements[num].type & N_RESTART)
@@ -409,12 +427,17 @@ int size = FL_DEFAULT_SIZE, type;
 
  display_size(num);
 
- if (type == BROWSER) 	{
+ if ((type == BROWSER) && elements[num].el_obj && *elements[num].el_obj) {
 	fl_set_browser_fontsize(*elements[num].el_obj, size);
 	fl_set_object_lsize(*elements[num].el_obj, size);
-	fl_redraw_object(*elements[num].el_obj);
-			}
+	fl_redraw_object(*elements[num].el_obj);			 }
+ else
+ if ((type == XTEXT) && elements[num].el_obj && *elements[num].el_obj)	{
+	fl_set_textedit_fontsize(*elements[num].el_obj, size);
+	fl_set_object_lsize(*elements[num].el_obj, size);
+	fl_redraw_object(*elements[num].el_obj);			}
 
+ return;
 }
 
 void create_form_form(void)
@@ -423,7 +446,7 @@ void create_form_form(void)
 
   form = fl_bgn_form(FL_NO_BOX,550.0,300.0);
   fl_add_box(FL_FLAT_BOX,0,0,550.0,300.0,"");
-  textobj = obj = fl_add_text(FL_NORMAL_TEXT,10.0,10.0,530.0,100.0,"XFMail\nCopyright (c) 1995,1996,1997 by\n Gennady B. Sorokopud\n\374\314\305\313\324\322\317\316\316\301\321 \320\317\336\324\301\n\351\360\345\370\350\367\354\340 \370\340\345\343");
+  textobj = obj = fl_add_text(FL_NORMAL_TEXT,10.0,10.0,530.0,100.0,"XFMail\nCopyright (c) 1995-1998 by\n Gennady B. Sorokopud\n\374\314\305\313\324\322\317\316\316\301\321 \320\317\336\324\301\n\351\360\345\370\350\367\354\340 \370\340\345\343");
     fl_set_object_boxtype(obj,FL_FRAME_BOX);
     fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
     fl_set_object_color(obj,FL_INDIANRED,FL_YELLOW);
@@ -534,7 +557,7 @@ int sep;
    elements[i].def_size = supp_charsets[k].font_size;
    elements[i].def_bg = FL_COL1;
    elements[i].def_fg = FL_BLACK;
-   sprintf(chdescr, "%s (%s)", elements[i].el_name, elements[i].el_descr);
+   snprintf(chdescr, sizeof(chdescr), "%s (%s)", elements[i].el_name, elements[i].el_descr);
    fl_add_browser_line(placeobj, chdescr);
    i++;
    k++;
@@ -595,7 +618,7 @@ void CHSET_Select_Call(FL_OBJECT *obj, long param)
 
 void CHSET_AAlias_Call(FL_OBJECT *obj, long param)
 {
-char *p, aname[13], adescr[13], buf[127];
+char *p, aname[MAX_CHARSET_NAME + 1], adescr[MAX_CHARSET_NAME + 1], buf[127];
 int i = fl_get_browser(charset_obj->CHSET_Select);
 
  if (i < 1)
@@ -606,17 +629,17 @@ int i = fl_get_browser(charset_obj->CHSET_Select);
 	display_msg(MSG_WARN, "add charset alias", "Can not make alias to alias");
 	return;						}
 
- sprintf(buf, "New alias name of charset %s:", supp_charsets[i].charset_name);
+ snprintf(buf, sizeof(buf), "New alias name of charset %s:", supp_charsets[i].charset_name);
  p=(char *)fl_show_input(buf, "");
  if (!p || !*p)
 	return;
 
- if (strlen(p) > 12) {
-	display_msg(MSG_WARN, "add charset alias", "charset name can not be longer then 12 characters");
+ if (strlen(p) > MAX_CHARSET_NAME) {
+	display_msg(MSG_WARN, "add charset alias", "charset name can not be longer then %d characters", MAX_CHARSET_NAME);
 	return;	     }
 
- strncpy(aname, p, 12);
- aname[13] = '\0';
+ strncpy(aname, p, MAX_CHARSET_NAME);
+ aname[MAX_CHARSET_NAME] = '\0';
  p = aname;
  while (*p != '\0') {
   if (!isgraph(*p)) {
@@ -628,12 +651,12 @@ int i = fl_get_browser(charset_obj->CHSET_Select);
  if (!p || !*p)
 	return;
 
- if (strlen(p) > 12) {
-	display_msg(MSG_WARN, "add charset alias", "charset description can not be longer then 12 characters");
+ if (strlen(p) > MAX_CHARSET_NAME) {
+	display_msg(MSG_WARN, "add charset alias", "charset description can not be longer then %d characters", MAX_CHARSET_NAME);
 	return;	     }
 
- strncpy(adescr, p, 12);
- adescr[13] = '\0';
+ strncpy(adescr, p, MAX_CHARSET_NAME);
+ adescr[MAX_CHARSET_NAME] = '\0';
  p = adescr;
  while (*p != '\0') {
   if (!isgraph(*p) || (*p == ' ')) {
@@ -682,7 +705,7 @@ int i, k;
  fl_clear_browser(charset_obj->CHSET_Select);
  while (supp_charsets[i].charset_code != CHAR_UNKNOWN) 	{
   k = is_charset_alias(supp_charsets[i].charset_name);
-  sprintf(buf, "%-12.12s %-13.13s %-11.11s %-4d %-2d",
+  snprintf(buf, sizeof(buf), "%-12.12s %-13.13s %-11.11s %-4d %-2d",
    supp_charsets[i].charset_name,
    supp_charsets[i].charset_descr ?  supp_charsets[i].charset_descr : "",
    k > 0 ? supp_charsets[k - 1].charset_name : "",
@@ -709,7 +732,7 @@ charset_conf(int set_default, FD_config_intern *form)
 void
 handle_charsets_input(FD_config_intern *form)
 { 
-char buf[255], aname[32];
+char buf[255], aname[MAX_CHARSET_NAME + 10];
 int i, k;
 
  charset_obj = form;
@@ -735,7 +758,7 @@ int i, k;
 									}
     k++;
 							 }
-   sprintf(aname, "%sAlias", supp_charsets[i].charset_name);
+   snprintf(aname, sizeof(aname), "%sAlias", supp_charsets[i].charset_name);
    if (*buf)
 	b_putcfg_str(conf_name, aname, buf);
    else	{
