@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -21,7 +21,7 @@
  *
  */
 
-/* $Id: pgp.c,v 2.8 1997/12/08 16:21:03 gena Exp $
+/* $Id: pgp.c,v 2.17 1998/04/13 13:35:18 gena Exp $
  */
 
 #include <fmail.h>
@@ -38,6 +38,12 @@ static int pgpext;		/* PGP exit status */
 static FD_config_pgp *pgp_obj;
 static FD_Lookup_PGPId *form_lookup_pgpid = NULL;
 
+#define	PGP_DEFAULT_VERS	263	/* use PGP-2.6.3 by default */
+#define	PGP_TIMEOUT		60	/* PGP process timeout (in seconds) */
+
+static char *get_pgp263_command_line(int, struct pgpargs *, char *, int);
+static char *get_pgp500_command_line(int, struct pgpargs *, char *, int);
+
 typedef struct _pgp_keylist {
 char *userid;
 unsigned long keyid;
@@ -71,7 +77,12 @@ cb_input_passphrase(FL_OBJECT *obj,long param)
 void
 PGP_Timer_Call(FL_OBJECT *obj,long param)
 {
+char *p;
+
  if (phrase) {
+	p = phrase;
+	while (*p)
+	 *p++ = '\0';
 	free(phrase);
 	phrase = NULL;
 	     }
@@ -116,18 +127,23 @@ char* input_passphrase()
 {
 char *rphrase;
 FD_Passphrase *pass;
+int ptimer = b_getcfg_int(conf_name, "pgptimer", 120);
 
  if (phrase)
 	goto retp;
 
- pass=create_form_Passphrase();
+ pass = create_form_Passphrase();
 
  fl_show_form(pass->Passphrase,FL_PLACE_MOUSE,FL_TRANSIENT,"Input Passphrase");
  fl_do_only_forms();
  phrase = strdup((char *)fl_get_input(pass->Input_Passphrase));
  scramble(phrase);
- if (pgp_timer)
-	fl_set_timer(pgp_timer, b_getcfg_int(conf_name, "pgptimer", 120)*60+1); 
+ if (pgp_timer) {
+	if (ptimer == PGP_MAX_TIMER)
+	 fl_set_timer(pgp_timer, 0); 
+	else
+	 fl_set_timer(pgp_timer, ptimer*60);
+		}
  fl_hide_form(pass->Passphrase);
  fl_free_form(pass->Passphrase);
  fl_free(pass);
@@ -136,6 +152,9 @@ retp:
  rphrase = strdup(phrase);
  scramble(rphrase);
 
+ if (ptimer == 0)
+	PGP_Timer_Call(NULL, 0);
+
  return rphrase;
 }      
 
@@ -156,7 +175,7 @@ pgp_fetch_key_from_http(char *host, unsigned int keyid)
     return -1;
 
   /* HKP (Horowitz Key Protocol) */
-  sprintf(buf, "GET /pks/lookup?op=get&exact=on&search=0x%X HTTP/1.0\n", keyid);
+  snprintf(buf, sizeof(buf), "GET /pks/lookup?op=get&exact=on&search=0x%X HTTP/1.0\n", keyid);
   strcat(buf, "User-Agent: XFMail (Unix)\n\r\n\r");
   if (send(sock, buf, strlen(buf), 0) == -1)
     return -1;
@@ -169,7 +188,7 @@ pgp_fetch_key_from_http(char *host, unsigned int keyid)
 
   *pbuf = '\0';
   do {
-    if ((n = my_check_io_forms(sock, 0)) < 0)	{
+    if ((n = my_check_io_forms(sock, 0, SOCKET_TIMEOUT)) < 0)	{
 	if (n == -2)
 		display_msg(MSG_WARN, "PGP keyserver (HKP)","transfer aborted");
 	res = -1;
@@ -212,25 +231,45 @@ pgp_fetch_key_from_http(char *host, unsigned int keyid)
    if (pgp_action(kfile, EXTKEY, NULL) == -1) {
 	unlink(kfile);  
 	return -1;			      }
+   display_msg(MSG_MSG, "PGP keyserver (HKP)", "Fetched and extracted PGP public key 0x%X from %s", keyid, host);
 		}
 
-  display_msg(MSG_MSG, "PGP keyserver (HKP)", "Fetched PGP public key 0x%X from %s", keyid, host);
   unlink(kfile);  
   return res;
 }
 
 static char*
 get_pgp_command_line(int mode, struct pgpargs *pgpargs, char *inbuf, int try)
+{
+  switch (b_getcfg_int(conf_name, "pgpversion", PGP_DEFAULT_VERS))	{
+   case 263:
+	return get_pgp263_command_line(mode, pgpargs, inbuf, try);
+   break;
+
+   case 500:
+	return get_pgp500_command_line(mode, pgpargs, inbuf, try);
+   break;
+
+   default:
+	display_msg(MSG_WARN, "PGP", "Unknown PGP version: %d", b_getcfg_int(conf_name, "pgpversion", PGP_DEFAULT_VERS));
+   break;
+							}
+
+  return NULL;
+}
+
+static char*
+get_pgp500_command_line(int mode, struct pgpargs *pgpargs, char *inbuf, int try)
 {
   char args[1024];
-  char pubring[255]="";
   const char pgpparm[]="+language=en +verbose=1 +clearsig=on";
+  char pubring[255]="";
   char *pgpprg;
   struct _mail_addr *addr;
   struct _abook_entry *entry;
 
   if (!(try&NORMAL) && (try&KEYRING))
-    sprintf(pubring, "+pubring=%s", b_getcfg_str(conf_name,"pgpkeyring",""));
+    snprintf(pubring, sizeof(pubring), "+pubring=%s", b_getcfg_str(conf_name,"pgpkeyring",""));
 
   pgpprg = strdup(b_getcfg_str(conf_name, "pgp", _PATH_PGP));
   if (pgpargs && (pgpargs->userid == NULL))	{
@@ -250,32 +289,95 @@ get_pgp_command_line(int mode, struct pgpargs *pgpargs, char *inbuf, int try)
 						}
 
   if ((mode&ENCODE) && (mode&SIGN))
-    sprintf(args, "%s %s +batchmode -fate %s -su %s ", pgpprg, pgpparm, pgpargs->recp, pgpargs->userid);
+    snprintf(args, sizeof(args), "%se %s +batchmode +NoBatchInvalidKeys=off -fat %s -su %s ", pgpprg, pgpparm, pgpargs->recp, pgpargs->userid);
   else
   if (mode&ENCODE)
-    sprintf(args, "%s %s -fate %s", pgpprg, pgpparm, pgpargs->recp);
+    snprintf(args, sizeof(args), "%se %s +batchmode +NoBatchInvalidKeys=off -fat %s", pgpprg, pgpparm, pgpargs->recp);
   else
   if (mode&SIGN)
-    sprintf(args, "%s %s +batchmode -fatsu %s ", pgpprg, pgpparm, pgpargs->userid);
+    snprintf(args, sizeof(args), "%ss %s +batchmode -fatu %s ", pgpprg, pgpparm, pgpargs->userid);
 
   if (mode&SIGNFL)	{
-    sprintf(args, "%s %s +batchmode -fatbsu %s", pgpprg, pgpparm, pgpargs->userid);
+    snprintf(args, sizeof(args), "%ss %s +batchmode -fatbu %s", pgpprg, pgpparm, pgpargs->userid);
+    if (pgpargs->recp == NULL)	{
+	free(pgpprg);
+	return NULL;		}
+			}
+
+  if (mode&(DECODE|VERIFY))
+    snprintf(args, sizeof(args), "%sv %s +batchmode %s -f", pgpprg, pgpparm, pubring);
+
+  if (mode&EXTKEY)
+    snprintf(args, sizeof(args), "%sk %s +batchmode -a %s", pgpprg, pgpparm, inbuf);
+
+  if (mode&ADDKEY)
+    snprintf(args, sizeof(args), "%sk %s -xa %s", pgpprg, pgpparm, pgpargs->recp);
+
+  if (mode&VRFYFL)
+    snprintf(args, sizeof(args), "%sv %s +batchmode %s %s -o %s", 
+            pgpprg, pgpparm, pubring, pgpargs->recp, inbuf);
+
+  free(pgpprg);
+  return strdup(args);
+}
+
+static char*
+get_pgp263_command_line(int mode, struct pgpargs *pgpargs, char *inbuf, int try)
+{
+  char args[1024];
+  char pubring[255]="";
+  const char pgpparm[]="+language=en +verbose=1 +clearsig=on";
+  char *pgpprg;
+  struct _mail_addr *addr;
+  struct _abook_entry *entry;
+
+  if (!(try&NORMAL) && (try&KEYRING))
+    snprintf(pubring, sizeof(pubring), "+pubring=%s", b_getcfg_str(conf_name,"pgpkeyring",""));
+
+  pgpprg = strdup(b_getcfg_str(conf_name, "pgp", _PATH_PGP));
+  if (pgpargs && (pgpargs->userid == NULL))	{
+    if ((mode & (SIGN|SIGNFL)) && pgpargs->msg && pgpargs->msg->header->From) {
+      addr = pgpargs->msg->header->From;
+      if (addr->pgpid == NULL)	{
+        if ((entry = find_addr(addr)) != NULL)
+		addr = entry->addr;
+				}
+      if (addr->pgpid && *addr->pgpid && strncmp(addr->pgpid, "0x", 2))
+	display_msg(MSG_WARN, "PGP", "Invalid PGP Id: %s", addr->pgpid);
+      else
+        pgpargs->userid = addr->pgpid;
+									      }
+    if (pgpargs->userid == NULL)
+	pgpargs->userid = b_getcfg_str(conf_name, "pgpuser", user_n);
+						}
+
+  if ((mode&ENCODE) && (mode&SIGN))
+    snprintf(args, sizeof(args), "%s %s +batchmode -fate %s -su %s ", pgpprg, pgpparm, pgpargs->recp, pgpargs->userid);
+  else
+  if (mode&ENCODE)
+    snprintf(args, sizeof(args), "%s %s -fate %s", pgpprg, pgpparm, pgpargs->recp);
+  else
+  if (mode&SIGN)
+    snprintf(args, sizeof(args), "%s %s +batchmode -fatsu %s ", pgpprg, pgpparm, pgpargs->userid);
+
+  if (mode&SIGNFL)	{
+    snprintf(args, sizeof(args), "%s %s +batchmode -fatbsu %s", pgpprg, pgpparm, pgpargs->userid);
     if (pgpargs->recp == NULL)	{
 	free(pgpprg);
 	return NULL;		}
 			}
     
   if (mode&(DECODE|VERIFY))
-    sprintf(args, "%s %s +batchmode %s -f", pgpprg, pgpparm, pubring);
+    snprintf(args, sizeof(args), "%s %s +batchmode %s -f", pgpprg, pgpparm, pubring);
 
   if (mode&EXTKEY)
-    sprintf(args, "%s %s +batchmode -f -ka", pgpprg, pgpparm);
+    snprintf(args, sizeof(args), "%s %s +batchmode -f -ka", pgpprg, pgpparm);
 
   if (mode&ADDKEY)
-    sprintf(args, "%s %s -f -kxa %s", pgpprg, pgpparm, pgpargs->recp);
+    snprintf(args, sizeof(args), "%s %s -f -kxa %s", pgpprg, pgpparm, pgpargs->recp);
 
   if (mode&VRFYFL)
-    sprintf(args, "%s %s +batchmode %s %s %s", 
+    snprintf(args, sizeof(args), "%s %s +batchmode %s %s %s", 
             pgpprg, pgpparm, pubring, pgpargs->recp, inbuf);
 
   free(pgpprg);
@@ -298,11 +400,12 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
   char nfile[64];
   char *args, *p, *q, *r;
   int n, move_result = 0, errfd;
-  int try=NORMAL, tries=0;
+  int try = NORMAL, tries = 0;
   unsigned int keyid = 0;
   struct _proc_info pinfo;
   FILE *file;
   struct stat sb;
+  int pgpvers = b_getcfg_int(conf_name, "pgpversion", PGP_DEFAULT_VERS);
 
   strcpy(nfile, (mode&SIGNFL) ? pgpargs->recp : get_temp_file("pgp"));
 
@@ -312,7 +415,7 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
   init_pinfo(&pinfo);
   pinfo.handle = pgp_cleanup;
   pinfo.init = pgp_init;
-  pinfo.fd_in[0]=pinfo.fd_out[0]=pinfo.fd_err[0]=0;
+  pinfo.fd_in[0] = pinfo.fd_out[0] = pinfo.fd_err[0] = 0;
 
   /* init pgp exit status */
   pgpext = -100;
@@ -327,7 +430,6 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
 						 }
 
   if (!(args = get_pgp_command_line(mode, pgpargs, inbuf, try))) {
-	/* if we can not find signature during decode - still procede */
 	if (mode & DECODE)
 		break;
 	unlink(nfile);
@@ -362,8 +464,12 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
     write(pinfo.fd_in[1],pgpargs->passphrase,strlen(pgpargs->passphrase));
   write(pinfo.fd_in[1],"\n",1);
 
+  if ((mode&EXTKEY) &&
+	(pgpvers == 500))
+	errfd = pinfo.fd_err[0];
+  else
   if (mode&VRFYFL)
-	errfd = pinfo.fd_out[0];
+	errfd = (pgpvers == 263) ? pinfo.fd_out[0] : pinfo.fd_err[0];
   else 	{
    if ((file = fopen(inbuf,"r")) != NULL) {
     while((n = fread(buf,1,1023,file)) > 0)
@@ -373,8 +479,19 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
     fclose(file);			  }
 
   if ((file=fopen(nfile,"w")) != NULL) 	{
-   while((n=read(pinfo.fd_out[0],buf,1023))>0)
-    fwrite(buf,1,n,file);
+   while (1)	{
+    if ((n = my_check_io_forms(pinfo.fd_out[0], 0, PGP_TIMEOUT)) < 0)	{
+      if (n == -3)	{
+	display_msg(MSG_WARN, "PGP Timeout", "Terminating process");
+	if (pinfo.pid > 0)
+		kill(pinfo.pid, SIGKILL);
+			}
+      break;								}
+    if ((n = read(pinfo.fd_out[0],buf,1023)) > 0)
+	fwrite(buf,1,n,file);
+    else
+	break;
+		}
    close(pinfo.fd_out[0]);
    pinfo.fd_out[0] = -1;
    fclose(file);			}
@@ -382,13 +499,17 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
   errfd = pinfo.fd_err[0];
 	}
 
-  while((n=read(errfd,buf,1023)) > 0) {
+  while((n = read(errfd, buf, 1023)) > 0) {
     buf[n] = '\0';
-    p = strstr(buf, "Current");
-    if (p)
+    if (pgpvers == 500)
+	p = buf;
+    else	{
+     p = strstr(buf, "Current");
+     if (p)
 	p = strchr(p, '\n');
-    if (p && *p)
+     if (p && *p)
 	p++;
+		}
 
     if (p && *p) {
      while ((q = strchr(p, '\n')) != NULL) {
@@ -398,11 +519,12 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
 	*q = '\n';
 	p = q + 1;
 					   }
-     if (strlen(p))
+     if (*p)
       display_msg(MSG_LOG, "PGP", p);
 		}
 
     if((p=strstr(buf,"Good signature")) ||
+	(p=strstr(buf,"BAD signature")) ||
        (p=strstr(buf,"WARNING:"))) {
 	if (strlen(p) > 255)
 		p[255] = '\0';
@@ -415,27 +537,48 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
 	else
 		display_msg(MSG_MSG,"PGP messsage (see Log for more details)",p);
 				  }
+
     if((p=strstr(buf,"ERROR:")) ||
        (p=strstr(buf,"Error:")) ||
        (p=strstr(buf,"You do not have the secret key")) ||
+       (p=strstr(buf,"No encryption keys found")) ||
        (p=strstr(buf,"Key matching expected")) ||
+       (p=strstr(buf,"unknown keyid")) ||
        (p=strstr(buf,"Encryption error")) ) {
 	/* if we decoding encoded and signed message */
         /* but no public key could be found - still show the results */
-	if (!(mode&DECODE) && !strstr(buf,"Key matching expected"))
-        	move_result=0;
+	if (!(mode&DECODE) &&
+		(!strstr(buf,"Key matching expected") ||
+			!strstr(buf,"unknown keyid")))
+        	move_result = 0;
 
-	if (strstr(buf,"Key matching expected")) {
-		char keystr[9];
+	/* can not find encryption key for one of the recipients */
+	if ((mode&ENCODE) &&
+		strstr(buf,"No encryption keys found"))
+		move_result = 0;
+
+	if (strstr(buf,"Key matching expected") ||
+		strstr(buf,"unknown keyid")) {
+		char *keystr;
 		if (!tries)	{
 			if (b_getcfg_int(conf_name,"pgpusekeyring",0))
 				try |= KEYRING;
 			if (b_getcfg_int(conf_name,"pgpusehttp",0))
 				try |= HTTP;
 				}
-		strncpy(keystr,p+29,8);
-		keystr[8]='\0';
-		keyid=strtoul(keystr,NULL,16);
+		if ((keystr = strstr(p, "0x")) == NULL)
+			keystr = (pgpvers == 263) ? p + 29 : p + 15;
+
+		if (strncmp(keystr, "0x", 2))	{
+		   buf[0] = '\0';
+		   if ((n = read(errfd, buf, 1023)) > 0)
+			buf[n] = '\0';
+		   if ((keystr = strstr(buf, "0x")) == NULL)
+			keystr = "";
+						}
+
+		if ((keyid = strtoul(keystr,NULL,16)) == 0)
+			try &= ~HTTP;
 		if ((tries == 0) &&
 			!(try & KEYRING) &&
 			!(try & HTTP))
@@ -466,11 +609,11 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
 		display_msg(MSG_WARN,"PGP Error (See Log for more details)",p);
 		}
 	      }
-	/* forget the pass phrase */
-      if((mode&(DECODE|SIGN|SIGNFL)) && p &&
-	strstr(p, "pass phrase"))
-	PGP_Timer_Call(NULL, 0);
     }
+	/* forget the pass phrase */
+      if((mode&(DECODE|SIGN|SIGNFL)) &&
+	strstr(buf, "pass phrase"))
+	PGP_Timer_Call(NULL, 0);
   }
   
   close(pinfo.fd_err[0]);
@@ -480,7 +623,7 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
 	close(pinfo.fd_in[1]);
 
   check_extprocs();
-  for (n=0; (pgpext == -100) && (n < 5); n++) 	{
+  for (n=0; (pgpext == -100) && (n < PGP_TIMEOUT); n++) 	{
 	check_extprocs();
 	sleep(1);				}
 
@@ -506,6 +649,9 @@ pgp_action(char *inbuf, int mode, struct pgpargs *pgpargs)
     if (mode&SIGNFL)
 	return 0;
 
+    if (mode&EXTKEY)
+	return 0;
+
     if (stat(nfile, &sb) == -1)
 	return -1;
 
@@ -568,7 +714,7 @@ int res;
 struct pgpargs pgpargs;
 
   init_pgpargs(&pgpargs);
-  pgpargs.passphrase=input_passphrase();
+  pgpargs.passphrase = input_passphrase();
  
   if ((res = pgp_action(file, DECODE, &pgpargs)) != 0)
         display_msg(MSG_WARN, "reply", "Failed to decode PGP message");
@@ -582,14 +728,21 @@ struct pgpargs pgpargs;
 void
 PGP_Conf_Lookup_CB(FL_OBJECT *obj, long data)  
 {
-  int keyid;
-  char buf[12];
-  keyid=PGP_Lookup_PGPId(b_getcfg_str(conf_name,"from",""),
-                         pgp_obj->config_pgp);
-  if (keyid) {
-    sprintf(buf,"0x%X",keyid);
+int keyid, ftype;
+char buf[12];
+struct _mail_addr *addr;
+
+  addr = get_address(b_getcfg_str(conf_name, "from", ""), ADDR_GET_FIRST);
+  keyid = PGP_Lookup_PGPId(addr ? get_charset_addr_line(NULL, addr,  &ftype) : user_n, pgp_obj->config_pgp);
+  if (addr)
+	discard_address(addr);
+
+  if (keyid)	{
+    snprintf(buf, sizeof(buf), "0x%X", keyid);
     fl_set_input(pgp_obj->PGP_User,buf);
-  }
+		}
+
+  return;
 }
 
 void
@@ -610,7 +763,7 @@ pgp_conf(int set_default, FD_config_pgp *form)
  fl_set_button(pgp_obj->PGP_Sign, b_getcfg_int_default(conf_name, "pgpsign", 0, set_default));
  fl_set_button(pgp_obj->PGP_Use_Keyring, b_getcfg_int_default(conf_name, "pgpusekeyring", 0, set_default));
  fl_set_button(pgp_obj->PGP_Use_HTTP, b_getcfg_int_default(conf_name, "pgpusehttp", 0, set_default));
- fl_set_counter_value(pgp_obj->PGP_Timer_Count, (double)b_getcfg_int_default(conf_name, "pgptimer", 120, set_default));
+ fl_set_button(pgp_obj->PGP_500, (b_getcfg_int_default(conf_name, "pgpversion", PGP_DEFAULT_VERS, set_default) == PGP_DEFAULT_VERS) ? 0 : 1);
 }
 
 void
@@ -625,7 +778,7 @@ handle_pgp_input(FD_config_pgp *form)
  b_putcfg_int(conf_name, "pgpsign", fl_get_button(pgp_obj->PGP_Sign));
  b_putcfg_int(conf_name, "pgpusekeyring", fl_get_button(pgp_obj->PGP_Use_Keyring));
  b_putcfg_int(conf_name, "pgpusehttp", fl_get_button(pgp_obj->PGP_Use_HTTP));
- b_putcfg_int(conf_name, "pgptimer", (int)fl_get_counter_value(pgp_obj->PGP_Timer_Count));
+ b_putcfg_int(conf_name, "pgpversion", fl_get_button(pgp_obj->PGP_500) ? 500 : 263);
 }
 
 int
@@ -651,13 +804,19 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
   unsigned long keyid;
   struct _pgp_keylist *keylist;
   FL_OBJECT *obj;
+  int pgpvers = b_getcfg_int(conf_name, "pgpversion", PGP_DEFAULT_VERS);
   
   if (form_lookup_pgpid)
     return 0;
 
   strcpy(keyfile, get_temp_file("pgpkeys"));
-  sprintf(buf, "%s +batchmode +language=en -kv",
+  if (pgpvers == 263)
+	snprintf(buf, sizeof(buf), "%s +batchmode +language=en -kv",
 				b_getcfg_str(conf_name, "pgp", _PATH_PGP));
+  else
+	snprintf(buf, sizeof(buf), "%sk +batchmode +language=en -l",
+				b_getcfg_str(conf_name, "pgp", _PATH_PGP));
+
   init_pinfo(&pinfo);
   pinfo.ofd = open(keyfile, O_WRONLY|O_CREAT, 00600);
   pinfo.wait = WAIT_BG;
@@ -676,7 +835,7 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
   check_extprocs();
   /* 
    * This routine heavily depends on the format of the output of
-   * "pgp +language=en -kv":
+   * "pgp +language=en -kv" (2.6.3) or "pgpk +language=en -l" (5.0)
    * 1. read all lines not beginning with "pub " (usually 7 lines)
    */
   if ((in = fopen(keyfile,"r")) == NULL) {
@@ -690,7 +849,8 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
       fclose(in);
       unlink(keyfile);
       return 0;			}
-  } while (strncmp(buf, "pub ", 4));
+  } while (strncmp(buf, "pub ", 4) &&
+	strncmp(buf, "sec  ", 5) && strncmp(buf, "sec+ ", 5));
 
   form_lookup_pgpid = create_form_Lookup_PGPId();
   fl_set_form_atclose(form_lookup_pgpid->Lookup_PGPId, cancel_form_lookup_cb, NULL);
@@ -719,11 +879,30 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
   keylist = (struct _pgp_keylist *)malloc(keyids_size * sizeof(struct _pgp_keylist));
   keyid = 0;
   do {
-    if (buf[10] != ' ')
-	keyid = strtoul(buf+10,NULL,16);
-    if ((userid = strchr(buf + 30,'\n')) != NULL)
+    userid = NULL;
+    if (pgpvers == 263)	{
+     if (buf[10] != ' ')
+	keyid = strtoul(buf+10, NULL, 16);
+     if ((userid = strchr(buf + 30,'\n')) != NULL)
       *userid = '\0';
-    userid = buf + 30;
+     userid = buf + 30;
+			}
+    else	{
+     if (!strncmp(buf, "pub  ", 5) ||
+		!strncmp(buf, "sec  ", 5) ||
+		!strncmp(buf, "sec+ ", 5))	{
+	keyid = 0;
+	keyid = strtoul(buf+10, NULL, 16);
+	continue;				}
+     if (!strncmp(buf, "sub  ", 5))
+	continue;
+     if (!strncmp(buf, "uid  ", 5))	{
+	if ((userid = strchr(buf + 5,'\n')) != NULL)
+		*userid = '\0';
+	userid = buf + 5;		}
+     else
+	keyid = 0;
+		}
     
     if (i == keyids_size)						{
       keyids_size += keyids_size;
@@ -732,6 +911,10 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
 
     if (keylist == NULL)
 	break;
+
+    if ((keyid == 0) || (userid == NULL))
+	continue;
+
     keylist[i].userid = strdup(userid);
     keylist[i].keyid = keyid;
     i++;
@@ -767,7 +950,7 @@ PGP_Lookup_PGPId(const char *name, FL_FORM *form)
         keyid = keylist[line-1].keyid;
         break;								}
       line = fl_get_browser(form_lookup_pgpid->PGP_Lookup_Browser);
-      sprintf(buf,"0x%X", (unsigned int)keylist[line-1].keyid);
+      snprintf(buf, sizeof(buf), "0x%X", (unsigned int)keylist[line-1].keyid);
       fl_set_object_label(form_lookup_pgpid->PGP_selected_id,buf);
       fl_set_timer(form_lookup_pgpid->PGP_Lookup_Timer, 0.4);
 							}
@@ -800,10 +983,10 @@ struct _mail_msg *msg;
 static char recp[1024];
 struct _mail_addr *ptr = NULL;
 struct _abook_entry *entry;
-int i;
+int i, pgpvers = b_getcfg_int(conf_name, "pgpversion", PGP_DEFAULT_VERS);
+
+  *recp = '\0';
 
-  *recp = '\0'; 
-   
   for (i = 0; i <= 1; i++) {
     if (i == 0)      
       ptr = msg->header->To;
@@ -815,16 +998,22 @@ int i;
          if ((entry->addr->pgpid) && strlen(entry->addr->pgpid)) {
            if ((strlen(entry->addr->pgpid) + strlen(recp) + 1) >= sizeof(recp))
              break;
+	   if (pgpvers == 500)
+            strcat(recp, "-r ");
            strcat(recp,entry->addr->pgpid);
            goto nextitem;
          }
      if (ptr->pgpid && strlen(ptr->pgpid)) {
        if ((strlen(ptr->pgpid) + strlen(recp) + 1) >= sizeof(recp))
         break;
+       if (pgpvers == 500)
+	strcat(recp, "-r ");
        strcat(recp,ptr->pgpid);
      } else {
        if ((strlen(ptr->addr) + strlen(recp) + 1) >= sizeof(recp))
         break;
+       if (pgpvers == 500)
+	strcat(recp, "-r ");
        strcat(recp,ptr->addr);
      }
 nextitem:
@@ -925,7 +1114,7 @@ FILE *efd;
  mime->flags |= ATTACHMENT;
  mime->flags &= ~TEXT_PART;
  mime->src_info = strdup(vfile);
- sprintf(buf, "%s/%s", mailcap[PGPENC_MAILCAP].type_text, mailcap[PGPENC_MAILCAP].subtype_text);
+ snprintf(buf, sizeof(buf), "%s/%s", mailcap[PGPENC_MAILCAP].type_text, mailcap[PGPENC_MAILCAP].subtype_text);
  replace_mime_field(mime, MIME_C_TYPE, buf);
 
  mime->mime_next = create_mime();
@@ -935,7 +1124,7 @@ FILE *efd;
  mime->flags |= ATTACHMENT;
  mime->flags &= ~TEXT_PART;
  mime->src_info = strdup(efile);
- sprintf(buf, "%s/%s", mailcap[BINARY_MAILCAP].type_text, mailcap[BINARY_MAILCAP].subtype_text);
+ snprintf(buf, sizeof(buf), "%s/%s", mailcap[BINARY_MAILCAP].type_text, mailcap[BINARY_MAILCAP].subtype_text);
  replace_mime_field(mime, MIME_C_TYPE, buf);
 
  if (update_mime(msg) == -1)	{
@@ -1042,7 +1231,7 @@ struct pgpargs pgpargs;
  mime->flags |= ATTACHMENT;
  mime->flags &= ~TEXT_PART;
  mime->src_info = strdup(vfile);
- sprintf(buf, "%s/%s", mailcap[PGPSIG_MAILCAP].type_text, mailcap[PGPSIG_MAILCAP].subtype_text);
+ snprintf(buf, sizeof(buf), "%s/%s", mailcap[PGPSIG_MAILCAP].type_text, mailcap[PGPSIG_MAILCAP].subtype_text);
  replace_mime_field(mime, MIME_C_TYPE, buf);
 
  mime1 = create_mime();
@@ -1056,7 +1245,7 @@ struct pgpargs pgpargs;
  if ((fld = find_field(msg, MIME_C_TYPE)) != NULL)
 	replace_mime_field(mime1, MIME_C_TYPE, fld->f_line);
  else	{
-	sprintf(buf, "%s: %s/%s", MIME_C_TYPE, mbody->mailcap->type_text,
+	snprintf(buf, sizeof(buf), "%s: %s/%s", MIME_C_TYPE, mbody->mailcap->type_text,
 		 mbody->mailcap->subtype_text);
 	replace_mime_field(mime1, MIME_C_TYPE, buf);
 	}
@@ -1066,7 +1255,7 @@ struct pgpargs pgpargs;
   if ((fld = find_field(msg, MIME_C_ENCR)) != NULL)
 	replace_mime_field(mime1, MIME_C_ENCR, fld->f_line);
   else	{
-	sprintf(buf, "%s: %s\n", MIME_C_ENCR, mbody->encoding->encoding_name);
+	snprintf(buf, sizeof(buf), "%s: %s\n", MIME_C_ENCR, mbody->encoding->encoding_name);
 	replace_mime_field(mime1, MIME_C_ENCR, buf);
 	}
 							}
