@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: rules.c,v 2.6 1997/12/16 16:48:31 gena Exp $
+/* $Id: rules.c,v 2.15 1998/04/07 12:34:50 gena Exp $
  */
 
 #include <fmail.h>
@@ -109,7 +109,7 @@ int regerr;
 int regflags;
 #endif
 
- sprintf(rfname, "%s/.xfmrules", configdir);
+ snprintf(rfname, sizeof(rfname), "%s/.xfmrules", configdir);
 
  for (i = 0; i < MAX_RULES; i++)    {
 	rules[i].action = 0;
@@ -143,7 +143,7 @@ int regflags;
 
 #ifdef HAVE_REGCOMP
 	regfree(&rules[i].rx);
-	regflags = REG_NOSUB | REG_EXTENDED;
+	regflags = REG_EXTENDED;
 	if (rules[i].flags & CASE_I)
 		regflags |= REG_ICASE;
 
@@ -178,7 +178,12 @@ FILE *rfd;
 char rfname[255];
 int i;
 
- sprintf(rfname, "%s/.xfmrules", configdir);
+ if (readonly)    {
+	r_modified = 0;
+	changed = 0;
+	return 0; }
+
+ snprintf(rfname, sizeof(rfname), "%s/.xfmrules", configdir);
 
  if (!(rfd = fopen(rfname, "w"))) {
 	display_msg(MSG_WARN, "Can not save rules database", "%s", rfname);
@@ -375,13 +380,32 @@ int i;
 			continue;
 		if (rules[i].flags & R_LOG)
 			display_msg(MSG_LOG, "rule", "%s matched message %ld in %s folder", rules[i].name, msg->uid, msg->folder ? msg->folder->sname : "incoming");
+		msg->free_text(msg);
 		return &rules[i];
 					}
 				}
 
+ msg->free_text(msg);
  return NULL;
 }
 
+int
+apply_rule_opts(rule, msg)
+struct _xf_rule *rule;
+struct _mail_msg *msg;
+{
+  if (!(msg->status & RECENT))
+	return 0;
+
+  if (rule->flags & R_SAVE_ADDR)
+	add_msg_addr(msg);
+
+  if (rule->flags & R_MARK_READ)
+	msg->flags &= ~UNREAD;
+
+  return (rule->flags & R_SILENT) ? 1 : 0;
+}
+
 int
 apply_rule(msg, delay)
 struct _mail_msg *msg;
@@ -392,6 +416,7 @@ struct _mail_msg *msg1;
 struct _mail_addr *addr;
 struct _head_field *hf;
 struct _xf_rule *rule;
+struct _retrieve_src *source;
 int action, mcount = 0, eres, mnum = 0;
 char *data = INBOX, buf[255], mfile[255], *p;
 struct _proc_info pinfo;
@@ -401,27 +426,15 @@ struct stat sb;
 		if (msg->status & NOTINCOMING)
 			return -1;
 		if (msg->status & MOUTGOING) {
-		    action = -1;
 		    if (((hf = find_field(msg, REPLY_ORGMSG)) != NULL) &&
-			((msg1 = get_msg_by_url(hf->f_line)) != NULL))	{
+			((msg1 = get_msg_by_url(hf->f_line)) != NULL))
 			msg1->flags |= ANSWERED;
-			if (b_getcfg_int(conf_name, "replykeep", 0) ||
-			  (msg1->folder && (msg1->folder->flags & FRKEEP))) {
-			  action = R_MOVE;
-			  data = get_folder_full_name(msg1->folder);	    }
-									}
 		    else
 		    if (((hf = find_field(msg, FWD_ORGMSG)) != NULL) &&
 			((msg1 = get_msg_by_url(hf->f_line)) != NULL))
 			msg1->flags |= FORWARDED;
 
-		    if (action == -1)	{
-			if (b_getcfg_int(conf_name, "smtpsave", 0)) 	{
-				action = R_MOVE;
-				data = sentm->sname;			}
-			else
-				action = R_DISCARD;
-					}
+		    action = R_DISCARD;
 					     }
 		else	{
 		  action = R_MOVE;
@@ -434,8 +447,10 @@ struct stat sb;
 		}
 
 	if (msg->status & RECENT) {
+	 source = get_msg_src(msg);
 	 if (rule) {
-	  if (rule->flags & R_SAVE_ADDR)
+	  if ((rule->flags & R_SAVE_ADDR) ||
+		(source && (source->flags & RSRC_SAVEADDR)))
 		add_msg_addr(msg);
 
 	  if (rule->flags & R_MARK_READ)
@@ -445,7 +460,8 @@ struct stat sb;
 		mcount = 1;
 		  }
 	 else	{
-	  if (b_getcfg_int(conf_name, "saveaddr", 0))
+	  if (b_getcfg_int(conf_name, "saveaddr", 0) ||
+		(source && (source->flags & RSRC_SAVEADDR)))
 		add_msg_addr(msg);
 		}
 				 }
@@ -476,7 +492,7 @@ struct stat sb;
                 break;
 
                 case R_FORWARD:
-                        msg1 = get_fwd_msg(msg);
+                        msg1 = get_fwd_msg(msg, NULL);
                         if (!msg1)
                                 return -1;
 
@@ -495,6 +511,8 @@ struct stat sb;
 				else	{
 				 if ((folder = get_folder_by_name(rule->data)) == NULL)
 					folder = inbox;
+				 if (apply_rule_opts(rule, msg) == 0)
+					mcount = 0;
 					}
 
 		 		msg->status &= ~NOTINCOMING;
@@ -530,6 +548,8 @@ struct stat sb;
 				else	{
 				 if ((folder = get_folder_by_name(rule->data)) == NULL)
 					folder = inbox;
+				 if (apply_rule_opts(rule, msg) == 0)
+					mcount = 0;
 					}
 
 		 		msg->status &= ~NOTINCOMING;
@@ -582,6 +602,8 @@ struct stat sb;
 				else	{
 				 if ((folder = get_folder_by_name(rule->data)) == NULL)
 					folder = inbox;
+				 if (apply_rule_opts(rule, msg) == 0)
+					mcount = 0;
 					}
 
 		 		msg->status &= ~NOTINCOMING;
@@ -617,7 +639,7 @@ struct stat sb;
 		  if ((mnum = get_new_name(ftemp)) == -1)
 			return -1;
 
-		  sprintf(buf, "%s/%d", ftemp->fold_path, mnum);
+		  snprintf(buf, sizeof(buf), "%s/%d", ftemp->fold_path, mnum);
 		  if ((pinfo.ofd = open(buf, O_WRONLY|O_CREAT|O_TRUNC, 00600))
 			<= 0)
 			return -1;
@@ -658,6 +680,8 @@ struct stat sb;
 			else	{
 			 if ((folder = get_folder_by_name(rule->data)) == NULL)
 				folder = inbox;
+			 if (apply_rule_opts(rule, msg) == 0)
+				mcount = 0;
 				}
 
 		 	msg->status &= ~NOTINCOMING;
@@ -821,8 +845,7 @@ void Rule_Up_Call(FL_OBJECT *obj, long param)
 {
 struct _xf_rule rule;
 int i, r1, r2;
-int k = fl_get_browser(rule_obj->Rule_List),
-    t = fl_get_browser_topline(rule_obj->Rule_List);
+int k = fl_get_browser(rule_obj->Rule_List);
 
  if (k < 2)
         return;
@@ -845,12 +868,8 @@ for (i = 0; i < MAX_RULES; i++) {
 
 	fl_add_browser_line(rule_obj->Rule_List, rules[i].name);
 				}
-if (k < t) {
- i = k - fl_get_browser_screenlines(rule_obj->Rule_List) + 1;
- fl_set_browser_topline(rule_obj->Rule_List, i > 0 ? i : 1);
-	   }
-else
- fl_set_browser_topline(rule_obj->Rule_List, t);
+
+fl_show_browser_line(rule_obj->Rule_List, k);
 fl_select_browser_line(rule_obj->Rule_List, k);
 fl_unfreeze_form(rule_obj->Rules_Form);
 changed = 1;
@@ -861,8 +880,7 @@ void Rule_Down_Call(FL_OBJECT *obj, long param)
 {
 struct _xf_rule rule;
 int i, r1, r2;
-int k = fl_get_browser(rule_obj->Rule_List),
-    t = fl_get_browser_topline(rule_obj->Rule_List);
+int k = fl_get_browser(rule_obj->Rule_List);
 
  if (k == fl_get_browser_maxline(rule_obj->Rule_List))
         return;
@@ -886,11 +904,7 @@ for (i = 0; i < MAX_RULES; i++) {
 	fl_add_browser_line(rule_obj->Rule_List, rules[i].name);
 				}
 
-if (k >= (t + fl_get_browser_screenlines(rule_obj->Rule_List)))
- fl_set_browser_topline(rule_obj->Rule_List, k);
-else
- fl_set_browser_topline(rule_obj->Rule_List, t);
-
+fl_show_browser_line(rule_obj->Rule_List, k);
 fl_select_browser_line(rule_obj->Rule_List, k);
 fl_unfreeze_form(rule_obj->Rules_Form);
 changed = 1;
@@ -947,11 +961,11 @@ int i, k, l;
 
 		  fl_add_browser_line(rule_obj->Rule_List, rules[k].name);
 		  l++;
-		  if (k == i) {
+		  if (k == i)	{
 			fl_select_browser_line(rule_obj->Rule_List, l);
-			fl_set_browser_topline(rule_obj->Rule_List, l); 
-			}
+			fl_show_browser_line(rule_obj->Rule_List, l); 
 				}
+						}
 
 		fl_unfreeze_form(rule_obj->Rules_Form);
  		changed = 1;
@@ -1067,11 +1081,12 @@ void Rule_Load_Call(FL_OBJECT *obj, long param)
 {
 int i;
 
- if (changed) {
+ if (changed && !readonly)	{
 	if (display_msg(MSG_QUEST, "Do you want to save changes?", "")) {
 	 changed = 0;
-         save_rules(); }
-	}
+         save_rules();							}
+				}
+ changed = 0;
 
  load_rules();
  fl_freeze_form(rule_obj->Rules_Form);
@@ -1221,7 +1236,7 @@ int regflags;
 #ifdef HAVE_REGCOMP
         regfree(&rules[num].rx);
 
-	regflags = REG_NOSUB | REG_EXTENDED;
+	regflags = REG_EXTENDED;
 	if (rules[i].flags & CASE_I)
 		regflags |= REG_ICASE;
 
@@ -1561,11 +1576,12 @@ int i;
  fl_show_form(rule_obj->Rules_Form, FL_PLACE_CENTER, FL_TRANSIENT, "Rules");
  fl_do_only_forms();
 
- if (changed) {
+ if (changed && !readonly)	{
 	if (display_msg(MSG_QUEST, "Do you want to save changes?", "")) {
 	 changed = 0;
-         save_rules(); }
-	}
+         save_rules();							}
+				}
+ changed = 0;
 
  fl_hide_form(rule_obj->Rules_Form);
  fl_free_form(rule_obj->Rules_Form);
