@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: sign.c,v 2.5 1997/12/17 14:56:56 gena Exp $
+/* $Id: sign.c,v 2.9 1998/04/07 12:34:51 gena Exp $
  */
 
 #include <fmail.h>
@@ -82,7 +82,7 @@ struct _xf_sign *sign;
 int i;
 
  if (ed_sign)   {
-  sprintf(buf, "%s/%s", homedir, ed_sign->file);
+  snprintf(buf, sizeof(buf), "%s/%s", homedir, ed_sign->file);
   fl_save_textedit(sign_obj->Sign_Enter, buf);
 
   strncpy(ed_sign->rfield, fl_get_input(sign_obj->Sign_Field), MAX_FIELD_NAME_LEN - 1);
@@ -102,7 +102,7 @@ int i;
  while (sign) 	{
   i++;
   sprintf(stmp, "sign%d", i);
-  sprintf(buf, "%s %s %s %s", sign->name, sign->file, sign->rfield, sign->rule);
+  snprintf(buf, sizeof(buf), "%s %s %s %s", sign->name, sign->file, sign->rfield, sign->rule);
   b_putcfg_str(conf_name, stmp, buf);
 
   sign = sign->next;
@@ -149,7 +149,7 @@ FILE *sf;
 	continue;
 								}
 
-  sprintf(buf, "%s/%s", homedir, sign->file);
+  snprintf(buf, sizeof(buf), "%s/%s", homedir, sign->file);
   if (access(buf, R_OK) == -1)  {
 	display_msg(MSG_WARN, "load sign", "%s does not exits, skipping", buf);
 	free(sign);
@@ -170,9 +170,9 @@ FILE *sf;
    sign->next = NULL;
    strcpy(sign->name, "default");
    strcpy(sign->file, ".signature");
-   sprintf(buf, "%s/.signature", homedir);
+   snprintf(buf, sizeof(buf), "%s/.signature", homedir);
    if (access(buf, R_OK) == -1) {
-     sprintf(buf, "%s/.signature.xfmail", homedir);
+     snprintf(buf, sizeof(buf), "%s/.signature.xfmail", homedir);
      strcpy(sign->file, ".signature.xfmail");
      if (access(buf, R_OK) == -1) {
 	if ((sf = fopen(buf, "w")) == NULL) {
@@ -186,7 +186,7 @@ FILE *sf;
 				}
 
    signatures = sign;
-   sprintf(buf, "%s %s %s %s", sign->name, sign->file, sign->rfield,sign->rule);
+   snprintf(buf, sizeof(buf), "%s %s %s %s", sign->name, sign->file, sign->rfield,sign->rule);
    b_putcfg_str(conf_name, "sign1", buf);
 			  }
 
@@ -205,7 +205,7 @@ int sl, i;
 
  sign_changed();
 
- sprintf(buf, "%s/%s", homedir, ed_sign->file);
+ snprintf(buf, sizeof(buf), "%s/%s", homedir, ed_sign->file);
  fl_load_textedit(sign_obj->Sign_Enter, buf);
  fl_set_input(sign_obj->Sign_Field, ed_sign->rfield);
  fl_set_input(sign_obj->Sign_Rule, ed_sign->rule);
@@ -275,7 +275,7 @@ FILE *sf;
   sign = sign->next;
 		}
 
- sprintf(stmp, ".signature.%s", sname);
+ snprintf(stmp, sizeof(stmp), ".signature.%s", sname);
 
  p = (char *)fl_show_file_selector("Signature file",homedir,".signature*",stmp);
 
@@ -346,7 +346,7 @@ int i;
   if (!display_msg(MSG_QUEST|MSG_DEFNO, "Delete signature", "Are you sure want to delete signature %s?", ed_sign->name))
 	return;
 
-  sprintf(buf, "%s/%s", homedir, ed_sign->file);
+  snprintf(buf, sizeof(buf), "%s/%s", homedir, ed_sign->file);
   if (access(buf, R_OK) == 0)   {
    if (display_msg(MSG_QUEST|MSG_DEFNO, "Delete signature", "Remove %s ?", ed_sign->file))
 	unlink(buf);
@@ -673,13 +673,13 @@ int regerr;
 
    strcpy(rule.fmatch, sign->rfield);
 #ifdef HAVE_REGCOMP
-   if ((regerr = regcomp(&rx, sign->rule, REG_NOSUB|REG_EXTENDED|REG_ICASE))) {
+   if ((regerr = regcomp(&rx, sign->rule, REG_EXTENDED|REG_ICASE)))	{
 	regerror(regerr, &rx, errbuf, sizeof(errbuf));
         display_msg(MSG_WARN, "Invalid regular expression", "%s", errbuf);
 	regfree(&rx);
    	sign = sign->next;
 	continue;
-									      }
+								      	}
    rule.rx = rx;
 #else
    strncpy(rule.tmatch, sign->rule, 254);
@@ -696,10 +696,11 @@ int regerr;
    sign = sign->next;
 		}
 
+   msg->free_text(msg);
    if (!dsign)
 	return NULL;
 
-   sprintf(sfile, "%s/%s", homedir, dsign->file);
+   snprintf(sfile, sizeof(sfile), "%s/%s", homedir, dsign->file);
 
    return sfile;
 }
@@ -741,7 +742,7 @@ struct _xf_sign *sign;
  if (!sign)
 	return NULL;
 
- sprintf(sfile, "%s/%s", homedir, sign->file);
+ snprintf(sfile, sizeof(sfile), "%s/%s", homedir, sign->file);
 
  return sfile;
 }
