@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1995,1996 by Gennady B. Sorokopud (gena@NetVision.net.il)
+ * Copyright(c) 1995-1998 by Gennady B. Sorokopud (gena@NetVision.net.il)
  *
  * This software can be freely redistributed and modified for 
  * non-commercial purposes as long as above copyright
@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: sort.c,v 2.3 1997/10/30 16:27:24 gena Exp $
+/* $Id: sort.c,v 2.6 1998/03/12 14:41:34 gena Exp $
  */
 
 #include <fmail.h>
@@ -320,7 +320,9 @@ struct _mail_folder *fld;
 {
  fld->status &= ~FALIAS;
  if (fld->sname)
-	free(fld->sname);
+	return;
+ free(fld->sname);
+
  switch (fld->type) {
   default:
   case F_MH:
@@ -329,7 +331,7 @@ struct _mail_folder *fld;
   break;
 
   case F_IMAP:
-   fld->sname = strdup(get_imap_folder_short_name(fld));
+   fld->sname = strdup(get_imap_folder_short_name(NULL, fld));
   break;
 		    }
  return;
@@ -421,6 +423,10 @@ struct _mail_folder *fld = (struct _mail_folder *)obj->u_vdata;
  fld->color = -1;
  fld->sort = -1;
 
+ if (fld->sname)
+	free(fld->sname);
+ fld->sname = NULL;
+
  set_fld_sname(fld);
  set_fld_values(fld);
 }
@@ -468,7 +474,7 @@ char ftitle[127], *p;
  fl_set_button(fold_obj->Folder_RKeep, (fld->flags & FRKEEP) ? 1 : 0);
  fl_set_button(fold_obj->Folder_DValidate, (fld->flags & FNVALD) ? 1 : 0);
 
- sprintf(ftitle, "Edit options for '%s'", fld->sname);
+ snprintf(ftitle, sizeof(ftitle), "Edit options for '%s'", fld->sname);
  fl_set_object_label(fold_obj->Folder_Text, ftitle);
 
  fl_show_form(fold_obj->Folder, FL_PLACE_CENTER, FL_TRANSIENT, ftitle);
@@ -616,7 +622,7 @@ char ftitle[64];
  fl_set_button(msg_obj->Message_Forwarded, (msg->flags & FORWARDED));
  fl_set_button(msg_obj->Message_Expire, (msg->flags & M_DONTEXPIRE));
 
- sprintf(ftitle, "%-.16s/%-.16s", msg->folder ? msg->folder->sname : "nofolder", msg->header->Subject ? msg->header->Subject : "nosubject");
+ snprintf(ftitle, sizeof(ftitle), "%-.16s/%-.16s", msg->folder ? msg->folder->sname : "nofolder", msg->header->Subject ? msg->header->Subject : "nosubject");
  fl_show_form(msg_obj->Message, FL_PLACE_CENTER, FL_TRANSIENT, ftitle);
  fl_do_only_forms();
 
